@@ -1,16 +1,24 @@
 /*
- * Geeqie
- * (C) 2004 John Ellis
- * Copyright (C) 2008 - 2012 The Geeqie Team
+ * Copyright (C) 2004 John Ellis
+ * Copyright (C) 2008 - 2016 The Geeqie Team
  *
  * Author: John Ellis
  *
- * This software is released under the GNU General Public License (GNU GPL).
- * Please read the included file COPYING for more information.
- * This software comes with no warranty of any kind, use at your own risk!
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-
 #include "main.h"
 #include "advanced_exif.h"
 
@@ -59,6 +67,15 @@ enum {
 	EXIF_ADVCOL_COUNT
 };
 
+gint display_order [6] = {
+	EXIF_ADVCOL_DESCRIPTION,
+	EXIF_ADVCOL_VALUE,
+	EXIF_ADVCOL_NAME,
+	EXIF_ADVCOL_TAG,
+	EXIF_ADVCOL_FORMAT,
+	EXIF_ADVCOL_ELEMENTS
+};
+
 static gboolean advanced_exif_row_enabled(const gchar *name)
 {
 	GList *list;
@@ -85,11 +102,11 @@ static void advanced_exif_update(ExifWin *ew)
 	ExifItem *item;
 
 	exif = exif_read_fd(ew->fd);
-	
+
 	gtk_widget_set_sensitive(ew->scrolled, !!exif);
 
 	if (!exif) return;
-	
+
 	exif_original = exif_get_original(exif);
 
 	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(ew->listview)));
@@ -114,7 +131,7 @@ static void advanced_exif_update(ExifWin *ew)
 		g_free(text);
 		elements = g_strdup_printf("%d", exif_item_get_elements(item));
 		description = exif_item_get_description(item);
-		if (!description || *description == '\0') 
+		if (!description || *description == '\0')
 			{
 			g_free(description);
 			description = g_strdup(tag_name);
@@ -176,10 +193,10 @@ static void advanced_exif_dnd_get(GtkWidget *listview, GdkDragContext *context,
 				  guint time, gpointer data)
 {
 	//ExifWin *ew = data;
-	GtkTreeSelection *sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(listview)); 
+	GtkTreeSelection *sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(listview));
 	GtkTreeIter iter;
 
-	if (gtk_tree_selection_get_selected(sel, NULL, &iter)) 
+	if (gtk_tree_selection_get_selected(sel, NULL, &iter))
 		{
 		GtkTreeModel *store = gtk_tree_view_get_model(GTK_TREE_VIEW(listview));
 		gchar *key;
@@ -196,10 +213,10 @@ static void advanced_exif_dnd_get(GtkWidget *listview, GdkDragContext *context,
 static void advanced_exif_dnd_begin(GtkWidget *listview, GdkDragContext *context, gpointer data)
 {
 	//ExifWin *ew = data;
-	GtkTreeSelection *sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(listview)); 
+	GtkTreeSelection *sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(listview));
 	GtkTreeIter iter;
 
-	if (gtk_tree_selection_get_selected(sel, NULL, &iter)) 
+	if (gtk_tree_selection_get_selected(sel, NULL, &iter))
 		{
 		GtkTreeModel *store = gtk_tree_view_get_model(GTK_TREE_VIEW(listview));
 		gchar *key;
@@ -230,7 +247,7 @@ static void advanced_exif_add_column(GtkWidget *listview, const gchar *title, gi
 		{
 		gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 		}
-	
+
 	gtk_tree_view_column_set_resizable(column, TRUE);
 	gtk_tree_view_column_set_sort_column_id(column, n);
 
@@ -295,6 +312,39 @@ static gint advanced_exif_sort_cb(GtkTreeModel *model, GtkTreeIter *a, GtkTreeIt
 	return ret;
 }
 
+static gboolean advanced_exif_mouseclick(GtkWidget *widget,
+						GdkEventButton *bevent, gpointer data)
+{
+	ExifWin *ew = data;
+	GtkTreePath *path;
+	GtkTreeViewColumn *column;
+	GtkTreeIter iter;
+	GtkTreeModel *store;
+	gchar *value;
+	GList *cols;
+	gint col_num;
+	GtkClipboard *clipboard;
+
+	gtk_tree_view_get_cursor(GTK_TREE_VIEW(ew->listview), &path, &column);
+	if (path && column)
+		{
+		store = gtk_tree_view_get_model(GTK_TREE_VIEW(ew->listview));
+		gtk_tree_model_get_iter(store, &iter, path);
+
+		cols = gtk_tree_view_get_columns(GTK_TREE_VIEW(ew->listview));
+		col_num = g_list_index(cols, (gpointer)column);
+		gtk_tree_model_get(store, &iter, display_order[col_num], &value, -1);
+
+		clipboard = gtk_clipboard_get(GDK_SELECTION_PRIMARY);
+		gtk_clipboard_set_text(clipboard, value, -1);
+
+		g_list_free(cols);
+		g_free(value);
+		}
+
+	return TRUE;
+}
+
 static gboolean advanced_exif_keypress(GtkWidget *widget, GdkEventKey *event, gpointer data)
 {
 	ExifWin *ew = data;
@@ -379,7 +429,7 @@ GtkWidget *advanced_exif_new(void)
 	advanced_exif_add_column(ew->listview, _("Tag"), EXIF_ADVCOL_TAG, FALSE);
 	advanced_exif_add_column(ew->listview, _("Format"), EXIF_ADVCOL_FORMAT, FALSE);
 	advanced_exif_add_column(ew->listview, _("Elements"), EXIF_ADVCOL_ELEMENTS, FALSE);
-	
+
 
 	gtk_drag_source_set(ew->listview,
 			   GDK_BUTTON1_MASK | GDK_BUTTON2_MASK,
@@ -395,6 +445,9 @@ GtkWidget *advanced_exif_new(void)
 	g_signal_connect(G_OBJECT(ew->window), "key_press_event",
 			 G_CALLBACK(advanced_exif_keypress), ew);
 
+	g_signal_connect(G_OBJECT(ew->listview), "button_release_event",
+			G_CALLBACK(advanced_exif_mouseclick), ew);
+
 	ew->scrolled = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(ew->scrolled), GTK_SHADOW_IN);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(ew->scrolled),
