@@ -1,16 +1,24 @@
 /*
- * Geeqie
- * (C) 2004 John Ellis
- * Copyright (C) 2008 - 2012 The Geeqie Team
+ * Copyright (C) 2004 John Ellis
+ * Copyright (C) 2008 - 2016 The Geeqie Team
  *
  * Author: Vladimir Nadvornik
  *
- * This software is released under the GNU General Public License (GNU GPL).
- * Please read the included file COPYING for more information.
- * This software comes with no warranty of any kind, use at your own risk!
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-
 #include "main.h"
 #include "bar.h"
 
@@ -40,7 +48,7 @@ struct _KnownPanes
 	const gchar *config;
 };
 
-static const gchar default_config_histogram[] = 
+static const gchar default_config_histogram[] =
 "<gq>"
 "    <layout id = '_current_'>"
 "        <bar>"
@@ -49,7 +57,7 @@ static const gchar default_config_histogram[] =
 "    </layout>"
 "</gq>";
 
-static const gchar default_config_title[] = 
+static const gchar default_config_title[] =
 "<gq>"
 "    <layout id = '_current_'>"
 "        <bar>"
@@ -58,7 +66,7 @@ static const gchar default_config_title[] =
 "    </layout>"
 "</gq>";
 
-static const gchar default_config_keywords[] = 
+static const gchar default_config_keywords[] =
 "<gq>"
 "    <layout id = '_current_'>"
 "        <bar>"
@@ -67,7 +75,7 @@ static const gchar default_config_keywords[] =
 "    </layout>"
 "</gq>";
 
-static const gchar default_config_comment[] = 
+static const gchar default_config_comment[] =
 "<gq>"
 "    <layout id = '_current_'>"
 "        <bar>"
@@ -75,8 +83,16 @@ static const gchar default_config_comment[] =
 "        </bar>"
 "    </layout>"
 "</gq>";
+static const gchar default_config_rating[] =
+"<gq>"
+"    <layout id = '_current_'>"
+"        <bar>"
+"            <pane_comment id = 'rating' expanded = 'true' key = '" RATING_KEY "' height = '10' />"
+"        </bar>"
+"    </layout>"
+"</gq>";
 
-static const gchar default_config_exif[] = 
+static const gchar default_config_exif[] =
 "<gq>"
 "    <layout id = '_current_'>"
 "        <bar>"
@@ -102,7 +118,7 @@ static const gchar default_config_exif[] =
 "    </layout>"
 "</gq>";
 
-static const gchar default_config_file_info[] = 
+static const gchar default_config_file_info[] =
 "<gq>"
 "    <layout id = '_current_'>"
 "        <bar>"
@@ -115,7 +131,7 @@ static const gchar default_config_file_info[] =
 "    </layout>"
 "</gq>";
 
-static const gchar default_config_location[] = 
+static const gchar default_config_location[] =
 "<gq>"
 "    <layout id = '_current_'>"
 "        <bar>"
@@ -132,7 +148,7 @@ static const gchar default_config_location[] =
 "    </layout>"
 "</gq>";
 
-static const gchar default_config_copyright[] = 
+static const gchar default_config_copyright[] =
 "<gq>"
 "    <layout id = '_current_'>"
 "        <bar>"
@@ -147,7 +163,7 @@ static const gchar default_config_copyright[] =
 
 #ifdef HAVE_LIBCHAMPLAIN
 #ifdef HAVE_LIBCHAMPLAIN_GTK
-static const gchar default_config_gps[] = 
+static const gchar default_config_gps[] =
 "<gq>"
 "    <layout id = '_current_'>"
 "        <bar>"
@@ -168,6 +184,7 @@ static const KnownPanes known_panes[] = {
 	{PANE_COMMENT,		"title",	N_("Title"),		default_config_title},
 	{PANE_KEYWORDS,		"keywords",	N_("Keywords"),		default_config_keywords},
 	{PANE_COMMENT,		"comment",	N_("Comment"),		default_config_comment},
+	{PANE_COMMENT,		"rating",	N_("Rating"),		default_config_rating},
 	{PANE_EXIF,		"exif",		N_("Exif"),		default_config_exif},
 /* other pre-configured panes */
 	{PANE_EXIF,		"file_info",	N_("File info"),	default_config_file_info},
@@ -188,6 +205,7 @@ struct _BarData
 	GtkWidget *vbox;
 	FileData *fd;
 	GtkWidget *label_file_name;
+	GtkWidget *add_button;
 
 	LayoutWindow *lw;
 	gint width;
@@ -202,9 +220,9 @@ static void bar_expander_move(GtkWidget *widget, gpointer data, gboolean up, gbo
 	if (!expander) return;
 	box = gtk_widget_get_ancestor(expander, GTK_TYPE_BOX);
 	if (!box) return;
-	
+
 	gtk_container_child_get(GTK_CONTAINER(box), expander, "position", &pos, NULL);
-	
+
 	if (single_step)
 		{
 		pos = up ? (pos - 1) : (pos + 1);
@@ -214,7 +232,7 @@ static void bar_expander_move(GtkWidget *widget, gpointer data, gboolean up, gbo
 		{
 		pos = up ? 0 : -1;
 		}
-	
+
 	gtk_box_reorder_child(GTK_BOX(box), expander, pos);
 }
 
@@ -253,14 +271,14 @@ static void bar_expander_add_cb(GtkWidget *widget, gpointer data)
 	const gchar *config;
 
 	if (!id) return;
-	
+
 	while (pane->id)
 		{
 		if (strcmp(pane->id, id) == 0) break;
 		pane++;
 		}
 	if (!pane->id) return;
-	
+
 	config = bar_pane_get_default_config(id);
 	if (config) load_config_from_buf(config, strlen(config), FALSE);
 
@@ -276,20 +294,20 @@ static void bar_menu_popup(GtkWidget *widget)
 	BarData *bd;
 
 	bd = g_object_get_data(G_OBJECT(widget), "bar_data");
-	if (bd) 
+	if (bd)
 		{
 		expander = NULL;
-		bar = widget; 
+		bar = widget;
 		}
 	else
 		{
 		expander = widget;
-		bar = widget->parent;
+		bar = gtk_widget_get_parent(widget);
 		while (bar && !g_object_get_data(G_OBJECT(bar), "bar_data"))
-			bar = bar->parent;
+			bar = gtk_widget_get_parent(bar);
 		if (!bar) return;
 		}
- 
+
 	menu = popup_menu_short_lived();
 
 	if (expander)
@@ -310,20 +328,65 @@ static void bar_menu_popup(GtkWidget *widget)
 		g_object_set_data(G_OBJECT(item), "pane_add_id", pane->id);
 		pane++;
 		}
-	
+
+	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, bar, 0, GDK_CURRENT_TIME);
+}
+
+static void bar_menu_add_popup(GtkWidget *widget)
+{
+	GtkWidget *menu;
+	GtkWidget *bar;
+	const KnownPanes *pane = known_panes;
+
+	bar = widget;
+
+	menu = popup_menu_short_lived();
+
+	while (pane->id)
+		{
+		GtkWidget *item;
+		item = menu_item_add_stock(menu, _(pane->title), GTK_STOCK_ADD, G_CALLBACK(bar_expander_add_cb), bar);
+		g_object_set_data(G_OBJECT(item), "pane_add_id", pane->id);
+		pane++;
+		}
+
 	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, bar, 0, GDK_CURRENT_TIME);
 }
 
 
-static gboolean bar_menu_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data) 
-{ 
+static gboolean bar_menu_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
+{
 	if (bevent->button == MOUSE_BUTTON_RIGHT)
 		{
 		bar_menu_popup(widget);
 		return TRUE;
 		}
 	return FALSE;
-} 
+}
+
+static void bar_expander_cb(GObject *object, GParamSpec *param_spec, gpointer data)
+{
+	GtkExpander *expander;
+	GtkWidget *child;
+
+	expander = GTK_EXPANDER(object);
+	child = gtk_bin_get_child(GTK_BIN(expander));
+
+	if (gtk_expander_get_expanded(expander))
+		{
+		gtk_widget_show_all(child);
+		}
+	else
+		{
+		gtk_widget_hide(child);
+		}
+}
+
+static gboolean bar_menu_add_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
+{
+	bar_menu_add_popup(widget);
+	return TRUE;
+}
 
 
 static void bar_pane_set_fd_cb(GtkWidget *expander, gpointer data)
@@ -344,7 +407,7 @@ void bar_set_fd(GtkWidget *bar, FileData *fd)
 	bd->fd = file_data_ref(fd);
 
 	gtk_container_foreach(GTK_CONTAINER(bd->vbox), bar_pane_set_fd_cb, fd);
-	
+
 	gtk_label_set_text(GTK_LABEL(bd->label_file_name), (bd->fd) ? bd->fd->name : "");
 
 }
@@ -371,19 +434,19 @@ gboolean bar_event(GtkWidget *bar, GdkEvent *event)
 	BarData *bd;
 	GList *list, *work;
 	gboolean ret = FALSE;
-	
+
 	bd = g_object_get_data(G_OBJECT(bar), "bar_data");
 	if (!bd) return FALSE;
 
 	list = gtk_container_get_children(GTK_CONTAINER(bd->vbox));
-	
+
 	work = list;
 	while (work)
 		{
 		GtkWidget *widget = gtk_bin_get_child(GTK_BIN(work->data));
 		PaneData *pd = g_object_get_data(G_OBJECT(widget), "pane_data");
 		if (!pd) continue;
-	
+
 		if (pd->pane_event && pd->pane_event(widget, event))
 			{
 			ret = TRUE;
@@ -400,21 +463,21 @@ GtkWidget *bar_find_pane_by_id(GtkWidget *bar, PaneType type, const gchar *id)
 	BarData *bd;
 	GList *list, *work;
 	GtkWidget *ret = NULL;
-	
+
 	if (!id || !id[0]) return NULL;
-	
+
 	bd = g_object_get_data(G_OBJECT(bar), "bar_data");
 	if (!bd) return NULL;
 
 	list = gtk_container_get_children(GTK_CONTAINER(bd->vbox));
-	
+
 	work = list;
 	while (work)
 		{
 		GtkWidget *widget = gtk_bin_get_child(GTK_BIN(work->data));
 		PaneData *pd = g_object_get_data(G_OBJECT(widget), "pane_data");
 		if (!pd) continue;
-	
+
 		if (type == pd->type && strcmp(id, pd->id) == 0)
 			{
 			ret = widget;
@@ -430,12 +493,12 @@ void bar_clear(GtkWidget *bar)
 {
 	BarData *bd;
 	GList *list, *work;
-	
+
 	bd = g_object_get_data(G_OBJECT(bar), "bar_data");
 	if (!bd) return;
 
 	list = gtk_container_get_children(GTK_CONTAINER(bd->vbox));
-	
+
 	work = list;
 	while (work)
 		{
@@ -452,23 +515,19 @@ void bar_write_config(GtkWidget *bar, GString *outstr, gint indent)
 	GList *list, *work;
 
 	if (!bar) return;
-	
+
 	bd = g_object_get_data(G_OBJECT(bar), "bar_data");
 	if (!bd) return;
 
 	WRITE_NL(); WRITE_STRING("<bar ");
-#if GTK_CHECK_VERSION(2,20,0)
 	write_bool_option(outstr, indent, "enabled", gtk_widget_get_visible(bar));
-#else
-	write_bool_option(outstr, indent, "enabled", GTK_WIDGET_VISIBLE(bar));
-#endif
 	write_uint_option(outstr, indent, "width", bd->width);
 	WRITE_STRING(">");
-	
+
 	indent++;
 	WRITE_NL(); WRITE_STRING("<clear/>");
 
-	list = gtk_container_get_children(GTK_CONTAINER(bd->vbox));	
+	list = gtk_container_get_children(GTK_CONTAINER(bd->vbox));
 	work = list;
 	while (work)
 		{
@@ -493,11 +552,11 @@ void bar_update_expander(GtkWidget *pane)
 {
 	PaneData *pd = g_object_get_data(G_OBJECT(pane), "pane_data");
 	GtkWidget *expander;
-	
+
 	if (!pd) return;
 
-	expander = pane->parent;
-	
+	expander = gtk_widget_get_parent(pane);
+
 	gtk_expander_set_expanded(GTK_EXPANDER(expander), pd->expanded);
 }
 
@@ -506,25 +565,26 @@ void bar_add(GtkWidget *bar, GtkWidget *pane)
 	GtkWidget *expander;
 	BarData *bd = g_object_get_data(G_OBJECT(bar), "bar_data");
 	PaneData *pd = g_object_get_data(G_OBJECT(pane), "pane_data");
-	
+
 	if (!bd) return;
 
 	pd->lw = bd->lw;
 	pd->bar = bar;
-	
+
 	expander = gtk_expander_new(NULL);
 	if (pd && pd->title)
 		{
 		gtk_expander_set_label_widget(GTK_EXPANDER(expander), pd->title);
 		gtk_widget_show(pd->title);
 		}
-		
+
 	gtk_box_pack_start(GTK_BOX(bd->vbox), expander, FALSE, TRUE, 0);
-	
-	g_signal_connect(expander, "button_release_event", G_CALLBACK(bar_menu_cb), bd); 
-	
+
+	g_signal_connect(expander, "button_release_event", G_CALLBACK(bar_menu_cb), bd);
+	g_signal_connect(expander, "notify::expanded", G_CALLBACK(bar_expander_cb), pd);
+
 	gtk_container_add(GTK_CONTAINER(expander), pane);
-	
+
 	gtk_expander_set_expanded(GTK_EXPANDER(expander), pd->expanded);
 
 	gtk_widget_show(expander);
@@ -535,9 +595,9 @@ void bar_add(GtkWidget *bar, GtkWidget *pane)
 
 void bar_populate_default(GtkWidget *bar)
 {
-	const gchar *populate_id[] = {"histogram", "title", "keywords", "comment", "exif", NULL};
+	const gchar *populate_id[] = {"histogram", "title", "keywords", "comment", "rating", "exif", NULL};
 	const gchar **id = populate_id;
-	
+
 	while (*id)
 		{
 		const gchar *config = bar_pane_get_default_config(*id);
@@ -549,14 +609,14 @@ void bar_populate_default(GtkWidget *bar)
 static void bar_size_allocate(GtkWidget *widget, GtkAllocation *allocation, gpointer data)
 {
 	BarData *bd = data;
-	
+
 	bd->width = allocation->width;
 }
 
 gint bar_get_width(GtkWidget *bar)
 {
 	BarData *bd;
-	
+
 	bd = g_object_get_data(G_OBJECT(bar), "bar_data");
 	if (!bd) return 0;
 
@@ -582,7 +642,7 @@ static void bar_destroy(GtkWidget *widget, gpointer data)
 }
 
 #ifdef HAVE_LIBCHAMPLAIN_GTK
-/* 
+/*
    FIXME: this is an ugly hack that works around this bug:
    https://bugzilla.gnome.org/show_bug.cgi?id=590692
    http://bugzilla.openedhand.com/show_bug.cgi?id=1751
@@ -601,11 +661,13 @@ GtkWidget *bar_new(LayoutWindow *lw)
 	BarData *bd;
 	GtkWidget *box;
 	GtkWidget *scrolled;
+	GtkWidget *tbar;
+	GtkWidget *add_box;
 
 	bd = g_new0(BarData, 1);
 
 	bd->lw = lw;
-	
+
 	bd->widget = gtk_vbox_new(FALSE, PREF_PAD_GAP);
 	g_object_set_data(G_OBJECT(bd->widget), "bar_data", bd);
 	g_signal_connect(G_OBJECT(bd->widget), "destroy",
@@ -614,7 +676,7 @@ GtkWidget *bar_new(LayoutWindow *lw)
 	g_signal_connect(G_OBJECT(bd->widget), "size-allocate",
 			 G_CALLBACK(bar_size_allocate), bd);
 
-	g_signal_connect(G_OBJECT(bd->widget), "button_release_event", G_CALLBACK(bar_menu_cb), bd); 
+	g_signal_connect(G_OBJECT(bd->widget), "button_release_event", G_CALLBACK(bar_menu_cb), bd);
 
 	bd->width = SIDEBAR_DEFAULT_WIDTH;
 	gtk_widget_set_size_request(bd->widget, bd->width, -1);
@@ -642,8 +704,16 @@ GtkWidget *bar_new(LayoutWindow *lw)
 	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled), bd->vbox);
 	gtk_viewport_set_shadow_type(GTK_VIEWPORT(gtk_bin_get_child(GTK_BIN(scrolled))), GTK_SHADOW_NONE);
 
+	add_box = gtk_vbox_new(FALSE, 0);
+	gtk_box_pack_end(GTK_BOX(bd->widget), add_box, FALSE, FALSE, 0);
+	tbar = pref_toolbar_new(add_box, GTK_TOOLBAR_ICONS);
+	bd->add_button = pref_toolbar_button(tbar, GTK_STOCK_ADD, NULL, FALSE,
+					     _("Add Pane"),
+					     G_CALLBACK(bar_menu_add_cb), bd);
+	gtk_widget_show(add_box);
+
 #ifdef HAVE_LIBCHAMPLAIN_GTK
-	g_signal_connect(G_OBJECT(gtk_bin_get_child(GTK_BIN(scrolled))), "unrealize", G_CALLBACK(bar_unrealize_clutter_fix_cb), NULL); 
+	g_signal_connect(G_OBJECT(gtk_bin_get_child(GTK_BIN(scrolled))), "unrealize", G_CALLBACK(bar_unrealize_clutter_fix_cb), NULL);
 #endif
 
 	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolled), GTK_SHADOW_NONE);
@@ -664,13 +734,22 @@ GtkWidget *bar_update_from_config(GtkWidget *bar, const gchar **attribute_names,
 
 		if (READ_BOOL_FULL("enabled", enabled)) continue;
 		if (READ_INT_FULL("width", width)) continue;
-		
+
 
 		log_printf("unknown attribute %s = %s\n", option, value);
 		}
-	
+
+#if !GTK_CHECK_VERSION(3,0,0)
+/* FIXME: In bar_size_allocate() the width obained is the allocated width. In GTK2 this
+ * is the actual width. In GTK3 it is the *minimum* width.
+ * This results in the info sidebar being able to increase, but not
+ * decrease. There does not seem to be a way in GTK3 to get the actual width of
+ * a widget. For GTK3 the only way is to disable it. The width of the sidebar
+ * is therefore not preserved across restarts.
+ */
 	gtk_widget_set_size_request(bar, width, -1);
-	if (enabled) 
+#endif
+	if (enabled)
 		{
 		gtk_widget_show(bar);
 		}
@@ -700,7 +779,7 @@ GtkWidget *bar_pane_expander_title(const gchar *title)
 gboolean bar_pane_translate_title(PaneType type, const gchar *id, gchar **title)
 {
 	const KnownPanes *pane = known_panes;
-	
+
 	if (!title) return FALSE;
 	while (pane->id)
 		{
@@ -708,9 +787,9 @@ gboolean bar_pane_translate_title(PaneType type, const gchar *id, gchar **title)
 		pane++;
 		}
 	if (!pane->id) return FALSE;
-	
+
 	if (*title && **title && strcmp(pane->title, *title) != 0) return FALSE;
-	
+
 	g_free(*title);
 	*title = g_strdup(_(pane->title));
 	return TRUE;
@@ -719,7 +798,7 @@ gboolean bar_pane_translate_title(PaneType type, const gchar *id, gchar **title)
 const gchar *bar_pane_get_default_config(const gchar *id)
 {
 	const KnownPanes *pane = known_panes;
-	
+
 	while (pane->id)
 		{
 		if (strcmp(pane->id, id) == 0) break;
@@ -728,5 +807,5 @@ const gchar *bar_pane_get_default_config(const gchar *id)
 	if (!pane->id) return NULL;
 	return pane->config;
 }
-	
+
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
