@@ -1,16 +1,24 @@
 /*
- * Geeqie
- * (C) 2004 John Ellis
- * Copyright (C) 2008 - 2012 The Geeqie Team
+ * Copyright (C) 2004 John Ellis
+ * Copyright (C) 2008 - 2016 The Geeqie Team
  *
  * Author: Vladimir Nadvornik
  *
- * This software is released under the GNU General Public License (GNU GPL).
- * Please read the included file COPYING for more information.
- * This software comes with no warranty of any kind, use at your own risk!
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-
 #include "main.h"
 #include "bar_exif.h"
 
@@ -54,8 +62,8 @@ struct _ExifEntry
 
 	PaneExifData *ped;
 };
-	
-	
+
+
 struct _PaneExifData
 {
 	PaneData pane;
@@ -64,10 +72,10 @@ struct _PaneExifData
 	GtkSizeGroup *size_group;
 
 	gint min_height;
-	
+
 	gboolean all_hidden;
 	gboolean show_all;
-	
+
 	FileData *fd;
 };
 
@@ -89,7 +97,7 @@ static void bar_pane_exif_entry_update_title(ExifEntry *ee);
 static void bar_pane_exif_update(PaneExifData *ped);
 static gboolean bar_pane_exif_menu_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data);
 static void bar_pane_exif_notify_cb(FileData *fd, NotifyType type, gpointer data);
-
+static gboolean bar_pane_exif_copy_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data);
 
 static void bar_pane_exif_entry_changed(GtkEntry *text_entry, gpointer data)
 {
@@ -143,7 +151,7 @@ static void bar_pane_exif_setup_entry_box(PaneExifData *ped, ExifEntry *ee)
 //		gtk_widget_set_size_request(ee->value_widget, 100, -1);
 		gtk_misc_set_alignment(GTK_MISC(ee->value_widget), 0.0, 0.5);
 		}
-		
+
 	gtk_box_pack_start(GTK_BOX(ee->box), ee->value_widget, TRUE, TRUE, 1);
 	gtk_widget_show(ee->value_widget);
 }
@@ -151,7 +159,7 @@ static void bar_pane_exif_setup_entry_box(PaneExifData *ped, ExifEntry *ee)
 static GtkWidget *bar_pane_exif_add_entry(PaneExifData *ped, const gchar *key, const gchar *title, gboolean if_set, gboolean editable)
 {
 	ExifEntry *ee = g_new0(ExifEntry, 1);
-	
+
 	ee->key = g_strdup(key);
 	if (title && title[0])
 		{
@@ -162,27 +170,28 @@ static GtkWidget *bar_pane_exif_add_entry(PaneExifData *ped, const gchar *key, c
 		ee->title = exif_get_description_by_key(key);
 		ee->auto_title = TRUE;
 		}
-		
+
 	ee->if_set = if_set;
 	ee->editable = editable;
-	
+
 	ee->ped = ped;
-	
+
 	ee->ebox = gtk_event_box_new();
 	g_object_set_data(G_OBJECT(ee->ebox), "entry_data", ee);
 	g_signal_connect_after(G_OBJECT(ee->ebox), "destroy",
 			       G_CALLBACK(bar_pane_exif_entry_destroy), ee);
-	
+
 	gtk_box_pack_start(GTK_BOX(ped->vbox), ee->ebox, FALSE, FALSE, 0);
 
 	bar_pane_exif_entry_dnd_init(ee->ebox);
 	g_signal_connect(ee->ebox, "button_release_event", G_CALLBACK(bar_pane_exif_menu_cb), ped);
-	
+	g_signal_connect(ee->ebox, "button_press_event", G_CALLBACK(bar_pane_exif_copy_cb), ped);
+
 	bar_pane_exif_setup_entry_box(ped, ee);
-	 
+
 	bar_pane_exif_entry_update_title(ee);
 	bar_pane_exif_update(ped);
-	
+
 	return ee->ebox;
 }
 
@@ -191,16 +200,16 @@ static void bar_pane_exif_reparent_entry(GtkWidget *entry, GtkWidget *pane)
 	PaneExifData *ped = g_object_get_data(G_OBJECT(pane), "pane_data");
 	PaneExifData *old_ped;
 	ExifEntry *ee = g_object_get_data(G_OBJECT(entry), "entry_data");
-	
+
 	if (!ped || !ee) return;
-	
+
 	old_ped = ee->ped;
-	
+
 	g_object_ref(entry);
-	
+
 	gtk_size_group_remove_widget(old_ped->size_group, ee->title_label);
 	gtk_container_remove(GTK_CONTAINER(old_ped->vbox), entry);
-	
+
 	ee->ped = ped;
 	gtk_size_group_add_widget(ped->size_group, ee->title_label);
 	gtk_box_pack_start(GTK_BOX(ped->vbox), entry, FALSE, FALSE, 0);
@@ -219,7 +228,7 @@ static void bar_pane_exif_update_entry(PaneExifData *ped, GtkWidget *entry, gboo
 {
 	gchar *text;
 	ExifEntry *ee = g_object_get_data(G_OBJECT(entry), "entry_data");
-	
+
 	if (!ee) return;
 	text = metadata_read_string(ped->fd, ee->key, ee->editable ? METADATA_PLAIN : METADATA_FORMATTED);
 
@@ -245,9 +254,9 @@ static void bar_pane_exif_update_entry(PaneExifData *ped, GtkWidget *entry, gboo
 		gtk_widget_show(entry);
 		ped->all_hidden = FALSE;
 		}
-		
+
 	g_free(text);
-	
+
 	if (update_title) bar_pane_exif_entry_update_title(ee);
 }
 
@@ -257,13 +266,13 @@ static void bar_pane_exif_update(PaneExifData *ped)
 
 	ped->all_hidden = TRUE;
 
-	list = gtk_container_get_children(GTK_CONTAINER(ped->vbox));	
+	list = gtk_container_get_children(GTK_CONTAINER(ped->vbox));
 	work = list;
 	while (work)
 		{
 		GtkWidget *entry = work->data;
 		work = work->next;
-		
+
 		bar_pane_exif_update_entry(ped, entry, FALSE);
 		}
 	g_list_free(list);
@@ -293,7 +302,7 @@ gint bar_pane_exif_event(GtkWidget *bar, GdkEvent *event)
 	ped = g_object_get_data(G_OBJECT(bar), "pane_data");
 	if (!ped) return FALSE;
 
-	list = gtk_container_get_children(GTK_CONTAINER(ped->vbox));	
+	list = gtk_container_get_children(GTK_CONTAINER(ped->vbox));
 	work = list;
 	while (!ret && work)
 		{
@@ -301,11 +310,7 @@ gint bar_pane_exif_event(GtkWidget *bar, GdkEvent *event)
 		ExifEntry *ee = g_object_get_data(G_OBJECT(entry), "entry_data");
 		work = work->next;
 
-#if GTK_CHECK_VERSION(2,20,0)
 		if (ee->editable && gtk_widget_has_focus(ee->value_widget)) ret = gtk_widget_event(ee->value_widget, event);
-#else
-		if (ee->editable && GTK_WIDGET_HAS_FOCUS(ee->value_widget)) ret = gtk_widget_event(ee->value_widget, event);
-#endif
 		}
 	g_list_free(list);
 	return ret;
@@ -314,7 +319,7 @@ gint bar_pane_exif_event(GtkWidget *bar, GdkEvent *event)
 static void bar_pane_exif_notify_cb(FileData *fd, NotifyType type, gpointer data)
 {
 	PaneExifData *ped = data;
-	if ((type & (NOTIFY_REREAD | NOTIFY_CHANGE | NOTIFY_METADATA)) && fd == ped->fd) 
+	if ((type & (NOTIFY_REREAD | NOTIFY_CHANGE | NOTIFY_METADATA)) && fd == ped->fd)
 		{
 		DEBUG_1("Notify pane_exif: %s %04x", fd->path, type);
 		bar_pane_exif_update(ped);
@@ -350,7 +355,7 @@ static void bar_pane_exif_entry_dnd_get(GtkWidget *entry, GdkDragContext *contex
 	switch (info)
 		{
 		case TARGET_APP_EXIF_ENTRY:
-			gtk_selection_data_set(selection_data, selection_data->target,
+			gtk_selection_data_set(selection_data, gtk_selection_data_get_target(selection_data),
 					       8, (gpointer) &entry, sizeof(entry));
 			break;
 
@@ -359,7 +364,7 @@ static void bar_pane_exif_entry_dnd_get(GtkWidget *entry, GdkDragContext *contex
 			gtk_selection_data_set_text(selection_data, ee->key, -1);
 			break;
 		}
-	
+
 }
 
 static void bar_pane_exif_dnd_receive(GtkWidget *pane, GdkDragContext *context,
@@ -371,42 +376,41 @@ static void bar_pane_exif_dnd_receive(GtkWidget *pane, GdkDragContext *context,
 	GList *work, *list;
 	gint pos;
 	GtkWidget *new_entry = NULL;
-	
+
 	ped = g_object_get_data(G_OBJECT(pane), "pane_data");
 	if (!ped) return;
 
 	switch (info)
 		{
 		case TARGET_APP_EXIF_ENTRY:
-			new_entry = *(gpointer *)selection_data->data;
-			
-			if (new_entry->parent && new_entry->parent != ped->vbox) bar_pane_exif_reparent_entry(new_entry, pane);
-			
+			new_entry = *(gpointer *)gtk_selection_data_get_data(selection_data);
+
+			if (gtk_widget_get_parent(new_entry) && gtk_widget_get_parent(new_entry) != ped->vbox) bar_pane_exif_reparent_entry(new_entry, pane);
+
 			break;
 		default:
 			/* FIXME: this needs a check for valid exif keys */
-			new_entry = bar_pane_exif_add_entry(ped, (gchar *)selection_data->data, NULL, TRUE, FALSE);
+			new_entry = bar_pane_exif_add_entry(ped, (gchar *)gtk_selection_data_get_data(selection_data), NULL, TRUE, FALSE);
 			break;
 		}
 
-	list = gtk_container_get_children(GTK_CONTAINER(ped->vbox));	
+	list = gtk_container_get_children(GTK_CONTAINER(ped->vbox));
 	work = list;
 	pos = 0;
 	while (work)
 		{
 		gint nx, ny;
 		GtkWidget *entry = work->data;
+		GtkAllocation allocation;
 		work = work->next;
-		
+
 		if (entry == new_entry) continue;
-		
-#if GTK_CHECK_VERSION(2,20,0)
+
+		gtk_widget_get_allocation(entry, &allocation);
+
 		if (gtk_widget_is_drawable(entry) &&
-#else
-		if (GTK_WIDGET_DRAWABLE(entry) && 
-#endif
 		    gtk_widget_translate_coordinates(pane, entry, x, y, &nx, &ny) &&
-		    ny < entry->allocation.height / 2) break;
+		    ny < allocation.height / 2) break;
 		pos++;
 		}
 	g_list_free(list);
@@ -417,7 +421,7 @@ static void bar_pane_exif_dnd_receive(GtkWidget *pane, GdkDragContext *context,
 static void bar_pane_exif_entry_dnd_begin(GtkWidget *entry, GdkDragContext *context, gpointer data)
 {
 	ExifEntry *ee = g_object_get_data(G_OBJECT(entry), "entry_data");
-	
+
 	if (!ee) return;
 	dnd_set_drag_label(entry, context, ee->key);
 }
@@ -472,14 +476,14 @@ static void bar_pane_exif_edit_cancel_cb(GenericDialog *gd, gpointer data)
 static void bar_pane_exif_edit_ok_cb(GenericDialog *gd, gpointer data)
 {
 	ConfDialogData *cdd = data;
-	
+
 	/* either one or the other */
 	PaneExifData *ped = g_object_get_data(G_OBJECT(cdd->widget), "pane_data");
 	ExifEntry *ee = g_object_get_data(G_OBJECT(cdd->widget), "entry_data");
 
 	if (ped)
 		{
-		bar_pane_exif_add_entry(ped, 
+		bar_pane_exif_add_entry(ped,
 					gtk_entry_get_text(GTK_ENTRY(cdd->key_entry)),
 					gtk_entry_get_text(GTK_ENTRY(cdd->title_entry)),
 					cdd->if_set, cdd->editable);
@@ -488,17 +492,17 @@ static void bar_pane_exif_edit_ok_cb(GenericDialog *gd, gpointer data)
 	if (ee)
 		{
 		const gchar *title;
-		GtkWidget *pane = cdd->widget->parent;
-		
+		GtkWidget *pane = gtk_widget_get_parent(cdd->widget);
+
 		while (pane)
 			{
 			ped = g_object_get_data(G_OBJECT(pane), "pane_data");
 			if (ped) break;
-			pane = pane->parent;
+			pane = gtk_widget_get_parent(pane);
 			}
-		
+
 		if (!pane) return;
-		
+
 		g_free(ee->key);
 		ee->key = g_strdup(gtk_entry_get_text(GTK_ENTRY(cdd->key_entry)));
 		title = gtk_entry_get_text(GTK_ENTRY(cdd->title_entry));
@@ -514,10 +518,10 @@ static void bar_pane_exif_edit_ok_cb(GenericDialog *gd, gpointer data)
 			ee->title = g_strdup(title);
 			ee->auto_title = FALSE;
 			}
-		
+
 		ee->if_set = cdd->if_set;
 		ee->editable = cdd->editable;
-		
+
 		bar_pane_exif_setup_entry_box(ped, ee);
 
 		bar_pane_exif_entry_update_title(ee);
@@ -536,13 +540,13 @@ static void bar_pane_exif_conf_dialog(GtkWidget *widget)
 	ExifEntry *ee = g_object_get_data(G_OBJECT(widget), "entry_data");
 
 	cdd = g_new0(ConfDialogData, 1);
-	
+
 	cdd->widget = widget;
 
 
 	cdd->if_set = ee ? ee->if_set : TRUE;
 	cdd->editable = ee ? ee->editable : FALSE;
-	
+
 	cdd->gd = gd = generic_dialog_new(ee ? _("Configure entry") : _("Add entry"), "exif_entry_edit",
 				widget, TRUE,
 				bar_pane_exif_edit_cancel_cb, cdd);
@@ -553,7 +557,7 @@ static void bar_pane_exif_conf_dialog(GtkWidget *widget)
 	g_signal_connect(G_OBJECT(widget), "destroy",
 			 G_CALLBACK(bar_pane_exif_edit_close_cb), gd);
 
-	generic_dialog_add_message(gd, NULL, ee ? _("Configure entry") : _("Add entry"), NULL);
+	generic_dialog_add_message(gd, NULL, ee ? _("Configure entry") : _("Add entry"), NULL, FALSE);
 
 	generic_dialog_add_button(gd, GTK_STOCK_OK, NULL,
 				  bar_pane_exif_edit_ok_cb, TRUE);
@@ -595,6 +599,19 @@ static void bar_pane_exif_delete_entry_cb(GtkWidget *menu_widget, gpointer data)
 	gtk_widget_destroy(entry);
 }
 
+static void bar_pane_exif_copy_entry_cb(GtkWidget *menu_widget, gpointer data)
+{
+	GtkWidget *widget = data;
+	GtkClipboard *clipboard;
+	const gchar *value;
+	ExifEntry *ee;
+
+	ee = g_object_get_data(G_OBJECT(widget), "entry_data");
+	value = gtk_label_get_text(GTK_LABEL(ee->value_widget));
+	clipboard = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
+	gtk_clipboard_set_text(clipboard, value, -1);
+}
+
 static void bar_pane_exif_toggle_show_all_cb(GtkWidget *menu_widget, gpointer data)
 {
 	PaneExifData *ped = data;
@@ -616,11 +633,13 @@ static void bar_pane_exif_menu_popup(GtkWidget *widget, PaneExifData *ped)
 		/* for the entry */
 		gchar *conf = g_strdup_printf(_("Configure \"%s\""), ee->title);
 		gchar *del = g_strdup_printf(_("Remove \"%s\""), ee->title);
-		
+		gchar *copy = g_strdup_printf(_("Copy \"%s\""), ee->title);
+
 		menu_item_add_stock(menu, conf, GTK_STOCK_EDIT, G_CALLBACK(bar_pane_exif_conf_dialog_cb), widget);
 		menu_item_add_stock(menu, del, GTK_STOCK_DELETE, G_CALLBACK(bar_pane_exif_delete_entry_cb), widget);
+		menu_item_add_stock(menu, copy, GTK_STOCK_COPY, G_CALLBACK(bar_pane_exif_copy_entry_cb), widget);
 		menu_item_add_divider(menu);
-		
+
 		g_free(conf);
 		g_free(del);
 		}
@@ -628,12 +647,12 @@ static void bar_pane_exif_menu_popup(GtkWidget *widget, PaneExifData *ped)
 	/* for the pane */
 	menu_item_add_stock(menu, _("Add entry"), GTK_STOCK_ADD, G_CALLBACK(bar_pane_exif_conf_dialog_cb), ped->widget);
 	menu_item_add_check(menu, _("Show hidden entries"), ped->show_all, G_CALLBACK(bar_pane_exif_toggle_show_all_cb), ped);
-	
+
 	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 0, GDK_CURRENT_TIME);
 }
 
-static gboolean bar_pane_exif_menu_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data) 
-{ 
+static gboolean bar_pane_exif_menu_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
+{
 	PaneExifData *ped = data;
 	if (bevent->button == MOUSE_BUTTON_RIGHT)
 		{
@@ -641,7 +660,26 @@ static gboolean bar_pane_exif_menu_cb(GtkWidget *widget, GdkEventButton *bevent,
 		return TRUE;
 		}
 	return FALSE;
-} 
+}
+
+static gboolean bar_pane_exif_copy_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
+{
+	const gchar *value;
+	GtkClipboard *clipboard;
+	ExifEntry *ee;
+
+	if (bevent->button == MOUSE_BUTTON_LEFT)
+		{
+		ee = g_object_get_data(G_OBJECT(widget), "entry_data");
+		value = gtk_label_get_text(GTK_LABEL(ee->value_widget));
+		clipboard = gtk_clipboard_get(GDK_SELECTION_PRIMARY);
+		gtk_clipboard_set_text(clipboard, value, -1);
+
+		return TRUE;
+		}
+
+	return FALSE;
+}
 
 
 
@@ -662,7 +700,7 @@ static void bar_pane_exif_write_config(GtkWidget *pane, GString *outstr, gint in
 {
 	PaneExifData *ped;
 	GList *work, *list;
-	
+
 	ped = g_object_get_data(G_OBJECT(pane), "pane_data");
 	if (!ped) return;
 
@@ -672,14 +710,14 @@ static void bar_pane_exif_write_config(GtkWidget *pane, GString *outstr, gint in
 	WRITE_BOOL(ped->pane, expanded);
 	WRITE_STRING(">");
 	indent++;
-	
-	list = gtk_container_get_children(GTK_CONTAINER(ped->vbox));	
+
+	list = gtk_container_get_children(GTK_CONTAINER(ped->vbox));
 	work = list;
 	while (work)
 		{
 		GtkWidget *entry = work->data;
 		work = work->next;
-		
+
 		bar_pane_exif_entry_write_config(entry, outstr, indent);
 		}
 	g_list_free(list);
@@ -709,6 +747,7 @@ static void bar_pane_exif_destroy(GtkWidget *widget, gpointer data)
 	g_free(ped);
 }
 
+#if !GTK_CHECK_VERSION(3,0,0)
 static void bar_pane_exif_size_request(GtkWidget *pane, GtkRequisition *requisition, gpointer data)
 {
 	PaneExifData *ped = data;
@@ -717,11 +756,15 @@ static void bar_pane_exif_size_request(GtkWidget *pane, GtkRequisition *requisit
 		requisition->height = ped->min_height;
 		}
 }
+#endif
 
 static void bar_pane_exif_size_allocate(GtkWidget *pane, GtkAllocation *alloc, gpointer data)
 {
 	PaneExifData *ped = data;
 	ped->min_height = alloc->height;
+#if GTK_CHECK_VERSION(3,0,0)
+	gtk_widget_set_size_request(ped->widget, -1, ped->min_height);
+#endif
 }
 
 static GtkWidget *bar_pane_exif_new(const gchar *id, const gchar *title, gboolean expanded)
@@ -748,11 +791,15 @@ static GtkWidget *bar_pane_exif_new(const gchar *id, const gchar *title, gboolea
 	g_object_set_data(G_OBJECT(ped->widget), "pane_data", ped);
 	g_signal_connect_after(G_OBJECT(ped->widget), "destroy",
 			       G_CALLBACK(bar_pane_exif_destroy), ped);
+#if GTK_CHECK_VERSION(3,0,0)
+	gtk_widget_set_size_request(ped->widget, -1, ped->min_height);
+#else
 	g_signal_connect(G_OBJECT(ped->widget), "size-request",
 			 G_CALLBACK(bar_pane_exif_size_request), ped);
+#endif
 	g_signal_connect(G_OBJECT(ped->widget), "size-allocate",
 			 G_CALLBACK(bar_pane_exif_size_allocate), ped);
-	
+
 	bar_pane_exif_dnd_init(ped->widget);
 	g_signal_connect(ped->widget, "button_release_event", G_CALLBACK(bar_pane_exif_menu_cb), ped);
 
@@ -781,7 +828,7 @@ GtkWidget *bar_pane_exif_new_from_config(const gchar **attribute_names, const gc
 
 		log_printf("unknown attribute %s = %s\n", option, value);
 		}
-	
+
 	bar_pane_translate_title(PANE_EXIF, id, &title);
 	ret = bar_pane_exif_new(id, title, expanded);
 	g_free(title);
@@ -805,7 +852,7 @@ void bar_pane_exif_update_from_config(GtkWidget *pane, const gchar **attribute_n
 		if (READ_CHAR_FULL("title", title)) continue;
 		if (READ_BOOL_FULL("expanded", ped->pane.expanded)) continue;
 		if (READ_CHAR_FULL("id", ped->pane.id)) continue;
-		
+
 
 		log_printf("unknown attribute %s = %s\n", option, value);
 		}
@@ -842,10 +889,10 @@ void bar_pane_exif_entry_add_from_config(GtkWidget *pane, const gchar **attribut
 		if (READ_CHAR_FULL("title", title)) continue;
 		if (READ_BOOL_FULL("if_set", if_set)) continue;
 		if (READ_BOOL_FULL("editable", editable)) continue;
-		
+
 		log_printf("unknown attribute %s = %s\n", option, value);
 		}
-	
+
 	if (key && key[0]) bar_pane_exif_add_entry(ped, key, title, if_set, editable);
 }
 
