@@ -1,16 +1,24 @@
 /*
- * Geeqie
- * (C) 2006 John Ellis
- * Copyright (C) 2008 - 2012 The Geeqie Team
+ * Copyright (C) 2004 John Ellis
+ * Copyright (C) 2008 - 2016 The Geeqie Team
  *
  * Author: John Ellis
  *
- * This software is released under the GNU General Public License (GNU GPL).
- * Please read the included file COPYING for more information.
- * This software comes with no warranty of any kind, use at your own risk!
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-
 #include "main.h"
 #include "bar_sort.h"
 
@@ -67,7 +75,7 @@ struct _SortData
 	SortModeType mode;
 	SortActionType action;
 	gchar *filter_key;
-	
+
 	SortSelectionType selection;
 
 	GtkWidget *folder_group;
@@ -77,8 +85,8 @@ struct _SortData
 	GtkWidget *undo_button;
 	SortActionType undo_action;
 	GList *undo_src_list;
-	gchar *undo_src;
-	gchar *undo_dest;
+	GList *undo_dest_list;
+	gchar *undo_collection;
 };
 
 
@@ -86,7 +94,7 @@ struct _SortData
 #define SORT_KEY_COLLECTIONS "sort_manager_collections"
 
 
-static void bar_sort_undo_set(SortData *sd, GList *src_list, FileData *src, const gchar *dest);
+static void bar_sort_undo_set(SortData *sd, GList *src_list, const gchar *dest);
 static void bar_sort_add_close(SortData *sd);
 
 
@@ -157,7 +165,7 @@ static void bar_sort_mode_sync(SortData *sd, SortModeType mode)
 
 	bar_sort_add_close(sd);
 
-	bar_sort_undo_set(sd, NULL, NULL, NULL);
+	bar_sort_undo_set(sd, NULL, NULL);
 }
 
 static void bar_sort_mode_cb(GtkWidget *combo, gpointer data)
@@ -175,28 +183,42 @@ static void bar_sort_mode_cb(GtkWidget *combo, gpointer data)
 }
 
 /* this takes control of src_list */
-static void bar_sort_undo_set(SortData *sd, GList *src_list, FileData *src, const gchar *dest)
+static void bar_sort_undo_set(SortData *sd, GList *src_list, const gchar *dest)
 {
 	string_list_free(sd->undo_src_list);
 	sd->undo_src_list = filelist_to_path_list(src_list);
 
-	g_free(sd->undo_src);
-	sd->undo_src = src ? g_strdup(src->path) : NULL;
-	g_free(sd->undo_dest);
-	sd->undo_dest = g_strdup(dest);
+	if (src_list)
+		{
+		/* we should create the undo_dest_list to use it later... */
+		string_list_free(sd->undo_dest_list);
+		sd->undo_dest_list=NULL;
+
+		GList *work = sd->undo_src_list;
+		while(work)
+			{
+			gchar *filename =  g_strdup(filename_from_path(work->data));
+			gchar *dest_path = g_build_filename(g_strdup(dest), filename, NULL);
+			sd->undo_dest_list = g_list_prepend(sd->undo_dest_list, g_strdup(dest_path));
+			work = work->next;
+			}
+		sd->undo_dest_list = g_list_reverse(sd->undo_dest_list);
+		}
 
 	sd->undo_action = sd->action;
 
 	if (sd->undo_button)
 		{
 		gtk_widget_set_sensitive(sd->undo_button,
-					 ((sd->undo_src_list || sd->undo_src) && sd->undo_dest) );
+					((sd->undo_src_list ) && sd->undo_dest_list));
 		}
 }
 
 static void bar_sort_undo_folder(SortData *sd, GtkWidget *button)
 {
-	if (!sd->undo_src || !sd->undo_dest) return;
+	gchar *origin;
+
+	if (!(sd->undo_src_list && sd->undo_dest_list)) return;
 
 	switch (sd->undo_action)
 		{
@@ -204,30 +226,57 @@ static void bar_sort_undo_folder(SortData *sd, GtkWidget *button)
 			{
 			GList *list;
 			gchar *src_dir;
+			gchar *src_path;
 
-			list = g_list_append(NULL, file_data_new_group(sd->undo_dest));
-			src_dir = remove_level_from_path(sd->undo_src);
-			file_util_move_simple(list, src_dir, sd->lw->window);
-			g_free(src_dir);
+			if (sd->undo_src_list)
+				{
+				GList *work = NULL;
+
+				src_path = g_strdup(sd->undo_src_list->data);
+				src_dir = remove_level_from_path(src_path);
+				list = sd->undo_dest_list;
+				while (list)
+					{
+					work = g_list_prepend(work, file_data_new_group(list->data));
+					list=list->next;
+					}
+				file_util_move_simple(work, src_dir, sd->lw->window);
+				g_free(src_dir);
+				g_free(src_path);
+				}
 			}
 			break;
+
 		case BAR_SORT_COPY:
-			file_util_delete(file_data_new_group(sd->undo_dest), NULL, button);
+		case BAR_SORT_FILTER:
+			if (sd->undo_src_list)
+				{
+				GList *delete_list;
+				GList *work = NULL;
+
+				delete_list = sd->undo_dest_list;
+				while (delete_list)
+					{
+					work = g_list_append(work, file_data_new_group(delete_list->data));
+					delete_list = delete_list->next;
+					}
+				file_util_delete(NULL, work, button);
+				}
 			break;
+
 		default:
-			/* undo external command */
-			file_util_delete(file_data_new_group(sd->undo_dest), NULL, button);
 			break;
 		}
 
 	layout_refresh(sd->lw);
+	origin = (sd->undo_src_list)->data;
 
-	if (isfile(sd->undo_src))
+	if (isfile(origin))
 		{
-		layout_image_set_fd(sd->lw, file_data_new_group(sd->undo_src));
+		layout_image_set_fd(sd->lw, file_data_new_group(origin));
 		}
 
-	bar_sort_undo_set(sd, NULL, NULL, NULL);
+	bar_sort_undo_set(sd, NULL, NULL);
 }
 
 static void bar_sort_undo_collection(SortData *sd)
@@ -238,13 +287,12 @@ static void bar_sort_undo_collection(SortData *sd)
 	while (work)
 		{
 		gchar *source;
-
 		source = work->data;
 		work = work->next;
-		collect_manager_remove(file_data_new_group(source), sd->undo_dest);
+		collect_manager_remove(file_data_new_group(source), sd->undo_collection);
 		}
 
-	bar_sort_undo_set(sd, NULL, NULL, NULL);
+	bar_sort_undo_set(sd, NULL,  NULL);
 }
 
 static void bar_sort_undo_cb(GtkWidget *button, gpointer data)
@@ -263,38 +311,40 @@ static void bar_sort_undo_cb(GtkWidget *button, gpointer data)
 
 static void bar_sort_bookmark_select_folder(SortData *sd, FileData *source, const gchar *path)
 {
-	GList *list;
-	gchar *dest_path;
+	GList *orig_list;
+	GList *action_list;
+	GList *undo_src_list;
 
 	if (!isdir(path)) return;
 
-	dest_path = g_build_filename(path, source->name, NULL);
-	bar_sort_undo_set(sd, NULL, source, dest_path);
+	orig_list = layout_selection_list(sd->lw);
+	action_list = orig_list;
+	undo_src_list = orig_list;
+	orig_list = NULL;
 
-	list = g_list_append(NULL, file_data_ref(source));
+	bar_sort_undo_set(sd, undo_src_list, path);
 
 	switch (sd->action)
 		{
 		case BAR_SORT_COPY:
-			file_util_copy_simple(list, path, sd->lw->window);
-			list = NULL;
+			file_util_copy_simple(action_list, path, sd->lw->window);
+			action_list = NULL;
 			layout_image_next(sd->lw);
 			break;
+
 		case BAR_SORT_MOVE:
-			file_util_move_simple(list, path, sd->lw->window);
-			list = NULL;
+			file_util_move_simple(action_list, path, sd->lw->window);
+			action_list = NULL;
 			break;
+
 		case BAR_SORT_FILTER:
-			file_util_start_filter_from_filelist(sd->filter_key, list, path, sd->lw->window);
-			list = NULL;
+			file_util_start_filter_from_filelist(sd->filter_key, action_list, path, sd->lw->window);
 			layout_image_next(sd->lw);
 			break;
+
 		default:
 			break;
 		}
-
-	g_list_free(list);
-	g_free(dest_path);
 }
 
 static void bar_sort_bookmark_select_collection(SortData *sd, FileData *source, const gchar *path)
@@ -315,11 +365,12 @@ static void bar_sort_bookmark_select_collection(SortData *sd, FileData *source,
 
 	if (!list)
 		{
-		bar_sort_undo_set(sd, NULL, NULL, NULL);
+		bar_sort_undo_set(sd, NULL, NULL);
 		return;
 		}
 
-	bar_sort_undo_set(sd, list, NULL, path);
+	bar_sort_undo_set(sd, list, path);
+	sd->undo_collection = g_strdup(path);
 
 	while (list)
 		{
@@ -509,7 +560,7 @@ static void bar_sort_add_cb(GtkWidget *button, gpointer data)
 				       bar_sort_add_cancel_cb, sd);
 	file_dialog_add_button(sd->dialog, GTK_STOCK_OK, NULL, bar_sort_add_ok_cb, TRUE);
 
-	generic_dialog_add_message(GENERIC_DIALOG(sd->dialog), NULL, title, NULL);
+	generic_dialog_add_message(GENERIC_DIALOG(sd->dialog), NULL, title, NULL, FALSE);
 
 	if (sd->mode == BAR_SORT_MODE_FOLDER)
 		{
@@ -550,8 +601,9 @@ static void bar_sort_destroy(GtkWidget *widget, gpointer data)
 	bar_sort_add_close(sd);
 
 	g_free(sd->filter_key);
-	g_free(sd->undo_src);
-	g_free(sd->undo_dest);
+	string_list_free(sd->undo_src_list);
+	string_list_free(sd->undo_dest_list);
+	g_free(sd->undo_collection);
 	g_free(sd);
 }
 
@@ -579,15 +631,16 @@ static GtkWidget *bar_sort_new(LayoutWindow *lw, SortActionType action,
 	sd->lw = lw;
 
 	sd->action = action;
-	
+
 	if (sd->action == BAR_SORT_FILTER && (!filter_key || !filter_key[0]))
 		{
 		sd->action = BAR_SORT_COPY;
 		}
-	
+
 	sd->selection = selection;
-	sd->undo_src = NULL;
-	sd->undo_dest = NULL;
+	sd->undo_src_list = NULL;
+	sd->undo_dest_list = NULL;
+	sd->undo_collection = NULL;
 
 	sd->vbox = gtk_vbox_new(FALSE, PREF_PAD_GAP);
 	g_object_set_data(G_OBJECT(sd->vbox), "bar_sort_data", sd);
@@ -599,12 +652,12 @@ static GtkWidget *bar_sort_new(LayoutWindow *lw, SortActionType action,
 	gtk_box_pack_start(GTK_BOX(sd->vbox), label, FALSE, FALSE, 0);
 	gtk_widget_show(label);
 
-	combo = gtk_combo_box_new_text();
+	combo = gtk_combo_box_text_new();
 	gtk_box_pack_start(GTK_BOX(sd->vbox), combo, FALSE, FALSE, 0);
 	gtk_widget_show(combo);
 
-	gtk_combo_box_append_text(GTK_COMBO_BOX(combo), _("Folders"));
-	gtk_combo_box_append_text(GTK_COMBO_BOX(combo), _("Collections"));
+	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(combo), _("Folders"));
+	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(combo), _("Collections"));
 
 	g_signal_connect(G_OBJECT(combo), "changed",
 			 G_CALLBACK(bar_sort_mode_cb), sd);
@@ -630,9 +683,9 @@ static GtkWidget *bar_sort_new(LayoutWindow *lw, SortActionType action,
 		gboolean select = FALSE;
 
 		work = work->next;
-			
+
 		if (!editor_is_filter(editor->key)) continue;
-		
+
 		key = g_strdup(editor->key);
 		if (sd->action == BAR_SORT_FILTER && strcmp(key, filter_key) == 0)
 			{
@@ -640,7 +693,7 @@ static GtkWidget *bar_sort_new(LayoutWindow *lw, SortActionType action,
 			select = TRUE;
 			have_filter = TRUE;
 			}
-		
+
 		button = pref_radiobutton_new(sd->folder_group, buttongrp,
 					      editor->name, select,
 					      G_CALLBACK(bar_sort_set_filter_cb), sd);
@@ -648,7 +701,7 @@ static GtkWidget *bar_sort_new(LayoutWindow *lw, SortActionType action,
 		g_object_set_data_full(G_OBJECT(button), "filter_key", key, bar_sort_edit_button_free);
 		}
 	g_list_free(editors_list);
-	
+
 	if (sd->action == BAR_SORT_FILTER && !have_filter) sd->action = BAR_SORT_COPY;
 
 	sd->collection_group = pref_box_new(sd->vbox, FALSE, GTK_ORIENTATION_VERTICAL, 0);
@@ -683,7 +736,7 @@ static GtkWidget *bar_sort_new(LayoutWindow *lw, SortActionType action,
 GtkWidget *bar_sort_new_from_config(LayoutWindow *lw, const gchar **attribute_names, const gchar **attribute_values)
 {
 	GtkWidget *bar;
-	
+
 	gboolean enabled = TRUE;
 	gint action = 0;
 	gint mode = 0;
@@ -710,6 +763,44 @@ GtkWidget *bar_sort_new_from_config(LayoutWindow *lw, const gchar **attribute_na
 	return bar;
 }
 
+/**
+ * @brief Sets the bar_sort_enabled flag
+ * @param lw 
+ * @param attribute_names 
+ * @param attribute_values 
+ * 
+ * Called from rcfile when processing geeqierc.xml on start-up.
+ * It is neccesary to set the bar_sort_enabled flag because
+ * the sort manager and desktop files are set up in the idle loop, and
+ * setup is not yet completed during initialisation.
+ * The flag is checked in layout_editors_reload_idle_cb.
+ * action, mode, selection and filter_key are ignored.
+ */
+void bar_sort_cold_start(LayoutWindow *lw, const gchar **attribute_names, const gchar **attribute_values)
+{
+	gboolean enabled = TRUE;
+	gint action = 0;
+	gint mode = 0;
+	gint selection = 0;
+	gchar *filter_key = NULL;
+
+	while (attribute_names && *attribute_names)
+		{
+		const gchar *option = *attribute_names++;
+		const gchar *value = *attribute_values++;
+
+		if (READ_BOOL_FULL("enabled", enabled)) continue;
+		if (READ_INT_CLAMP_FULL("action", action, 0, BAR_SORT_ACTION_COUNT - 1)) continue;
+		if (READ_INT_CLAMP_FULL("mode", mode, 0, BAR_SORT_MODE_COUNT - 1)) continue;
+		if (READ_INT_CLAMP_FULL("selection", selection, 0, BAR_SORT_SELECTION_COUNT - 1)) continue;
+		if (READ_CHAR_FULL("filter_key", filter_key)) continue;
+
+		log_printf("unknown attribute %s = %s\n", option, value);
+		}
+
+	lw->bar_sort_enabled = enabled;
+}
+
 GtkWidget *bar_sort_new_default(LayoutWindow *lw)
 {
 	return bar_sort_new_from_config(lw, NULL, NULL);
@@ -724,11 +815,7 @@ void bar_sort_write_config(GtkWidget *bar, GString *outstr, gint indent)
 	if (!sd) return;
 
 	WRITE_NL(); WRITE_STRING("<bar_sort ");
-#if GTK_CHECK_VERSION(2,20,0)
 	write_bool_option(outstr, indent, "enabled", gtk_widget_get_visible(bar));
-#else
-	write_bool_option(outstr, indent, "enabled", GTK_WIDGET_VISIBLE(bar));
-#endif
 	WRITE_INT(*sd, mode);
 	WRITE_INT(*sd, action);
 	WRITE_INT(*sd, selection);
