@@ -1,13 +1,22 @@
 /*
- * Geeqie
- * (C) 2006 John Ellis
- * Copyright (C) 2008 - 2012 The Geeqie Team
+ * Copyright (C) 2006 John Ellis
+ * Copyright (C) 2008 - 2016 The Geeqie Team
  *
  * Author: John Ellis
  *
- * This software is released under the GNU General Public License (GNU GPL).
- * Please read the included file COPYING for more information.
- * This software comes with no warranty of any kind, use at your own risk!
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include "main.h"
@@ -23,6 +32,7 @@
 #include "ui_spinner.h"
 #include "ui_tabcomp.h"
 #include "ui_utildlg.h"
+#include "window.h"
 
 
 typedef struct _CMData CMData;
@@ -38,6 +48,7 @@ struct _CMData
 	GtkWidget *button_close;
 	gboolean clear;
 	gboolean metadata;
+	gboolean remote;
 };
 
 #define PURGE_DIALOG_WIDTH 400
@@ -49,25 +60,6 @@ struct _CMData
  *-------------------------------------------------------------------
  */
 
-#if 0
-static gint extension_truncate(gchar *path, const gchar *ext)
-{
-	gint l;
-	gint el;
-
-	if (!path || !ext) return FALSE;
-
-	l = strlen(path);
-	el = strlen(ext);
-
-	if (l < el || strcmp(path + (l - el), ext) != 0) return FALSE;
-
-	path[l - el] = '\0';
-
-	return TRUE;
-}
-#endif
-
 static gchar *extension_find_dot(gchar *path)
 {
 	gchar *dot = NULL;
@@ -126,11 +118,14 @@ static void cache_maintain_home_stop(CMData *cm)
 		cm->idle_id = 0;
 		}
 
-	gtk_entry_set_text(GTK_ENTRY(cm->entry), _("done"));
-	spinner_set_interval(cm->spinner, -1);
+	if (!cm->remote)
+		{
+		gtk_entry_set_text(GTK_ENTRY(cm->entry), _("done"));
+		spinner_set_interval(cm->spinner, -1);
 
-	gtk_widget_set_sensitive(cm->button_stop, FALSE);
-	gtk_widget_set_sensitive(cm->button_close, TRUE);
+		gtk_widget_set_sensitive(cm->button_stop, FALSE);
+		gtk_widget_set_sensitive(cm->button_close, TRUE);
+		}
 }
 
 static gboolean cache_maintain_home_cb(gpointer data)
@@ -143,6 +138,7 @@ static gboolean cache_maintain_home_cb(gpointer data)
 	gboolean still_have_a_file = TRUE;
 	gsize base_length;
 	const gchar *cache_folder;
+	gboolean filter_disable;
 
 	if (cm->metadata)
 		{
@@ -167,6 +163,13 @@ static gboolean cache_maintain_home_cb(gpointer data)
 
 	DEBUG_1("purge chk (%d) \"%s\"", (cm->clear && !cm->metadata), fd->path);
 
+/**
+ * It is necessary to disable the file filter when clearing the cache,
+ * otherwise the .sim (file similarity) files are not deleted.
+ */
+	filter_disable = options->file_filter.disable;
+	options->file_filter.disable = TRUE;
+
 	if (g_list_find(cm->done_list, fd) == NULL)
 		{
 		cm->done_list = g_list_prepend(cm->done_list, fd);
@@ -203,6 +206,8 @@ static gboolean cache_maintain_home_cb(gpointer data)
 				}
 			}
 		}
+	options->file_filter.disable = filter_disable;
+
 	filelist_free(list);
 
 	cm->list = g_list_concat(dlist, cm->list);
@@ -233,7 +238,7 @@ static gboolean cache_maintain_home_cb(gpointer data)
 		file_data_unref(fd);
 		}
 
-	if (cm->list)
+	if (cm->list && !cm->remote)
 		{
 		const gchar *buf;
 
@@ -256,11 +261,7 @@ static void cache_maintain_home_close_cb(GenericDialog *gd, gpointer data)
 {
 	CMData *cm = data;
 
-#if GTK_CHECK_VERSION(2,20,0)
 	if (!gtk_widget_get_sensitive(cm->button_close)) return;
-#else
-	if (!GTK_WIDGET_SENSITIVE(cm->button_close)) return;
-#endif
 
 	cache_maintain_home_close(cm);
 }
@@ -305,6 +306,7 @@ void cache_maintain_home(gboolean metadata, gboolean clear, GtkWidget *parent)
 	cm->done_list = NULL;
 	cm->clear = clear;
 	cm->metadata = metadata;
+	cm->remote = FALSE;
 
 	if (metadata)
 		{
@@ -330,7 +332,7 @@ void cache_maintain_home(gboolean metadata, gboolean clear, GtkWidget *parent)
 	cm->button_stop = generic_dialog_add_button(cm->gd, GTK_STOCK_STOP, NULL,
 						    cache_maintain_home_stop_cb, FALSE);
 
-	generic_dialog_add_message(cm->gd, NULL, msg, NULL);
+	generic_dialog_add_message(cm->gd, NULL, msg, NULL, FALSE);
 	gtk_window_set_default_size(GTK_WINDOW(cm->gd->dialog), PURGE_DIALOG_WIDTH, -1);
 
 	hbox = gtk_hbox_new(FALSE, 0);
@@ -338,7 +340,7 @@ void cache_maintain_home(gboolean metadata, gboolean clear, GtkWidget *parent)
 	gtk_widget_show(hbox);
 
 	cm->entry = gtk_entry_new();
-	GTK_WIDGET_UNSET_FLAGS(cm->entry, GTK_CAN_FOCUS);
+	gtk_widget_set_can_focus(cm->entry, FALSE);
 	gtk_editable_set_editable(GTK_EDITABLE(cm->entry), FALSE);
 	gtk_box_pack_start(GTK_BOX(hbox), cm->entry, TRUE, TRUE, 0);
 	gtk_widget_show(cm->entry);
@@ -352,160 +354,41 @@ void cache_maintain_home(gboolean metadata, gboolean clear, GtkWidget *parent)
 	cm->idle_id = g_idle_add(cache_maintain_home_cb, cm);
 }
 
-#if 0
-/* This checks all files in ~/GQ_RC_DIR/thumbnails and
- * removes them if thay have no source counterpart.
- * (this assumes all cache files have an extension of 4 chars including '.')
- */
-gint cache_maintain_home_dir(const gchar *dir, gint recursive, gint clear)
+void cache_maintain_home_remote(gboolean metadata, gboolean clear)
 {
-	gchar *base;
-	gint base_length;
-	GList *dlist = NULL;
+	CMData *cm;
+	GList *dlist;
 	FileData *dir_fd;
-	GList *flist = NULL;
-	gboolean still_have_a_file = FALSE;
+	const gchar *cache_folder;
 
-	DEBUG_1("maintainance check: %s", dir);
-
-	base_length = strlen(homedir()) + strlen("/") + strlen(GQ_CACHE_RC_THUMB);
-	base = g_strconcat(homedir(), "/", GQ_CACHE_RC_THUMB, dir, NULL);
-	dir_fd = file_data_new_dir(base);
-	g_free(base);
-
-	if (filelist_read(dir_fd, &flist, &dlist))
+	if (metadata)
 		{
-		GList *work;
-
-		work = dlist;
-		while (work)
-			{
-			FileData *fd = work->data;
-			if (recursive && strlen(fd->path) > base_length &&
-			    !cache_maintain_home_dir(fd->path + base_length, recursive, clear))
-				{
-				DEBUG_1("Deleting thumb dir: %s", fd->path);
-				if (!rmdir_utf8(fd->path))
-					{
-					log_printf("Unable to delete dir: %s\n", fd->path);
-					}
-				}
-			else
-				{
-				still_have_a_file = TRUE;
-				}
-			work = work->next;
-			}
-
-		work = flist;
-		while (work)
-			{
-			FileData *fd = work->data;
-			gchar *path = g_strdup(fd->path);
-			gchar *dot;
-
-			dot = extension_find_dot(path);
-
-			if (dot) *dot = '\0';
-			if (clear ||
-			    (strlen(path) > base_length && !isfile(path + base_length)) )
-				{
-				if (dot) *dot = '.';
-				if (!unlink_file(path)) log_printf("failed to delete:%s\n", path);
-				}
-			else
-				{
-				still_have_a_file = TRUE;
-				}
-			g_free(path);
-
-			work = work->next;
-			}
+		cache_folder = get_metadata_cache_dir();
+		}
+	else
+		{
+		cache_folder = get_thumbnails_cache_dir();
 		}
 
-	filelist_free(dlist);
-	filelist_free(flist);
-	file_data_unref(dir_fd);
+	dir_fd = file_data_new_dir(cache_folder);
+	if (!filelist_read(dir_fd, NULL, &dlist))
+		{
+		file_data_unref(dir_fd);
+		return;
+		}
 
-	return still_have_a_file;
+	dlist = g_list_append(dlist, dir_fd);
+
+	cm = g_new0(CMData, 1);
+	cm->list = dlist;
+	cm->done_list = NULL;
+	cm->clear = clear;
+	cm->metadata = metadata;
+	cm->remote = TRUE;
+
+	cm->idle_id = g_idle_add(cache_maintain_home_cb, cm);
 }
 
-/* This checks relative caches in dir/.thumbnails and
- * removes them if they have no source counterpart.
- */
-gint cache_maintain_dir(FileData *dir_fd, gint recursive, gint clear)
-{
-	GList *list = NULL;
-	gchar *cachedir;
-	FileData *cachedir_fd;
-	gboolean still_have_a_file = FALSE;
-	GList *work;
-
-	cachedir = g_build_filename(dir, GQ_CACHE_LOCAL_THUMB, NULL);
-	cachedir_fd = file_data_new_dir(cachedir);
-	g_free(cachedir);
-
-	filelist_read(cachedir_fd, &list, NULL);
-	work = list;
-
-	while (work)
-		{
-		FileData *fd;
-		gchar *source;
-
-		fd = work->data;
-		work = work->next;
-
-		source = g_build_filename(dir->path, fd->name, NULL);
-
-		if (clear ||
-		    extension_truncate(source, GQ_CACHE_EXT_THUMB) ||
-		    extension_truncate(source, GQ_CACHE_EXT_SIM))
-			{
-			if (!clear && isfile(source))
-				{
-				still_have_a_file = TRUE;
-				}
-			else
-				{
-				if (!unlink_file(fd->path))
-					{
-					DEBUG_1("Failed to remove cache file %s", fd->path);
-					still_have_a_file = TRUE;
-					}
-				}
-			}
-		else
-			{
-			still_have_a_file = TRUE;
-			}
-		g_free(source);
-		}
-
-	filelist_free(list);
-	file_data_unref(cachedir_fd);
-
-	if (recursive)
-		{
-		list = NULL;
-
-		filelist_read(dir_fd, NULL, &list);
-		work = list;
-		while (work)
-			{
-			FileData *fd = work->data;
-			work = work->next;
-
-			still_have_a_file |= cache_maintain_dir(fd->path, recursive, clear);
-			}
-
-		filelist_free(list);
-		}
-
-	return still_have_a_file;
-}
-#endif
-
 static void cache_file_move(const gchar *src, const gchar *dest)
 {
 	if (!dest || !src || !isfile(src)) return;
@@ -626,7 +509,7 @@ static void cache_maint_copied(FileData *fd)
 void cache_notify_cb(FileData *fd, NotifyType type, gpointer data)
 {
 	if (!(type & NOTIFY_CHANGE) || !fd->change) return;
-	
+
 	DEBUG_1("Notify cache_maint: %s %04x", fd->path, type);
 	switch (fd->change->type)
 		{
@@ -693,6 +576,8 @@ struct _CleanData
 	gboolean local;
 	gboolean recurse;
 
+	gboolean remote;
+
 	guint idle_id; /* event source id */
 };
 
@@ -712,11 +597,7 @@ static void cache_manager_render_close_cb(GenericDialog *fd, gpointer data)
 {
 	CleanData *cd = data;
 
-#if GTK_CHECK_VERSION(2,20,0)
 	if (!gtk_widget_get_sensitive(cd->button_close)) return;
-#else
-	if (!GTK_WIDGET_SENSITIVE(cd->button_close)) return;
-#endif
 
 	cache_manager_render_reset(cd);
 	generic_dialog_close(cd->gd);
@@ -726,14 +607,16 @@ static void cache_manager_render_close_cb(GenericDialog *fd, gpointer data)
 static void cache_manager_render_finish(CleanData *cd)
 {
 	cache_manager_render_reset(cd);
+	if (!cd->remote)
+		{
+		gtk_entry_set_text(GTK_ENTRY(cd->progress), _("done"));
+		spinner_set_interval(cd->spinner, -1);
 
-	gtk_entry_set_text(GTK_ENTRY(cd->progress), _("done"));
-	spinner_set_interval(cd->spinner, -1);
-
-	gtk_widget_set_sensitive(cd->group, TRUE);
-	gtk_widget_set_sensitive(cd->button_start, TRUE);
-	gtk_widget_set_sensitive(cd->button_stop, FALSE);
-	gtk_widget_set_sensitive(cd->button_close, TRUE);
+		gtk_widget_set_sensitive(cd->group, TRUE);
+		gtk_widget_set_sensitive(cd->button_start, TRUE);
+		gtk_widget_set_sensitive(cd->button_stop, FALSE);
+		gtk_widget_set_sensitive(cd->button_close, TRUE);
+		}
 }
 
 static void cache_manager_render_stop_cb(GenericDialog *fd, gpointer data)
@@ -795,7 +678,10 @@ static gboolean cache_manager_render_file(CleanData *cd)
 		success = thumb_loader_start((ThumbLoader *)cd->tl, fd);
 		if (success)
 			{
-			gtk_entry_set_text(GTK_ENTRY(cd->progress), fd->path);
+			if (!cd->remote)
+				{
+				gtk_entry_set_text(GTK_ENTRY(cd->progress), fd->path);
+				}
 			}
 		else
 			{
@@ -831,30 +717,62 @@ static void cache_manager_render_start_cb(GenericDialog *fd, gpointer data)
 	CleanData *cd = data;
 	gchar *path;
 
-#if GTK_CHECK_VERSION(2,20,0)
-	if (cd->list || !gtk_widget_get_sensitive(cd->button_start)) return;
-#else
-	if (cd->list || !GTK_WIDGET_SENSITIVE(cd->button_start)) return;
-#endif
+	if(!cd->remote)
+		{
+		if (cd->list || !gtk_widget_get_sensitive(cd->button_start)) return;
+		}
 
 	path = remove_trailing_slash((gtk_entry_get_text(GTK_ENTRY(cd->entry))));
 	parse_out_relatives(path);
 
 	if (!isdir(path))
 		{
-		warning_dialog(_("Invalid folder"),
-				_("The specified folder can not be found."),
-			       GTK_STOCK_DIALOG_WARNING, cd->gd->dialog);
+		if (!cd->remote)
+			{
+			warning_dialog(_("Invalid folder"),
+			_("The specified folder can not be found."),
+			GTK_STOCK_DIALOG_WARNING, cd->gd->dialog);
+			}
+		else
+			{
+			log_printf("The specified folder can not be found: %s\n", path);
+			}
 		}
 	else
 		{
 		FileData *dir_fd;
-		gtk_widget_set_sensitive(cd->group, FALSE);
-		gtk_widget_set_sensitive(cd->button_start, FALSE);
-		gtk_widget_set_sensitive(cd->button_stop, TRUE);
-		gtk_widget_set_sensitive(cd->button_close, FALSE);
+		if(!cd->remote)
+			{
+			gtk_widget_set_sensitive(cd->group, FALSE);
+			gtk_widget_set_sensitive(cd->button_start, FALSE);
+			gtk_widget_set_sensitive(cd->button_stop, TRUE);
+			gtk_widget_set_sensitive(cd->button_close, FALSE);
 
-		spinner_set_interval(cd->spinner, SPINNER_SPEED);
+			spinner_set_interval(cd->spinner, SPINNER_SPEED);
+			}
+		dir_fd = file_data_new_dir(path);
+		cache_manager_render_folder(cd, dir_fd);
+		file_data_unref(dir_fd);
+		while (cache_manager_render_file(cd));
+		}
+
+	g_free(path);
+}
+
+static void cache_manager_render_start_render_remote(CleanData *cd, const gchar *user_path)
+{
+	gchar *path;
+
+	path = remove_trailing_slash(user_path);
+	parse_out_relatives(path);
+
+	if (!isdir(path))
+		{
+		log_printf("The specified folder can not be found: %s\n", path);
+		}
+	else
+		{
+		FileData *dir_fd;
 
 		dir_fd = file_data_new_dir(path);
 		cache_manager_render_folder(cd, dir_fd);
@@ -873,6 +791,7 @@ static void cache_manager_render_dialog(GtkWidget *widget, const gchar *path)
 	GtkWidget *button;
 
 	cd = g_new0(CleanData, 1);
+	cd->remote = FALSE;
 
 	cd->gd = generic_dialog_new(_("Create thumbnails"),
 				    "create_thumbnails",
@@ -888,7 +807,7 @@ static void cache_manager_render_dialog(GtkWidget *widget, const gchar *path)
 						    cache_manager_render_stop_cb, FALSE);
 	gtk_widget_set_sensitive(cd->button_stop, FALSE);
 
-	generic_dialog_add_message(cd->gd, NULL, _("Create thumbnails"), NULL);
+	generic_dialog_add_message(cd->gd, NULL, _("Create thumbnails"), NULL, FALSE);
 
 	hbox = pref_box_new(cd->gd->vbox, FALSE, GTK_ORIENTATION_HORIZONTAL, 0);
 	pref_spacer(hbox, PREF_PAD_INDENT);
@@ -910,7 +829,7 @@ static void cache_manager_render_dialog(GtkWidget *widget, const gchar *path)
 	hbox = pref_box_new(cd->gd->vbox, FALSE, GTK_ORIENTATION_HORIZONTAL, PREF_PAD_SPACE);
 
 	cd->progress = gtk_entry_new();
-	GTK_WIDGET_UNSET_FLAGS(cd->progress, GTK_CAN_FOCUS);
+	gtk_widget_set_can_focus(cd->progress, FALSE);
 	gtk_editable_set_editable(GTK_EDITABLE(cd->progress), FALSE);
 	gtk_entry_set_text(GTK_ENTRY(cd->progress), _("click start to begin"));
 	gtk_box_pack_start(GTK_BOX(hbox), cd->progress, TRUE, TRUE, 0);
@@ -925,18 +844,23 @@ static void cache_manager_render_dialog(GtkWidget *widget, const gchar *path)
 	gtk_widget_show(cd->gd->dialog);
 }
 
+void cache_manager_render_remote(const gchar *path, gboolean recurse, gboolean local)
+{
+	CleanData *cd;
 
+	cd = g_new0(CleanData, 1);
+	cd->recurse = recurse;
+	cd->local = local;
+	cd->remote = TRUE;
 
+	cache_manager_render_start_render_remote(cd, path);
+}
 
 static void cache_manager_standard_clean_close_cb(GenericDialog *gd, gpointer data)
 {
 	CleanData *cd = data;
 
-#if GTK_CHECK_VERSION(2,20,0)
 	if (!gtk_widget_get_sensitive(cd->button_close)) return;
-#else
-	if (!GTK_WIDGET_SENSITIVE(cd->button_close)) return;
-#endif
 
 	generic_dialog_close(cd->gd);
 
@@ -947,12 +871,14 @@ static void cache_manager_standard_clean_close_cb(GenericDialog *gd, gpointer da
 
 static void cache_manager_standard_clean_done(CleanData *cd)
 {
-	gtk_widget_set_sensitive(cd->button_stop, FALSE);
-	gtk_widget_set_sensitive(cd->button_close, TRUE);
-
-	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(cd->progress), 1.0);
-	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(cd->progress), _("done"));
+	if (!cd->remote)
+		{
+		gtk_widget_set_sensitive(cd->button_stop, FALSE);
+		gtk_widget_set_sensitive(cd->button_close, TRUE);
 
+		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(cd->progress), 1.0);
+		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(cd->progress), _("done"));
+		}
 	if (cd->idle_id)
 		{
 		g_source_remove(cd->idle_id);
@@ -990,10 +916,13 @@ static gint cache_manager_standard_clean_clear_cb(gpointer data)
 		file_data_unref(next_fd);
 
 		cd->count_done++;
-		if (cd->count_total != 0)
+		if (!cd->remote)
 			{
-			gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(cd->progress),
-						      (gdouble)cd->count_done / cd->count_total);
+			if (cd->count_total != 0)
+				{
+				gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(cd->progress),
+							      (gdouble)cd->count_done / cd->count_total);
+				}
 			}
 
 		return TRUE;
@@ -1017,10 +946,13 @@ static void cache_manager_standard_clean_valid_cb(const gchar *path, gboolean va
 			}
 
 		cd->count_done++;
-		if (cd->count_total != 0)
+		if (!cd->remote)
 			{
-			gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(cd->progress),
-						      (gdouble)cd->count_done / cd->count_total);
+			if (cd->count_total != 0)
+				{
+				gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(cd->progress),
+							      (gdouble)cd->count_done / cd->count_total);
+				}
 			}
 		}
 
@@ -1042,40 +974,39 @@ static void cache_manager_standard_clean_valid_cb(const gchar *path, gboolean va
 		}
 }
 
-static void cache_manager_standard_clean_start_cb(GenericDialog *gd, gpointer data)
+static void cache_manager_standard_clean_start(GenericDialog *gd, gpointer data)
 {
 	CleanData *cd = data;
 	GList *list;
 	gchar *path;
 	FileData *dir_fd;
 
-#if GTK_CHECK_VERSION(2,20,0)
-	if (cd->list || !gtk_widget_get_sensitive(cd->button_start)) return;
-#else
-	if (cd->list || !GTK_WIDGET_SENSITIVE(cd->button_start)) return;
-#endif
+	if (!cd->remote)
+	{
+		if (cd->list || !gtk_widget_get_sensitive(cd->button_start)) return;
 
-	gtk_widget_set_sensitive(cd->button_start, FALSE);
-	gtk_widget_set_sensitive(cd->button_stop, TRUE);
-	gtk_widget_set_sensitive(cd->button_close, FALSE);
+		gtk_widget_set_sensitive(cd->button_start, FALSE);
+		gtk_widget_set_sensitive(cd->button_stop, TRUE);
+		gtk_widget_set_sensitive(cd->button_close, FALSE);
 
-	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(cd->progress), _("running..."));
+		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(cd->progress), _("running..."));
+	}
 
-	path = g_build_filename(homedir(), THUMB_FOLDER_GLOBAL, THUMB_FOLDER_NORMAL, NULL);
+	path = g_build_filename(get_thumbnails_standard_cache_dir(), THUMB_FOLDER_NORMAL, NULL);
 	dir_fd = file_data_new_dir(path);
 	filelist_read(dir_fd, &list, NULL);
 	cd->list = list;
 	file_data_unref(dir_fd);
 	g_free(path);
 
-	path = g_build_filename(homedir(), THUMB_FOLDER_GLOBAL, THUMB_FOLDER_LARGE, NULL);
+	path = g_build_filename(get_thumbnails_standard_cache_dir(), THUMB_FOLDER_LARGE, NULL);
 	dir_fd = file_data_new_dir(path);
 	filelist_read(dir_fd, &list, NULL);
 	cd->list = g_list_concat(cd->list, list);
 	file_data_unref(dir_fd);
 	g_free(path);
 
-	path = g_build_filename(homedir(), THUMB_FOLDER_GLOBAL, THUMB_FOLDER_FAIL, NULL);
+	path = g_build_filename(get_thumbnails_standard_cache_dir(), THUMB_FOLDER_FAIL, NULL);
 	dir_fd = file_data_new_dir(path);
 	filelist_read(dir_fd, &list, NULL);
 	cd->list = g_list_concat(cd->list, list);
@@ -1096,6 +1027,11 @@ static void cache_manager_standard_clean_start_cb(GenericDialog *gd, gpointer da
 		}
 }
 
+static void cache_manager_standard_clean_start_cb(GenericDialog *gd, gpointer data)
+{
+	cache_manager_standard_clean_start(gd, data);
+}
+
 static void cache_manager_standard_process(GtkWidget *widget, gboolean clear)
 {
 	CleanData *cd;
@@ -1104,6 +1040,7 @@ static void cache_manager_standard_process(GtkWidget *widget, gboolean clear)
 
 	cd = g_new0(CleanData, 1);
 	cd->clear = clear;
+	cd->remote = FALSE;
 
 	if (clear)
 		{
@@ -1129,10 +1066,13 @@ static void cache_manager_standard_process(GtkWidget *widget, gboolean clear)
 						    cache_manager_standard_clean_stop_cb, FALSE);
 	gtk_widget_set_sensitive(cd->button_stop, FALSE);
 
-	generic_dialog_add_message(cd->gd, stock_id, msg, NULL);
+	generic_dialog_add_message(cd->gd, stock_id, msg, NULL, FALSE);
 
 	cd->progress = gtk_progress_bar_new();
 	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(cd->progress), _("click start to begin"));
+#if GTK_CHECK_VERSION(3,0,0)
+	gtk_progress_bar_set_show_text(GTK_PROGRESS_BAR(cd->progress), TRUE);
+#endif
 	gtk_box_pack_start(GTK_BOX(cd->gd->vbox), cd->progress, FALSE, FALSE, 0);
 	gtk_widget_show(cd->progress);
 
@@ -1143,6 +1083,20 @@ static void cache_manager_standard_process(GtkWidget *widget, gboolean clear)
 	gtk_widget_show(cd->gd->dialog);
 }
 
+void cache_manager_standard_process_remote(gboolean clear)
+{
+	CleanData *cd;
+
+	cd = g_new0(CleanData, 1);
+	cd->clear = clear;
+	cd->days = 30;
+	cd->tl = NULL;
+	cd->idle_id = 0;
+	cd->remote = TRUE;
+
+	cache_manager_standard_clean_start(NULL, cd);
+}
+
 static void cache_manager_standard_clean_cb(GtkWidget *widget, gpointer data)
 {
 	cache_manager_standard_process(widget, FALSE);
@@ -1178,7 +1132,7 @@ void cache_manager_main_clear_confirm(GtkWidget *parent)
 				"clear_cache", parent, TRUE,
 				dummy_cancel_cb, NULL);
 	generic_dialog_add_message(gd, GTK_STOCK_DIALOG_QUESTION, _("Clear cache"),
-				   _("This will remove all thumbnails that have\nbeen saved to disk, continue?"));
+				   _("This will remove all thumbnails that have\nbeen saved to disk, continue?"), TRUE);
 	generic_dialog_add_button(gd, GTK_STOCK_OK, NULL, cache_manager_main_clear_ok_cb, TRUE);
 
 	gtk_widget_show(gd->dialog);
@@ -1213,19 +1167,21 @@ static void cache_manager_close_cb(GenericDialog *gd, gpointer data)
 	cache_manager = NULL;
 }
 
+static void cache_manager_help_cb(GenericDialog *gd, gpointer data)
+{
+	help_window_show("GuideReferenceManagement.html");
+}
+
 static GtkWidget *cache_manager_location_label(GtkWidget *group, const gchar *subdir)
 {
 	GtkWidget *label;
 	gchar *buf;
-	gchar *path;
 
-	path = g_build_filename(homedir(), subdir, NULL);
-	buf = g_strdup_printf(_("Location: %s"), path);
-	g_free(path);
+	buf = g_strdup_printf(_("Location: %s"), subdir);
 	label = pref_label_new(group, buf);
 	g_free(buf);
 	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
-	
+
 	return label;
 }
 
@@ -1236,6 +1192,7 @@ void cache_manager_show(void)
 	GtkWidget *button;
 	GtkWidget *table;
 	GtkSizeGroup *sizegroup;
+	gchar *path;
 
 	if (cache_manager)
 		{
@@ -1254,12 +1211,14 @@ void cache_manager_show(void)
 	gd->cancel_cb = cache_manager_close_cb;
 	generic_dialog_add_button(gd, GTK_STOCK_CLOSE, NULL,
 				  cache_manager_close_cb, FALSE);
+	generic_dialog_add_button(gd, GTK_STOCK_HELP, NULL,
+				  cache_manager_help_cb, FALSE);
 
-	generic_dialog_add_message(gd, NULL, _("Cache and Data Maintenance"), NULL);
+	generic_dialog_add_message(gd, NULL, _("Cache and Data Maintenance"), NULL, FALSE);
 
 	sizegroup = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
 
-	group = pref_group_new(gd->vbox, FALSE, _("Thumbnail cache"), GTK_ORIENTATION_VERTICAL);
+	group = pref_group_new(gd->vbox, FALSE, _("Geeqie thumbnail cache"), GTK_ORIENTATION_VERTICAL);
 
 	cache_manager_location_label(group, get_thumbnails_cache_dir());
 
@@ -1273,12 +1232,14 @@ void cache_manager_show(void)
 	button = pref_table_button(table, 0, 1, GTK_STOCK_DELETE, _("Clear cache"), FALSE,
 				   G_CALLBACK(cache_manager_main_clear_cb), cache_manager);
 	gtk_size_group_add_widget(sizegroup, button);
-	pref_table_label(table, 1, 1, _("Delete all cached thumbnails."), 0.0);
+	pref_table_label(table, 1, 1, _("Delete all cached data."), 0.0);
 
 
 	group = pref_group_new(gd->vbox, FALSE, _("Shared thumbnail cache"), GTK_ORIENTATION_VERTICAL);
 
-	cache_manager_location_label(group, THUMB_FOLDER_GLOBAL);
+	path = g_build_filename(get_thumbnails_standard_cache_dir(), NULL);
+	cache_manager_location_label(group, path);
+	g_free(path);
 
 	table = pref_table_new(group, 2, 2, FALSE, FALSE);
 
