@@ -1,22 +1,20 @@
-/* cellrenderericon.c, based on:
+/*
+ * Copyright (C) 2000 Red Hat, Inc., Jonathan Blandford <jrb@redhat.com>
+ * Copyright (C) 2008 - 2016 The Geeqie Team
  *
- * gtkcellrendererpixbuf.c
- * Copyright (C) 2000  Red Hat, Inc.,  Jonathan Blandford <jrb@redhat.com>
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include <stdlib.h>
@@ -27,9 +25,6 @@
 
 #define FIXED_ICON_SIZE_MAX 512
 
-#define TOGGLE_WIDTH 13
-#define TOGGLE_SPACING 18
-
 
 static void gqv_cell_renderer_icon_get_property(GObject		*object,
 						guint		param_id,
@@ -42,13 +37,31 @@ static void gqv_cell_renderer_icon_set_property(GObject		*object,
 static void gqv_cell_renderer_icon_init(GQvCellRendererIcon *celltext);
 static void gqv_cell_renderer_icon_class_init(GQvCellRendererIconClass *class);
 static void gqv_cell_renderer_icon_finalize(GObject *object);
-static void gqv_cell_renderer_icon_get_size(GtkCellRenderer	*cell,
-					    GtkWidget		*widget,
-					    GdkRectangle	*rectangle,
-					    gint		*x_offset,
-					    gint		*y_offset,
-					    gint		*width,
-					    gint		*height);
+#if GTK_CHECK_VERSION(3,0,0)
+static void gqv_cell_renderer_icon_get_size(GtkCellRenderer    *cell,
+					    GtkWidget	       *widget,
+					    const GdkRectangle *rectangle,
+					    gint	       *x_offset,
+					    gint	       *y_offset,
+					    gint	       *width,
+					    gint	       *height);
+#else
+static void gqv_cell_renderer_icon_get_size(GtkCellRenderer    *cell,
+					    GtkWidget	       *widget,
+					    GdkRectangle       *rectangle,
+					    gint	       *x_offset,
+					    gint	       *y_offset,
+					    gint	       *width,
+					    gint	       *height);
+#endif
+#if GTK_CHECK_VERSION(3,0,0)
+static void gqv_cell_renderer_icon_render(GtkCellRenderer *cell,
+					   cairo_t *cr,
+					   GtkWidget *widget,
+					   const GdkRectangle *background_area,
+					   const GdkRectangle *cell_area,
+					   GtkCellRendererState flags);
+#else
 static void gqv_cell_renderer_icon_render(GtkCellRenderer	*cell,
 					   GdkWindow		*window,
 					   GtkWidget		*widget,
@@ -56,14 +69,26 @@ static void gqv_cell_renderer_icon_render(GtkCellRenderer	*cell,
 					   GdkRectangle		*cell_area,
 					   GdkRectangle		*expose_area,
 					   GtkCellRendererState	flags);
+#endif
 
-static gint gqv_cell_renderer_icon_activate(GtkCellRenderer      *cell,
-					    GdkEvent             *event,
-					    GtkWidget            *widget,
-					    const gchar          *path,
-					    GdkRectangle         *background_area,
-					    GdkRectangle         *cell_area,
-					    GtkCellRendererState  flags);
+
+#if GTK_CHECK_VERSION(3,0,0)
+static gboolean gqv_cell_renderer_icon_activate(GtkCellRenderer      *cell,
+						GdkEvent             *event,
+						GtkWidget            *widget,
+						const gchar          *path,
+						const GdkRectangle   *background_area,
+						const GdkRectangle   *cell_area,
+						GtkCellRendererState  flags);
+#else
+static gboolean gqv_cell_renderer_icon_activate(GtkCellRenderer      *cell,
+						GdkEvent             *event,
+						GtkWidget            *widget,
+						const gchar          *path,
+						GdkRectangle         *background_area,
+						GdkRectangle         *cell_area,
+						GtkCellRendererState  flags);
+#endif
 
 enum {
   TOGGLED,
@@ -125,9 +150,8 @@ gqv_cell_renderer_icon_get_type(void)
 static void
 gqv_cell_renderer_icon_init(GQvCellRendererIcon *cellicon)
 {
-	GTK_CELL_RENDERER(cellicon)->mode = GTK_CELL_RENDERER_MODE_ACTIVATABLE;
-	GTK_CELL_RENDERER(cellicon)->xpad = 2;
-	GTK_CELL_RENDERER(cellicon)->ypad = 2;
+	g_object_set(G_OBJECT(cellicon), "mode", GTK_CELL_RENDERER_MODE_ACTIVATABLE, NULL);
+	gtk_cell_renderer_set_padding(GTK_CELL_RENDERER(cellicon), 2, 2);
 }
 
 static void
@@ -541,18 +565,32 @@ gqv_cell_renderer_icon_new(void)
 	return g_object_new(GQV_TYPE_CELL_RENDERER_ICON, NULL);
 }
 
-static void
-gqv_cell_renderer_icon_get_size(GtkCellRenderer *cell,
-				GtkWidget	*widget,
-				GdkRectangle	*cell_area,
-				gint		*x_offset,
-				gint		*y_offset,
-				gint		*width,
-				gint		*height)
+#if GTK_CHECK_VERSION(3,0,0)
+static void gqv_cell_renderer_icon_get_size(GtkCellRenderer    *cell,
+					    GtkWidget          *widget,
+					    const GdkRectangle *cell_area,
+					    gint	       *x_offset,
+					    gint	       *y_offset,
+					    gint	       *width,
+					    gint	       *height)
+#else
+static void gqv_cell_renderer_icon_get_size(GtkCellRenderer    *cell,
+					    GtkWidget	       *widget,
+					    GdkRectangle       *cell_area,
+					    gint	       *x_offset,
+					    gint	       *y_offset,
+					    gint	       *width,
+					    gint	       *height)
+#endif
 {
 	GQvCellRendererIcon *cellicon = (GQvCellRendererIcon *) cell;
 	gint calc_width;
 	gint calc_height;
+	gint xpad, ypad;
+	gfloat xalign, yalign;
+
+	gtk_cell_renderer_get_padding(cell, &xpad, &ypad);
+	gtk_cell_renderer_get_alignment(cell, &xalign, &yalign);
 
 	if (cellicon->fixed_width > 0)
 		{
@@ -591,8 +629,8 @@ gqv_cell_renderer_icon_get_size(GtkCellRenderer *cell,
 		calc_width = MAX(calc_width, TOGGLE_SPACING * cellicon->num_marks);
 		}
 
-	calc_width += (gint)cell->xpad * 2;
-	calc_height += (gint)cell->ypad * 2;
+	calc_width += xpad * 2;
+	calc_height += ypad * 2;
 
 	if (x_offset) *x_offset = 0;
 	if (y_offset) *y_offset = 0;
@@ -601,13 +639,13 @@ gqv_cell_renderer_icon_get_size(GtkCellRenderer *cell,
 		{
 		if (x_offset)
 			{
-			*x_offset = (cell->xalign * (cell_area->width - calc_width - 2 * cell->xpad));
-			*x_offset = MAX(*x_offset, 0) + cell->xpad;
+			*x_offset = (xalign * (cell_area->width - calc_width - 2 * xpad));
+			*x_offset = MAX(*x_offset, 0) + xpad;
 			}
 		if (y_offset)
 			{
-			*y_offset = (cell->yalign * (cell_area->height - calc_height - 2 * cell->ypad));
-			*y_offset = MAX(*y_offset, 0) + cell->ypad;
+			*y_offset = (yalign * (cell_area->height - calc_height - 2 * ypad));
+			*y_offset = MAX(*y_offset, 0) + ypad;
 			}
 		}
 
@@ -615,6 +653,17 @@ gqv_cell_renderer_icon_get_size(GtkCellRenderer *cell,
 	if (height) *height = calc_height;
 }
 
+#if GTK_CHECK_VERSION(3,0,0)
+static void gqv_cell_renderer_icon_render(GtkCellRenderer *cell,
+					   cairo_t *cr,
+					   GtkWidget *widget,
+					   const GdkRectangle *background_area,
+					   const GdkRectangle *cell_area,
+					   GtkCellRendererState flags)
+
+{
+	GtkStyleContext *context = gtk_widget_get_style_context(widget);
+#else
 static void
 gqv_cell_renderer_icon_render(GtkCellRenderer		*cell,
 			      GdkWindow			*window,
@@ -625,49 +674,73 @@ gqv_cell_renderer_icon_render(GtkCellRenderer		*cell,
 			      GtkCellRendererState	flags)
 
 {
+	cairo_t *cr = gdk_cairo_create(window);
+#endif
 	GQvCellRendererIcon *cellicon = (GQvCellRendererIcon *) cell;
 	GdkPixbuf *pixbuf;
 	const gchar *text;
 	GdkRectangle cell_rect;
+#if GTK_CHECK_VERSION(3,0,0)
+	GtkStateFlags state;
+#else
 	GtkStateType state;
+#endif
+	gint xpad, ypad;
+
 
 	pixbuf = cellicon->pixbuf;
 	text = cellicon->text;
 
-	if (!pixbuf && !text) return;
+	if (!pixbuf && !text)
+		{
+#if !GTK_CHECK_VERSION(3,0,0)
+		cairo_destroy(cr);
+#endif
+		return;
+		}
+
+	gtk_cell_renderer_get_padding(cell, &xpad, &ypad);
 
 	gqv_cell_renderer_icon_get_size(cell, widget, cell_area,
 					&cell_rect.x, &cell_rect.y,
 					&cell_rect.width, &cell_rect.height);
 
-	cell_rect.x += cell->xpad;
-	cell_rect.y += cell->ypad;
-	cell_rect.width -= cell->xpad * 2;
-	cell_rect.height -= cell->ypad * 2;
+	cell_rect.x += xpad;
+	cell_rect.y += ypad;
+	cell_rect.width -= xpad * 2;
+	cell_rect.height -= ypad * 2;
 
 	if ((flags & GTK_CELL_RENDERER_SELECTED) == GTK_CELL_RENDERER_SELECTED)
 		{
-#if GTK_CHECK_VERSION(2,20,0)
 		if (gtk_widget_has_focus(widget))
+#if GTK_CHECK_VERSION(3,0,0)
+			state = GTK_STATE_FLAG_SELECTED;
+		else
+			state = GTK_STATE_FLAG_ACTIVE;
 #else
-		if (GTK_WIDGET_HAS_FOCUS(widget))
-#endif
 			state = GTK_STATE_SELECTED;
 		else
 			state = GTK_STATE_ACTIVE;
+#endif
 		}
 	else
 		{
-#if GTK_CHECK_VERSION(2,20,0)
 		if (gtk_widget_get_state(widget) == GTK_STATE_INSENSITIVE)
+#if GTK_CHECK_VERSION(3,0,0)
+			state = GTK_STATE_FLAG_INSENSITIVE;
+		else
+			state = GTK_STATE_FLAG_NORMAL;
 #else
-		if (GTK_WIDGET_STATE(widget) == GTK_STATE_INSENSITIVE)
-#endif
 			state = GTK_STATE_INSENSITIVE;
 		else
 			state = GTK_STATE_NORMAL;
+#endif
 		}
 
+#if GTK_CHECK_VERSION(3,0,0)
+	gtk_style_context_set_state(context, state);
+#endif
+
 	if (pixbuf)
 		{
 		GdkRectangle pix_rect;
@@ -680,28 +753,27 @@ gqv_cell_renderer_icon_render(GtkCellRenderer		*cell,
 
 		if (cellicon->fixed_height > 0)
 			{
-			pix_rect.y = cell_area->y + cell->ypad + (cellicon->fixed_height - pix_rect.height) / 2;
+			pix_rect.y = cell_area->y + ypad + (cellicon->fixed_height - pix_rect.height) / 2;
 			}
 		else
 			{
 			pix_rect.y = cell_area->y + cell_rect.y;
 			}
 
-		if (gdk_rectangle_intersect(cell_area, &pix_rect, &draw_rect) &&
-		    gdk_rectangle_intersect(expose_area, &draw_rect, &draw_rect))
+		if (gdk_rectangle_intersect(cell_area, &pix_rect, &draw_rect)
+#if !GTK_CHECK_VERSION(3,0,0)
+		    && gdk_rectangle_intersect(expose_area, &draw_rect, &draw_rect)
+#endif
+		   )
 			{
-			gdk_draw_pixbuf(window,
-					widget->style->black_gc,
-					pixbuf,
-					/* pixbuf 0, 0 is at pix_rect.x, pix_rect.y */
-					draw_rect.x - pix_rect.x,
-					draw_rect.y - pix_rect.y,
+			gdk_cairo_set_source_pixbuf(cr, pixbuf, pix_rect.x, pix_rect.y);
+			cairo_rectangle (cr,
 					draw_rect.x,
 					draw_rect.y,
 					draw_rect.width,
-					draw_rect.height,
-					GDK_RGB_DITHER_NORMAL,
-					0, 0);
+					draw_rect.height);
+
+			cairo_fill (cr);
 			}
 		}
 
@@ -711,31 +783,37 @@ gqv_cell_renderer_icon_render(GtkCellRenderer		*cell,
 		PangoRectangle text_rect;
 		GdkRectangle pix_rect;
 		GdkRectangle draw_rect;
-
 		layout = gqv_cell_renderer_icon_get_layout(cellicon, widget, TRUE);
 		pango_layout_get_pixel_extents(layout, NULL, &text_rect);
 
 		pix_rect.width = text_rect.width;
 		pix_rect.height = text_rect.height;
-		pix_rect.x = cell_area->x + cell->xpad + (cell_rect.width - text_rect.width + 1) / 2;
-		pix_rect.y = cell_area->y + cell->ypad + (cell_rect.height - text_rect.height);
-		
+		pix_rect.x = cell_area->x + xpad + (cell_rect.width - text_rect.width + 1) / 2;
+		pix_rect.y = cell_area->y + ypad + (cell_rect.height - text_rect.height);
+
 		if (cellicon->show_marks)
 			{
 			pix_rect.y -= TOGGLE_SPACING;
 			}
 
-		if (gdk_rectangle_intersect(cell_area, &pix_rect, &draw_rect) &&
-		    gdk_rectangle_intersect(expose_area, &draw_rect, &draw_rect))
+		if (gdk_rectangle_intersect(cell_area, &pix_rect, &draw_rect)
+#if !GTK_CHECK_VERSION(3,0,0)
+		    && gdk_rectangle_intersect(expose_area, &draw_rect, &draw_rect)
+#endif
+		    )
 			{
-			gtk_paint_layout(widget->style, window,
+#if GTK_CHECK_VERSION(3,0,0)
+			gtk_render_layout(context, cr, pix_rect.x - text_rect.x, pix_rect.y, layout);
+#else
+
+			gtk_paint_layout(gtk_widget_get_style(widget), window,
 					 state, TRUE,
 					 cell_area, widget,
 					 "cellrenderertext",
 					 pix_rect.x - text_rect.x, pix_rect.y,
 					 layout);
+#endif
 			}
-
 		g_object_unref(layout);
 		}
 
@@ -747,47 +825,117 @@ gqv_cell_renderer_icon_render(GtkCellRenderer		*cell,
 
 		pix_rect.width = TOGGLE_SPACING * cellicon->num_marks;
 		pix_rect.height = TOGGLE_SPACING;
-		pix_rect.x = cell_area->x + cell->xpad + (cell_rect.width - pix_rect.width + 1) / 2 + (TOGGLE_SPACING - TOGGLE_WIDTH) / 2;
-		pix_rect.y = cell_area->y + cell->ypad + (cell_rect.height - pix_rect.height) + (TOGGLE_SPACING - TOGGLE_WIDTH) / 2;
-		
-		if (gdk_rectangle_intersect(cell_area, &pix_rect, &draw_rect) &&
-		    gdk_rectangle_intersect(expose_area, &draw_rect, &draw_rect))
+		pix_rect.x = cell_area->x + xpad + (cell_rect.width - pix_rect.width + 1) / 2 + (TOGGLE_SPACING - TOGGLE_WIDTH) / 2;
+		pix_rect.y = cell_area->y + ypad + (cell_rect.height - pix_rect.height) + (TOGGLE_SPACING - TOGGLE_WIDTH) / 2;
+
+		if (gdk_rectangle_intersect(cell_area, &pix_rect, &draw_rect)
+#if !GTK_CHECK_VERSION(3,0,0)
+		    && gdk_rectangle_intersect(expose_area, &draw_rect, &draw_rect)
+#endif
+		    )
 			{
 			for (i = 0; i < cellicon->num_marks; i++)
 				{
-				gtk_paint_check(widget->style, window,
+#if GTK_CHECK_VERSION(3,0,0)
+#if GTK_CHECK_VERSION(3,14,0)
+  				state &= ~(GTK_STATE_FLAG_CHECKED);
+
+				if ((cellicon->marks & (1 << i)))
+					state |= GTK_STATE_FLAG_CHECKED;
+#else
+  				state &= ~(GTK_STATE_FLAG_ACTIVE);
+
+				if ((cellicon->marks & (1 << i)))
+					state |= GTK_STATE_FLAG_ACTIVE;
+#endif
+				cairo_save (cr);
+
+				cairo_rectangle(cr,
+						pix_rect.x + i * TOGGLE_SPACING + (TOGGLE_WIDTH - TOGGLE_SPACING) / 2,
+						pix_rect.y,
+						TOGGLE_WIDTH, TOGGLE_WIDTH);
+				cairo_clip (cr);
+
+				gtk_style_context_save(context);
+				gtk_style_context_set_state(context, state);
+
+				gtk_style_context_add_class(context, GTK_STYLE_CLASS_CHECK);
+
+				gtk_style_context_add_class(context, "marks");
+				GtkStyleProvider *provider;
+				provider = (GtkStyleProvider *)gtk_css_provider_new();
+				gtk_css_provider_load_from_data(GTK_CSS_PROVIDER(provider),
+						".marks {\n"
+						"border-color: #808080;\n"
+						"border-style: solid;\n"
+						"border-width: 1px;\n"
+						"border-radius: 0px;\n"
+						"}\n"
+						,-1, NULL);
+				gtk_style_context_add_provider(context, provider,
+							GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+
+				gtk_render_check(context, cr,
+					 pix_rect.x + i * TOGGLE_SPACING + (TOGGLE_WIDTH - TOGGLE_SPACING) / 2,
+					 pix_rect.y,
+					 TOGGLE_WIDTH, TOGGLE_WIDTH);
+				gtk_render_frame(context, cr,
+					 pix_rect.x + i * TOGGLE_SPACING + (TOGGLE_WIDTH - TOGGLE_SPACING) / 2,
+					 pix_rect.y,
+					 TOGGLE_WIDTH, TOGGLE_WIDTH);
+
+				if (cellicon->focused && gtk_widget_has_focus(widget))
+					{
+					gtk_render_focus(context, cr,
+						pix_rect.x + i * TOGGLE_SPACING + (TOGGLE_WIDTH - TOGGLE_SPACING) / 2,
+						pix_rect.y, TOGGLE_WIDTH, TOGGLE_WIDTH);
+					}
+				gtk_style_context_restore(context);
+				cairo_restore(cr);
+
+#else
+				gtk_paint_check(gtk_widget_get_style(widget), window,
 					 state, (cellicon->marks & (1 << i)) ? GTK_SHADOW_IN : GTK_SHADOW_OUT,
 					 cell_area, widget, "cellcheck",
 					 pix_rect.x + i * TOGGLE_SPACING + (TOGGLE_WIDTH - TOGGLE_SPACING) / 2,
 					 pix_rect.y,
 					 TOGGLE_WIDTH, TOGGLE_WIDTH);
+#endif
 				}
 			}
-                }
+		}
 
-#if GTK_CHECK_VERSION(2,20,0)
+#if !GTK_CHECK_VERSION(3,0,0)
 	if (cellicon->focused && gtk_widget_has_focus(widget))
-#else
-	if (cellicon->focused && GTK_WIDGET_HAS_FOCUS(widget))
-#endif
 		{
-		gtk_paint_focus(widget->style, window,
+		gtk_paint_focus(gtk_widget_get_style(widget), window,
 				state,
 				cell_area, widget,
 				"cellrendererfocus",
 				cell_area->x, cell_area->y,
 				cell_area->width, cell_area->height);
 		}
+	cairo_destroy(cr);
+#endif
 }
 
-static gint
-gqv_cell_renderer_icon_activate(GtkCellRenderer      *cell,
-			 	GdkEvent             *event,
-				GtkWidget            *widget,
-				const gchar          *path,
-				GdkRectangle         *background_area,
-				GdkRectangle         *cell_area,
-				GtkCellRendererState  flags)
+#if GTK_CHECK_VERSION(3,0,0)
+static gboolean gqv_cell_renderer_icon_activate(GtkCellRenderer      *cell,
+						GdkEvent             *event,
+						GtkWidget            *widget,
+						const gchar          *path,
+						const GdkRectangle   *background_area,
+						const GdkRectangle   *cell_area,
+						GtkCellRendererState  flags)
+#else
+static gboolean gqv_cell_renderer_icon_activate(GtkCellRenderer      *cell,
+						GdkEvent             *event,
+						GtkWidget            *widget,
+						const gchar          *path,
+						GdkRectangle	     *background_area,
+						GdkRectangle	     *cell_area,
+						GtkCellRendererState  flags)
+#endif
 {
 	GQvCellRendererIcon *cellicon = (GQvCellRendererIcon *) cell;
 	GdkEventButton *bevent = &event->button;
@@ -800,23 +948,26 @@ gqv_cell_renderer_icon_activate(GtkCellRenderer      *cell,
 		GdkRectangle rect;
 		GdkRectangle cell_rect;
 		gint i;
-		
+		gint xpad, ypad;
+
+		gtk_cell_renderer_get_padding(cell, &xpad, &ypad);
+
 		gqv_cell_renderer_icon_get_size(cell, widget, cell_area,
 						&cell_rect.x, &cell_rect.y,
 						&cell_rect.width, &cell_rect.height);
 
-		cell_rect.x += cell->xpad;
-		cell_rect.y += cell->ypad;
-		cell_rect.width -= cell->xpad * 2;
-		cell_rect.height -= cell->ypad * 2;
+		cell_rect.x += xpad;
+		cell_rect.y += ypad;
+		cell_rect.width -= xpad * 2;
+		cell_rect.height -= ypad * 2;
 
 		rect.width = TOGGLE_WIDTH;
 		rect.height = TOGGLE_WIDTH;
-		rect.y = cell_area->y + cell->ypad + (cell_rect.height - TOGGLE_SPACING) + (TOGGLE_SPACING - TOGGLE_WIDTH) / 2;
+		rect.y = cell_area->y + ypad + (cell_rect.height - TOGGLE_SPACING) + (TOGGLE_SPACING - TOGGLE_WIDTH) / 2;
 		for (i = 0; i < cellicon->num_marks; i++)
 			{
-			rect.x = cell_area->x + cell->xpad + (cell_rect.width - TOGGLE_SPACING * cellicon->num_marks + 1) / 2 + i * TOGGLE_SPACING;
-			
+			rect.x = cell_area->x + xpad + (cell_rect.width - TOGGLE_SPACING * cellicon->num_marks + 1) / 2 + i * TOGGLE_SPACING;
+
 			if (bevent->x >= rect.x && bevent->x < rect.x + rect.width &&
 			    bevent->y >= rect.y && bevent->y < rect.y + rect.height)
 				{
