@@ -1,12 +1,21 @@
 /*
- * Geeqie
- * Copyright (C) 2008 - 2010 The Geeqie Team
+ * Copyright (C) 2008 - 2016 The Geeqie Team
  *
  * Author: Vladimir Nadvornik
  *
- * This software is released under the GNU General Public License (GNU GPL).
- * Please read the included file COPYING for more information.
- * This software comes with no warranty of any kind, use at your own risk!
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include "config.h"
@@ -16,6 +25,7 @@
 #include <exiv2/image.hpp>
 #include <exiv2/exif.hpp>
 #include <iostream>
+#include <string>
 
 // EXIV2_TEST_VERSION is defined in Exiv2 0.15 and newer.
 #ifndef EXIV2_TEST_VERSION
@@ -101,13 +111,18 @@ static const AltKey alt_keys[] = {
 	{NULL, NULL, NULL}
 	};
 
-static void debug_exception(Exiv2::AnyError& e)
+static void _debug_exception(const char* file,
+                             int line,
+                             const char* func,
+                             Exiv2::AnyError& e)
 {
 	gchar *str = g_locale_from_utf8(e.what(), -1, NULL, NULL, NULL);
-	DEBUG_1("Exiv2: %s", str);
+	DEBUG_1("%s:%d:%s:Exiv2: %s", file, line, func, str);
 	g_free(str);
 }
 
+#define debug_exception(e) _debug_exception(__FILE__, __LINE__, __func__, e)
+
 struct _ExifData
 {
 	Exiv2::ExifData::const_iterator exifIter; /* for exif_get_next_item */
@@ -119,7 +134,7 @@ struct _ExifData
 	virtual ~_ExifData()
 	{
 	}
-	
+
 	virtual void writeMetadata(gchar *path = NULL)
 	{
 		g_critical("Unsupported method of writing metadata");
@@ -131,7 +146,7 @@ struct _ExifData
 	}
 
 	virtual Exiv2::Image *image() = 0;
-	
+
 	virtual Exiv2::ExifData &exifData() = 0;
 
 	virtual Exiv2::IptcData &iptcData() = 0;
@@ -143,6 +158,10 @@ struct _ExifData
 	virtual void add_jpeg_color_profile(unsigned char *cp_data, guint cp_length) = 0;
 
 	virtual guchar *get_jpeg_color_profile(guint *data_len) = 0;
+
+	virtual std::string image_comment() const = 0;
+
+	virtual void set_image_comment(const std::string& comment) = 0;
 };
 
 // This allows read-only access to the original metadata
@@ -155,6 +174,7 @@ protected:
 	unsigned char *cp_data_;
 	guint cp_length_;
 	gboolean valid_;
+	gchar *pathl_;
 
 	Exiv2::ExifData emptyExifData_;
 	Exiv2::IptcData emptyIptcData_;
@@ -176,11 +196,11 @@ public:
 		cp_data_ = NULL;
 		cp_length_ = 0;
 		valid_ = TRUE;
-		
-		gchar *pathl = path_from_utf8(path);
-		try 
+
+		pathl_ = path_from_utf8(path);
+		try
 			{
-			image_ = Exiv2::ImageFactory::open(pathl);
+			image_ = Exiv2::ImageFactory::open(pathl_);
 //			g_assert (image.get() != 0);
 			image_->readMetadata();
 
@@ -210,24 +230,24 @@ public:
 				}
 #endif
 			}
-		catch (Exiv2::AnyError& e) 
+		catch (Exiv2::AnyError& e)
 			{
 			valid_ = FALSE;
 			}
-		g_free(pathl);
 	}
-	
+
 	virtual ~_ExifDataOriginal()
 	{
 		if (cp_data_) g_free(cp_data_);
+		if (pathl_) g_free(pathl_);
 	}
-	
+
 	virtual Exiv2::Image *image()
 	{
 		if (!valid_) return NULL;
 		return image_.get();
 	}
-	
+
 	virtual Exiv2::ExifData &exifData ()
 	{
 		if (!valid_) return emptyExifData_;
@@ -264,6 +284,17 @@ public:
 		}
 		return NULL;
 	}
+
+	virtual std::string image_comment() const
+	{
+		return image_.get() ? image_->comment() : "";
+	}
+
+	virtual void set_image_comment(const std::string& comment)
+	{
+		if (image_.get())
+			image_->setComment(comment);
+	}
 };
 
 extern "C" {
@@ -334,7 +365,7 @@ public:
 		if (!path)
 			{
 #if EXIV2_TEST_VERSION(0,17,0)
-			if (options->metadata.save_legacy_IPTC) 
+			if (options->metadata.save_legacy_IPTC)
 				copyXmpToIptc(xmpData_, iptcData_);
 			else
 				iptcData_.clear();
@@ -342,7 +373,7 @@ public:
 			copyXmpToExif(xmpData_, exifData_);
 #endif
 			Exiv2::Image *image = imageData_->image();
-			
+
 			if (!image) Exiv2::Error(21);
 			image->setExifData(exifData_);
 			image->setIptcData(iptcData_);
@@ -350,14 +381,14 @@ public:
 			image->setXmpData(xmpData_);
 #endif
 			image->writeMetadata();
-			} 
+			}
 		else
 			{
 #if EXIV2_TEST_VERSION(0,17,0)
 			gchar *pathl = path_from_utf8(path);;
 
 			Exiv2::Image::AutoPtr sidecar = Exiv2::ImageFactory::create(Exiv2::ImageType::xmp, pathl);
-				
+
 			g_free(pathl);
 
 			sidecar->setXmpData(xmpData_);
@@ -367,12 +398,12 @@ public:
 #endif
 			}
 	}
-	
+
 	virtual Exiv2::Image *image()
 	{
 		return imageData_->image();
 	}
-	
+
 	virtual Exiv2::ExifData &exifData ()
 	{
 		return exifData_;
@@ -399,6 +430,16 @@ public:
 	{
 		return imageData_->get_jpeg_color_profile(data_len);
 	}
+
+	virtual std::string image_comment() const
+	{
+		return imageData_->image_comment();
+	}
+
+	virtual void set_image_comment(const std::string& comment)
+	{
+		imageData_->set_image_comment(comment);
+	}
 };
 
 
@@ -431,7 +472,7 @@ ExifData *exif_read(gchar *path, gchar *sidecar_path, GHashTable *modified_xmp)
 		debug_exception(e);
 		return NULL;
 	}
-	
+
 }
 
 gboolean exif_write(ExifData *exif)
@@ -456,7 +497,7 @@ gboolean exif_write_sidecar(ExifData *exif, gchar *path)
 		debug_exception(e);
 		return FALSE;
 	}
-	
+
 }
 
 
@@ -574,7 +615,7 @@ ExifItem *exif_get_first_item(ExifData *exif)
 			}
 #endif
 		return NULL;
-			
+
 	}
 	catch (Exiv2::AnyError& e) {
 		debug_exception(e);
@@ -708,8 +749,8 @@ static guint format_id_trans_tbl [] = {
 	EXIF_FORMAT_STRING,
 	EXIF_FORMAT_STRING
 	};
-	
-	
+
+
 
 guint exif_item_get_format_id(ExifItem *item)
 {
@@ -855,7 +896,7 @@ gchar *exif_get_tag_description_by_key(const gchar *key)
 static const AltKey *find_alt_key(const gchar *xmp_key)
 {
 	gint i = 0;
-	
+
 	while (alt_keys[i].xmp_key)
 		{
 		if (strcmp(xmp_key, alt_keys[i].xmp_key) == 0) return &alt_keys[i];
@@ -871,7 +912,7 @@ static gint exif_update_metadata_simple(ExifData *exif, const gchar *key, const
 
 		try {
 			Exiv2::ExifKey ekey(key);
-			
+
 			Exiv2::ExifData::iterator pos = exif->exifData().findKey(ekey);
 			while (pos != exif->exifData().end())
 				{
@@ -887,7 +928,7 @@ static gint exif_update_metadata_simple(ExifData *exif, const gchar *key, const
 		}
 		catch (Exiv2::AnyError& e) {
 #if EXIV2_TEST_VERSION(0,16,0)
-			try 
+			try
 #endif
 			{
 				Exiv2::IptcKey ekey(key);
@@ -933,7 +974,7 @@ static gint exif_update_metadata_simple(ExifData *exif, const gchar *key, const
 gint exif_update_metadata(ExifData *exif, const gchar *key, const GList *values)
 {
 	gint ret = exif_update_metadata_simple(exif, key, values);
-	
+
 	if (
 #if !EXIV2_TEST_VERSION(0,17,0)
 	    TRUE || /* no conversion support */
@@ -970,17 +1011,17 @@ static GList *exif_add_value_to_glist(GList *list, Exiv2::Metadatum &item, Metad
 	    id == Exiv2::comment
 	    )
 		{
-#endif 
+#endif
 		/* read as a single entry */
 		std::string str;
-		
+
 		if (format == METADATA_FORMATTED)
 			{
 #if EXIV2_TEST_VERSION(0,17,0)
 			str = item.print(
 #if EXIV2_TEST_VERSION(0,18,0)
 					metadata
-#endif 
+#endif
 					);
 #else
 			std::stringstream stream;
@@ -1015,7 +1056,7 @@ static GList *exif_add_value_to_glist(GList *list, Exiv2::Metadatum &item, Metad
 			std::string::size_type pos = str.find_first_of(' ');
 			if (pos != std::string::npos) str = str.substr(pos+1);
 			}
-		list = g_list_append(list, utf8_validate_or_convert(str.c_str())); 
+		list = g_list_append(list, utf8_validate_or_convert(str.c_str()));
 #if EXIV2_TEST_VERSION(0,16,0)
 		}
 	else
@@ -1023,9 +1064,9 @@ static GList *exif_add_value_to_glist(GList *list, Exiv2::Metadatum &item, Metad
 		/* read as a list */
 		gint i;
 		for (i = 0; i < item.count(); i++)
-			list = g_list_append(list, utf8_validate_or_convert(item.toString(i).c_str())); 
+			list = g_list_append(list, utf8_validate_or_convert(item.toString(i).c_str()));
 		}
-#endif 
+#endif
 	return list;
 }
 
@@ -1081,9 +1122,9 @@ GList *exif_get_metadata(ExifData *exif, const gchar *key, MetadataFormat format
 		text = exif_get_formatted_by_key(exif, key, &key_valid);
 		if (key_valid) return g_list_append(NULL, text);
 		}
-		
+
 	list = exif_get_metadata_simple(exif, key, format);
-	
+
 	/* the following code can be ifdefed out as soon as Exiv2 supports it */
 	if (!list)
 		{
@@ -1091,7 +1132,7 @@ GList *exif_get_metadata(ExifData *exif, const gchar *key, MetadataFormat format
 		if (alt_key && alt_key->iptc_key)
 			list = exif_get_metadata_simple(exif, alt_key->iptc_key, format);
 
-#if !EXIV2_TEST_VERSION(0,17,0)	
+#if !EXIV2_TEST_VERSION(0,17,0)
 		/* with older Exiv2 versions exif is not synced */
 		if (!list && alt_key && alt_key->exif_key)
 			list = exif_get_metadata_simple(exif, alt_key->exif_key, format);
@@ -1117,6 +1158,23 @@ guchar *exif_get_color_profile(ExifData *exif, guint *data_len)
 	return ret;
 }
 
+gchar* exif_get_image_comment(FileData* fd)
+{
+	if (!fd || !fd->exif)
+		return g_strdup("");
+
+	return g_strdup(fd->exif->image_comment().c_str());
+}
+
+void exif_set_image_comment(FileData* fd, const gchar* comment)
+{
+	if (!fd || !fd->exif)
+		return;
+
+	fd->exif->set_image_comment(comment ? comment : "");
+}
+
+
 #if EXIV2_TEST_VERSION(0,17,90)
 
 guchar *exif_get_preview(ExifData *exif, guint *data_len, gint requested_width, gint requested_height)
@@ -1124,11 +1182,11 @@ guchar *exif_get_preview(ExifData *exif, guint *data_len, gint requested_width,
 	if (!exif) return NULL;
 
 	if (!exif->image()) return NULL;
-	
-	const char* path = exif->image()->io().path().c_str();
+
+	std::string const path = exif->image()->io().path();
 	/* given image pathname, first do simple (and fast) file extension test */
-	gboolean is_raw = filter_file_class(path, FORMAT_CLASS_RAWIMAGE);
-	
+	gboolean is_raw = filter_file_class(path.c_str(), FORMAT_CLASS_RAWIMAGE);
+
 	if (!is_raw && requested_width == 0) return NULL;
 
 	try {
@@ -1141,7 +1199,7 @@ guchar *exif_get_preview(ExifData *exif, guint *data_len, gint requested_width,
 			{
 			Exiv2::PreviewPropertiesList::iterator pos;
 			Exiv2::PreviewPropertiesList::iterator last = --list.end();
-			
+
 			if (requested_width == 0)
 				{
 				pos = last; // the largest
@@ -1155,11 +1213,11 @@ guchar *exif_get_preview(ExifData *exif, guint *data_len, gint requested_width,
 					    pos->height_ >= (uint32_t)requested_height) break;
 					++pos;
 					}
-				
+
 				// we are not interested in smaller thumbnails in normal image formats - we can use full image instead
-				if (!is_raw) 
+				if (!is_raw)
 					{
-					if (pos->width_ < (uint32_t)requested_width || pos->height_ < (uint32_t)requested_height) return NULL; 
+					if (pos->width_ < (uint32_t)requested_width || pos->height_ < (uint32_t)requested_height) return NULL;
 					}
 				}
 
@@ -1227,10 +1285,10 @@ extern "C" guchar *exif_get_preview(ExifData *exif, guint *data_len, gint reques
 	if (!exif) return NULL;
 	if (!exif->image()) return NULL;
 
-	const char* path = exif->image()->io().path().c_str();
+	std::string const path = exif->image()->io().path();
 
 	/* given image pathname, first do simple (and fast) file extension test */
-	if (!filter_file_class(path, FORMAT_CLASS_RAWIMAGE)) return NULL;
+	if (!filter_file_class(path.c_str(), FORMAT_CLASS_RAWIMAGE)) return NULL;
 
 	try {
 		struct stat st;
@@ -1238,12 +1296,12 @@ extern "C" guchar *exif_get_preview(ExifData *exif, guint *data_len, gint reques
 		size_t map_len;
 		UnmapData *ud;
 		int fd;
-		
+
 		RawFile rf(exif->image()->io());
 		offset = rf.preview_offset();
-		DEBUG_1("%s: offset %lu", path, offset);
-		
-		fd = open(path, O_RDONLY);
+		DEBUG_1("%s: offset %lu", path.c_str(), offset);
+
+		fd = open(path.c_str(), O_RDONLY);
 		if (fd == -1)
 			{
 			return NULL;
@@ -1266,10 +1324,10 @@ extern "C" guchar *exif_get_preview(ExifData *exif, guint *data_len, gint reques
 		ud->ptr = map_data + offset;
 		ud->map_data = map_data;
 		ud->map_len = map_len;
-		
+
 		exif_unmap_list = g_list_prepend(exif_unmap_list, ud);
 		return ud->ptr;
-		
+
 	}
 	catch (Exiv2::AnyError& e) {
 		debug_exception(e);
@@ -1281,7 +1339,7 @@ extern "C" guchar *exif_get_preview(ExifData *exif, guint *data_len, gint reques
 void exif_free_preview(guchar *buf)
 {
 	GList *work = exif_unmap_list;
-	
+
 	while (work)
 		{
 		UnmapData *ud = (UnmapData *)work->data;
@@ -1317,7 +1375,7 @@ RawFile::RawFile(BasicIo &io) : io_(io), map_data(NULL), map_len(0), offset(0)
         if (io.open() != 0) {
             throw Error(9, io.path(), strError());
         }
-        
+
         map_data = io.mmap();
         map_len = io.size();
 
@@ -1371,7 +1429,7 @@ RawFile::RawFile(BasicIo &io) : io_(io), map_data(NULL), map_len(0), offset(0)
 	if (0 == rootDir.get()) {
 		throw Error(1, "No root element defined in TIFF structure");
 	}
-	
+
 	if (tiffHeader)
 		{
 		if (!tiffHeader->read(map_data, map_len)) throw Error(3, "TIFF");
@@ -1381,12 +1439,12 @@ RawFile::RawFile(BasicIo &io) : io_(io), map_data(NULL), map_len(0), offset(0)
 		rootDir->setStart(map_data + tiffHeader->ifdOffset());
 #endif
 		}
-		
+
 	if (cr2Header)
 		{
 		rootDir->setStart(map_data + cr2Header->offset());
 		}
-	
+
 	TiffRwState::AutoPtr state(new TiffRwState(tiffHeader ? tiffHeader->byteOrder() : littleEndian, 0, createFct));
 
 	TiffReader reader(map_data,
@@ -1395,7 +1453,7 @@ RawFile::RawFile(BasicIo &io) : io_(io), map_data(NULL), map_len(0), offset(0)
 			  state);
 
 	rootDir->accept(reader);
-	
+
 	if (tiffHeader)
 		delete tiffHeader;
 	if (cr2Header)
@@ -1426,7 +1484,7 @@ unsigned long RawFile::preview_offset(void)
 {
 	const Value *val;
 	if (offset) return offset;
-	
+
 	if (type == Exiv2::ImageType::cr2)
 		{
 		val = find(0x111, Group::ifd0);
@@ -1434,7 +1492,7 @@ unsigned long RawFile::preview_offset(void)
 
 		return 0;
 		}
-	
+
 	val = find(0x201, Group::sub0_0);
 	if (val) return val->toLong();
 
