@@ -1,18 +1,30 @@
 /*
- * Geeqie
- * (C) 2004 John Ellis
- * Copyright (C) 2008 - 2010 The Geeqie Team
+ * Copyright (C) 2004 John Ellis
+ * Copyright (C) 2008 - 2016 The Geeqie Team
  *
  * Author: John Ellis
  *
- * This software is released under the GNU General Public License (GNU GPL).
- * Please read the included file COPYING for more information.
- * This software comes with no warranty of any kind, use at your own risk!
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-
 #include "main.h"
 #include "image-load.h"
+#include "image_load_gdk.h"
+#include "image_load_jpeg.h"
+#include "image_load_tiff.h"
+#include "image_load_ffmpegthumbnailer.h"
 
 #include "exif.h"
 #include "filedata.h"
@@ -35,6 +47,7 @@ enum {
 	SIGNAL_ERROR,
 	SIGNAL_DONE,
 	SIGNAL_PERCENT,
+	SIGNAL_SIZE,
 	SIGNAL_COUNT
 };
 
@@ -48,7 +61,7 @@ static void image_loader_stop(ImageLoader *il);
 GType image_loader_get_type(void)
 {
 	static GType type = 0;
-	if (type == 0) 
+	if (type == 0)
 		{
 		static const GTypeInfo info = {
 			sizeof(ImageLoaderClass),
@@ -88,13 +101,22 @@ static void image_loader_init(GTypeInstance *instance, gpointer g_class)
 
 	il->requested_width = 0;
 	il->requested_height = 0;
+	il->actual_width = 0;
+	il->actual_height = 0;
 	il->shrunk = FALSE;
 
 	il->can_destroy = TRUE;
 
 #ifdef HAVE_GTHREAD
+#if GLIB_CHECK_VERSION(2,32,0)
+	il->data_mutex = g_new(GMutex, 1);
+	g_mutex_init(il->data_mutex);
+	il->can_destroy_cond = g_new(GCond, 1);
+	g_cond_init(il->can_destroy_cond);
+#else
 	il->data_mutex = g_mutex_new();
 	il->can_destroy_cond = g_cond_new();
+#endif
 #endif
 	DEBUG_1("new image loader %p, bufsize=%" G_GSIZE_FORMAT " idle_loop=%u", il, il->read_buffer_size, il->idle_read_loop_count);
 }
@@ -150,6 +172,17 @@ static void image_loader_class_init(ImageLoaderClass *class)
 			     G_TYPE_NONE, 1,
 			     G_TYPE_DOUBLE);
 
+	signals[SIGNAL_SIZE] =
+		g_signal_new("size_prepared",
+			     G_OBJECT_CLASS_TYPE(gobject_class),
+			     G_SIGNAL_RUN_LAST,
+			     G_STRUCT_OFFSET(ImageLoaderClass, area_ready),
+			     NULL, NULL,
+			     gq_marshal_VOID__INT_INT,
+			     G_TYPE_NONE, 2,
+			     G_TYPE_INT,
+			     G_TYPE_INT);
+
 }
 
 static void image_loader_finalize(GObject *object)
@@ -168,12 +201,12 @@ static void image_loader_finalize(GObject *object)
 		il->idle_done_id = 0;
 		}
 
-	while (g_source_remove_by_user_data(il)) 
+	while (g_source_remove_by_user_data(il))
 		{
 		DEBUG_2("pending signals detected");
 		}
-	
-	while (il->area_param_list) 
+
+	while (il->area_param_list)
 		{
 		DEBUG_1("pending area_ready signals detected");
 		while (g_source_remove_by_user_data(il->area_param_list->data)) {}
@@ -181,21 +214,28 @@ static void image_loader_finalize(GObject *object)
 		il->area_param_list = g_list_delete_link(il->area_param_list, il->area_param_list);
 		}
 
-	while (il->area_param_delayed_list) 
+	while (il->area_param_delayed_list)
 		{
 		g_free(il->area_param_delayed_list->data);
 		il->area_param_delayed_list = g_list_delete_link(il->area_param_delayed_list, il->area_param_delayed_list);
 		}
 
 	if (il->pixbuf) g_object_unref(il->pixbuf);
-	
+
 	if (il->error) g_error_free(il->error);
 
 	file_data_unref(il->fd);
 #ifdef HAVE_GTHREAD
+#if GLIB_CHECK_VERSION(2,32,0)
+	g_mutex_clear(il->data_mutex);
+	g_free(il->data_mutex);
+	g_cond_clear(il->can_destroy_cond);
+	g_free(il->can_destroy_cond);
+#else
 	g_mutex_free(il->data_mutex);
 	g_cond_free(il->can_destroy_cond);
 #endif
+#endif
 }
 
 void image_loader_free(ImageLoader *il)
@@ -212,9 +252,9 @@ ImageLoader *image_loader_new(FileData *fd)
 	if (!fd) return NULL;
 
 	il = (ImageLoader *) g_object_new(TYPE_IMAGE_LOADER, NULL);
-	
+
 	il->fd = file_data_ref(fd);
-	
+
 	return il;
 }
 
@@ -235,12 +275,18 @@ static gboolean image_loader_emit_area_ready_cb(gpointer data)
 {
 	ImageLoaderAreaParam *par = data;
 	ImageLoader *il = par->il;
-	g_signal_emit(il, signals[SIGNAL_AREA_READY], 0, par->x, par->y, par->w, par->h);
+	guint x, y, w, h;
 	g_mutex_lock(il->data_mutex);
 	il->area_param_list = g_list_remove(il->area_param_list, par);
+	x = par->x;
+	y = par->y;
+	w = par->w;
+	h = par->h;
 	g_free(par);
 	g_mutex_unlock(il->data_mutex);
-	
+
+	g_signal_emit(il, signals[SIGNAL_AREA_READY], 0, x, y, w, h);
+
 	return FALSE;
 }
 
@@ -265,6 +311,19 @@ static gboolean image_loader_emit_percent_cb(gpointer data)
 	return FALSE;
 }
 
+static gboolean image_loader_emit_size_cb(gpointer data)
+{
+	gint width, height;
+	ImageLoader *il = data;
+	g_mutex_lock(il->data_mutex);
+	width = il->actual_width;
+	height = il->actual_height;
+	g_mutex_unlock(il->data_mutex);
+	g_signal_emit(il, signals[SIGNAL_SIZE], 0, width, height);
+	return FALSE;
+}
+
+
 /* DONE and ERROR are emited only once, thus they can have normal priority
    PERCENT and AREA_READY should be processed ASAP
 */
@@ -284,35 +343,77 @@ static void image_loader_emit_percent(ImageLoader *il)
 	g_idle_add_full(G_PRIORITY_HIGH, image_loader_emit_percent_cb, il, NULL);
 }
 
-/* this function expects that il->data_mutex is locked by caller */
-static void image_loader_emit_area_ready(ImageLoader *il, guint x, guint y, guint w, guint h)
+static void image_loader_emit_size(ImageLoader *il)
 {
+	g_idle_add_full(G_PRIORITY_HIGH, image_loader_emit_size_cb, il, NULL);
+}
+
+static ImageLoaderAreaParam *image_loader_queue_area_ready(ImageLoader *il, GList **list, guint x, guint y, guint w, guint h)
+{
+	if (*list)
+		{
+		ImageLoaderAreaParam *prev_par = (*list)->data;
+		if (prev_par->x == x && prev_par->w == w &&
+		    prev_par->y + prev_par->h == y)
+			{
+			/* we can merge the notifications */
+			prev_par->h += h;
+			return NULL;
+			}
+		if (prev_par->x == x && prev_par->w == w &&
+		    y + h == prev_par->y)
+			{
+			/* we can merge the notifications */
+			prev_par->h += h;
+			prev_par->y = y;
+			return NULL;
+			}
+		if (prev_par->y == y && prev_par->h == h &&
+		    prev_par->x + prev_par->w == x)
+			{
+			/* we can merge the notifications */
+			prev_par->w += w;
+			return NULL;
+			}
+		if (prev_par->y == y && prev_par->h == h &&
+		    x + w == prev_par->x)
+			{
+			/* we can merge the notifications */
+			prev_par->w += w;
+			prev_par->x = x;
+			return NULL;
+			}
+		}
+
 	ImageLoaderAreaParam *par = g_new0(ImageLoaderAreaParam, 1);
 	par->il = il;
 	par->x = x;
 	par->y = y;
 	par->w = w;
 	par->h = h;
-	
-	il->area_param_list = g_list_prepend(il->area_param_list, par);
-	
-	g_idle_add_full(G_PRIORITY_HIGH, image_loader_emit_area_ready_cb, par, NULL);
+
+	*list = g_list_prepend(*list, par);
+	return par;
+}
+
+/* this function expects that il->data_mutex is locked by caller */
+static void image_loader_emit_area_ready(ImageLoader *il, guint x, guint y, guint w, guint h)
+{
+	ImageLoaderAreaParam *par = image_loader_queue_area_ready(il, &il->area_param_list, x, y, w, h);
+
+	if (par)
+		{
+		g_idle_add_full(G_PRIORITY_HIGH, image_loader_emit_area_ready_cb, par, NULL);
+		}
 }
 
 /**************************************************************************************/
 /* the following functions may be executed in separate thread */
 
 /* this function expects that il->data_mutex is locked by caller */
-static void image_loader_queue_delayed_erea_ready(ImageLoader *il, guint x, guint y, guint w, guint h)
+static void image_loader_queue_delayed_area_ready(ImageLoader *il, guint x, guint y, guint w, guint h)
 {
-	ImageLoaderAreaParam *par = g_new0(ImageLoaderAreaParam, 1);
-	par->il = il;
-	par->x = x;
-	par->y = y;
-	par->w = w;
-	par->h = h;
-	
-	il->area_param_delayed_list = g_list_prepend(il->area_param_delayed_list, par);
+	image_loader_queue_area_ready(il, &il->area_param_delayed_list, x, y, w, h);
 }
 
 
@@ -333,16 +434,16 @@ static gboolean image_loader_get_stopping(ImageLoader *il)
 static void image_loader_sync_pixbuf(ImageLoader *il)
 {
 	GdkPixbuf *pb;
-	
+
 	g_mutex_lock(il->data_mutex);
-	
-	if (!il->loader) 
+
+	if (!il->loader)
 		{
 		g_mutex_unlock(il->data_mutex);
 		return;
 		}
 
-	pb = gdk_pixbuf_loader_get_pixbuf(il->loader);
+	pb = il->backend.get_pixbuf(il->loader);
 
 	if (pb == il->pixbuf)
 		{
@@ -350,6 +451,11 @@ static void image_loader_sync_pixbuf(ImageLoader *il)
 		return;
 		}
 
+	if (g_ascii_strcasecmp(".jps", il->fd->extension) == 0)
+		{
+		g_object_set_data(G_OBJECT(pb), "stereo_data", GINT_TO_POINTER(STEREO_PIXBUF_CROSS));
+		}
+
 	if (il->pixbuf) g_object_unref(il->pixbuf);
 
 	il->pixbuf = pb;
@@ -358,7 +464,7 @@ static void image_loader_sync_pixbuf(ImageLoader *il)
 	g_mutex_unlock(il->data_mutex);
 }
 
-static void image_loader_area_updated_cb(GdkPixbufLoader *loader,
+static void image_loader_area_updated_cb(gpointer loader,
 				 guint x, guint y, guint w, guint h,
 				 gpointer data)
 {
@@ -375,78 +481,89 @@ static void image_loader_area_updated_cb(GdkPixbufLoader *loader,
 
 	g_mutex_lock(il->data_mutex);
 	if (il->delay_area_ready)
-		image_loader_queue_delayed_erea_ready(il, x, y, w, h);
+		image_loader_queue_delayed_area_ready(il, x, y, w, h);
 	else
 		image_loader_emit_area_ready(il, x, y, w, h);
+
+	if (il->stopping) il->backend.abort(il->loader);
+
 	g_mutex_unlock(il->data_mutex);
 }
 
-static void image_loader_area_prepared_cb(GdkPixbufLoader *loader, gpointer data)
+static void image_loader_area_prepared_cb(gpointer loader, gpointer data)
 {
+	ImageLoader *il = data;
 	GdkPixbuf *pb;
 	guchar *pix;
 	size_t h, rs;
-	
-	/* a workaround for 
-	   http://bugzilla.gnome.org/show_bug.cgi?id=547669 
+
+	/* a workaround for
+	   http://bugzilla.gnome.org/show_bug.cgi?id=547669
 	   http://bugzilla.gnome.org/show_bug.cgi?id=589334
 	*/
-	gchar *format = gdk_pixbuf_format_get_name(gdk_pixbuf_loader_get_format(loader));
+	gchar *format = il->backend.get_format_name(loader);
 	if (strcmp(format, "svg") == 0 ||
 	    strcmp(format, "xpm") == 0)
 		{
 		g_free(format);
 		return;
 		}
-	
+
 	g_free(format);
 
-	pb = gdk_pixbuf_loader_get_pixbuf(loader);
-	
+	pb = il->backend.get_pixbuf(loader);
+
 	h = gdk_pixbuf_get_height(pb);
 	rs = gdk_pixbuf_get_rowstride(pb);
 	pix = gdk_pixbuf_get_pixels(pb);
-	
+
 	memset(pix, 0, rs * h); /*this should be faster than pixbuf_fill */
 
 }
 
-static void image_loader_size_cb(GdkPixbufLoader *loader,
+static void image_loader_size_cb(gpointer loader,
 				 gint width, gint height, gpointer data)
 {
 	ImageLoader *il = data;
-	GdkPixbufFormat *format;
 	gchar **mime_types;
 	gboolean scale = FALSE;
 	gint n;
 
 	g_mutex_lock(il->data_mutex);
-	if (il->requested_width < 1 || il->requested_height < 1) 
+	il->actual_width = width;
+	il->actual_height = height;
+	if (il->requested_width < 1 || il->requested_height < 1)
 		{
 		g_mutex_unlock(il->data_mutex);
+		image_loader_emit_size(il);
 		return;
 		}
 	g_mutex_unlock(il->data_mutex);
 
-	format = gdk_pixbuf_loader_get_format(loader);
-	if (!format) return;
-
-	mime_types = gdk_pixbuf_format_get_mime_types(format);
+#ifdef HAVE_FFMPEGTHUMBNAILER
+	if (il->fd->format_class == FORMAT_CLASS_VIDEO)
+		scale = TRUE;
+#endif
+	mime_types = il->backend.get_format_mime_types(loader);
 	n = 0;
-	while (mime_types[n])
+	while (mime_types[n] && !scale)
 		{
 		if (strstr(mime_types[n], "jpeg")) scale = TRUE;
 		n++;
 		}
 	g_strfreev(mime_types);
 
-	if (!scale) return;
+	if (!scale)
+		{
+		image_loader_emit_size(il);
+		return;
+		}
 
 	g_mutex_lock(il->data_mutex);
 
+	gint nw, nh;
 	if (width > il->requested_width || height > il->requested_height)
 		{
-		gint nw, nh;
 
 		if (((gdouble)il->requested_width / width) < ((gdouble)il->requested_height / height))
 			{
@@ -461,11 +578,14 @@ static void image_loader_size_cb(GdkPixbufLoader *loader,
 			if (nw < 1) nw = 1;
 			}
 
-		gdk_pixbuf_loader_set_size(loader, nw, nh);
+		il->actual_width = nw;
+		il->actual_height = nh;
+		il->backend.set_size(loader, nw, nh);
 		il->shrunk = TRUE;
 		}
-	g_mutex_unlock(il->data_mutex);
 
+	g_mutex_unlock(il->data_mutex);
+	image_loader_emit_size(il);
 }
 
 static void image_loader_stop_loader(ImageLoader *il)
@@ -475,9 +595,9 @@ static void image_loader_stop_loader(ImageLoader *il)
 	if (il->loader)
 		{
 		/* some loaders do not have a pixbuf till close, order is important here */
-		gdk_pixbuf_loader_close(il->loader, il->error ? NULL : &il->error); /* we are interested in the first error only */
+		il->backend.close(il->loader, il->error ? NULL : &il->error); /* we are interested in the first error only */
 		image_loader_sync_pixbuf(il);
-		g_object_unref(G_OBJECT(il->loader));
+		il->backend.free(il->loader);
 		il->loader = NULL;
 		}
 	g_mutex_lock(il->data_mutex);
@@ -488,14 +608,37 @@ static void image_loader_stop_loader(ImageLoader *il)
 static void image_loader_setup_loader(ImageLoader *il)
 {
 	g_mutex_lock(il->data_mutex);
-	il->loader = gdk_pixbuf_loader_new();
+#ifdef HAVE_FFMPEGTHUMBNAILER
+	if (il->fd->format_class == FORMAT_CLASS_VIDEO)
+		{
+		DEBUG_1("Using custom ffmpegthumbnailer loader");
+		image_loader_backend_set_ft(&il->backend);
+		}
+	else
+#endif
+#ifdef HAVE_JPEG
+	if (il->bytes_total >= 2 && il->mapped_file[0] == 0xff && il->mapped_file[1] == 0xd8)
+		{
+		DEBUG_1("Using custom jpeg loader");
+		image_loader_backend_set_jpeg(&il->backend);
+		}
+	else
+#endif
+#ifdef HAVE_TIFF
+	if (il->bytes_total >= 10 &&
+	    (memcmp(il->mapped_file, "MM\0*", 4) == 0 ||
+	     memcmp(il->mapped_file, "MM\0+\0\x08\0\0", 8) == 0 ||
+	     memcmp(il->mapped_file, "II+\0\x08\0\0\0", 8) == 0 ||
+	     memcmp(il->mapped_file, "II*\0", 4) == 0))
+	     	{
+		DEBUG_1("Using custom tiff loader");
+		image_loader_backend_set_tiff(&il->backend);
+		}
+	else
+#endif
+		image_loader_backend_set_default(&il->backend);
 
-	g_signal_connect(G_OBJECT(il->loader), "area_updated",
-			 G_CALLBACK(image_loader_area_updated_cb), il);
-	g_signal_connect(G_OBJECT(il->loader), "size_prepared",
-			 G_CALLBACK(image_loader_size_cb), il);
-	g_signal_connect(G_OBJECT(il->loader), "area_prepared",
-			 G_CALLBACK(image_loader_area_prepared_cb), il);
+	il->loader = il->backend.loader_new(image_loader_area_updated_cb, image_loader_size_cb, image_loader_area_prepared_cb, il);
 	g_mutex_unlock(il->data_mutex);
 }
 
@@ -534,7 +677,7 @@ static gboolean image_loader_continue(ImageLoader *il)
 			return FALSE;
 			}
 
-		if (b < 0 || (b > 0 && !gdk_pixbuf_loader_write(il->loader, il->mapped_file + il->bytes_read, b, &il->error)))
+		if (b < 0 || (b > 0 && !il->backend.write(il->loader, il->mapped_file + il->bytes_read, b, &il->error)))
 			{
 			image_loader_error(il);
 			return FALSE;
@@ -556,7 +699,7 @@ static gboolean image_loader_continue(ImageLoader *il)
 static gboolean image_loader_begin(ImageLoader *il)
 {
 	gssize b;
-	
+
 	if (il->pixbuf) return FALSE;
 
 	b = MIN(il->read_buffer_size, il->bytes_total - il->bytes_read);
@@ -564,7 +707,16 @@ static gboolean image_loader_begin(ImageLoader *il)
 
 	image_loader_setup_loader(il);
 
-	if (!gdk_pixbuf_loader_write(il->loader, il->mapped_file + il->bytes_read, b, &il->error))
+	g_assert(il->bytes_read == 0);
+	if (il->backend.load) {
+		b = il->bytes_total;
+		if (!il->backend.load(il->loader, il->mapped_file, b, &il->error))
+			{
+			image_loader_stop_loader(il);
+			return FALSE;
+			}
+	}
+	else if (!il->backend.write(il->loader, il->mapped_file, b, &il->error))
 		{
 		image_loader_stop_loader(il);
 		return FALSE;
@@ -573,10 +725,10 @@ static gboolean image_loader_begin(ImageLoader *il)
 	il->bytes_read += b;
 
 	/* read until size is known */
-	while (il->loader && !gdk_pixbuf_loader_get_pixbuf(il->loader) && b > 0 && !image_loader_get_stopping(il))
+	while (il->loader && !il->backend.get_pixbuf(il->loader) && b > 0 && !image_loader_get_stopping(il))
 		{
 		b = MIN(il->read_buffer_size, il->bytes_total - il->bytes_read);
-		if (b < 0 || (b > 0 && !gdk_pixbuf_loader_write(il->loader, il->mapped_file + il->bytes_read, b, &il->error)))
+		if (b < 0 || (b > 0 && !il->backend.write(il->loader, il->mapped_file + il->bytes_read, b, &il->error)))
 			{
 			image_loader_stop_loader(il);
 			return FALSE;
@@ -635,12 +787,12 @@ static gboolean image_loader_setup_source(ImageLoader *il)
 		exif_free_fd(il->fd, exif);
 		}
 
-	
+
 	if (!il->mapped_file)
 		{
 		/* normal file */
 		gint load_fd;
-	
+
 		pathl = path_from_utf8(il->fd->path);
 		load_fd = open(pathl, O_RDONLY | O_NONBLOCK);
 		g_free(pathl);
@@ -655,7 +807,7 @@ static gboolean image_loader_setup_source(ImageLoader *il)
 			close(load_fd);
 			return FALSE;
 			}
-		
+
 		il->mapped_file = mmap(0, il->bytes_total, PROT_READ|PROT_WRITE, MAP_PRIVATE, load_fd, 0);
 		close(load_fd);
 		if (il->mapped_file == MAP_FAILED)
@@ -665,14 +817,14 @@ static gboolean image_loader_setup_source(ImageLoader *il)
 			}
 		il->preview = FALSE;
 		}
-		
+
 	return TRUE;
 }
 
 static void image_loader_stop_source(ImageLoader *il)
 {
 	if (!il) return;
-	
+
 	if (il->mapped_file)
 		{
 		if (il->preview)
@@ -696,7 +848,7 @@ static void image_loader_stop(ImageLoader *il)
 		g_source_remove(il->idle_id);
 		il->idle_id = 0;
 		}
-		
+
 	if (il->thread)
 		{
 		/* stop loader in the other thread */
@@ -708,7 +860,7 @@ static void image_loader_stop(ImageLoader *il)
 
 	image_loader_stop_loader(il);
 	image_loader_stop_source(il);
-	
+
 }
 
 void image_loader_delay_area_ready(ImageLoader *il, gboolean enable)
@@ -729,7 +881,7 @@ void image_loader_delay_area_ready(ImageLoader *il, gboolean enable)
 			{
 			ImageLoaderAreaParam *par = work->data;
 			work = work->next;
-			
+
 			g_signal_emit(il, signals[SIGNAL_AREA_READY], 0, par->x, par->y, par->w, par->h);
 			g_free(par);
 			}
@@ -755,12 +907,12 @@ static gboolean image_loader_idle_cb(gpointer data)
 		{
 		ret = image_loader_continue(il);
 		}
-	
+
 	if (!ret)
 		{
 		image_loader_stop_source(il);
 		}
-	
+
 	return ret;
 }
 
@@ -768,13 +920,13 @@ static gboolean image_loader_idle_cb(gpointer data)
 static gboolean image_loader_start_idle(ImageLoader *il)
 {
 	gboolean ret;
-	
+
 	if (!il) return FALSE;
 
 	if (!il->fd) return FALSE;
 
 	if (!image_loader_setup_source(il)) return FALSE;
-	
+
 	ret = image_loader_begin(il);
 
 	if (ret && !il->done) il->idle_id = g_idle_add_full(il->idle_priority, image_loader_idle_cb, il, NULL);
@@ -810,7 +962,7 @@ static void image_loader_thread_leave_high(void)
 static void image_loader_thread_wait_high(void)
 {
 	g_mutex_lock(image_loader_prio_mutex);
-	while (image_loader_prio_num) 
+	while (image_loader_prio_num)
 		{
 		g_cond_wait(image_loader_prio_cond, image_loader_prio_mutex);
 		}
@@ -824,8 +976,8 @@ static void image_loader_thread_run(gpointer data, gpointer user_data)
 	ImageLoader *il = data;
 	gboolean cont;
 	gboolean err;
-	
-	if (il->idle_priority > G_PRIORITY_DEFAULT_IDLE) 
+
+	if (il->idle_priority > G_PRIORITY_DEFAULT_IDLE)
 		{
 		/* low prio, wait untill high prio tasks finishes */
 		image_loader_thread_wait_high();
@@ -835,24 +987,24 @@ static void image_loader_thread_run(gpointer data, gpointer user_data)
 		/* high prio */
 		image_loader_thread_enter_high();
 		}
-	
+
 	err = !image_loader_begin(il);
-	
+
 	if (err)
 		{
-		/* 
-		loader failed, we have to send signal 
+		/*
+		loader failed, we have to send signal
 		(idle mode returns the image_loader_begin return value directly)
 		(success is always reported indirectly from image_loader_begin)
 		*/
 		image_loader_emit_error(il);
 		}
-	
+
 	cont = !err;
-	
+
 	while (cont && !image_loader_get_is_done(il) && !image_loader_get_stopping(il))
 		{
-		if (il->idle_priority > G_PRIORITY_DEFAULT_IDLE) 
+		if (il->idle_priority > G_PRIORITY_DEFAULT_IDLE)
 			{
 			/* low prio, wait untill high prio tasks finishes */
 			image_loader_thread_wait_high();
@@ -861,7 +1013,7 @@ static void image_loader_thread_run(gpointer data, gpointer user_data)
 		}
 	image_loader_stop_loader(il);
 
-	if (il->idle_priority <= G_PRIORITY_DEFAULT_IDLE) 
+	if (il->idle_priority <= G_PRIORITY_DEFAULT_IDLE)
 		{
 		/* high prio */
 		image_loader_thread_leave_high();
@@ -882,21 +1034,28 @@ static gboolean image_loader_start_thread(ImageLoader *il)
 	if (!il->fd) return FALSE;
 
 	il->thread = TRUE;
-	
+
 	if (!image_loader_setup_source(il)) return FALSE;
 
-        if (!image_loader_thread_pool) 
+        if (!image_loader_thread_pool)
 		{
 		image_loader_thread_pool = g_thread_pool_new(image_loader_thread_run, NULL, -1, FALSE, NULL);
+#if GLIB_CHECK_VERSION(2,32,0)
+		if (!image_loader_prio_cond) image_loader_prio_cond = g_new(GCond, 1);
+		g_cond_init(image_loader_prio_cond);
+		if (!image_loader_prio_mutex) image_loader_prio_mutex = g_new(GMutex, 1);
+		g_mutex_init(image_loader_prio_mutex);
+#else
 		image_loader_prio_cond = g_cond_new();
 		image_loader_prio_mutex = g_mutex_new();
+#endif
 		}
 
 	il->can_destroy = FALSE; /* ImageLoader can't be freed until image_loader_thread_run finishes */
 
 	g_thread_pool_push(image_loader_thread_pool, il, NULL);
 	DEBUG_1("Thread pool num threads: %d", g_thread_pool_get_num_threads(image_loader_thread_pool));
-		
+
 	return TRUE;
 }
 #endif /* HAVE_GTHREAD */
@@ -925,34 +1084,13 @@ GdkPixbuf *image_loader_get_pixbuf(ImageLoader *il)
 {
 	GdkPixbuf *ret;
 	if (!il) return NULL;
-	
+
 	g_mutex_lock(il->data_mutex);
 	ret = il->pixbuf;
 	g_mutex_unlock(il->data_mutex);
 	return ret;
 }
 
-gchar *image_loader_get_format(ImageLoader *il)
-{
-	GdkPixbufFormat *format;
-	gchar **mimev;
-	gchar *mime;
-
-	if (!il || !il->loader) return NULL;
-
-	format = gdk_pixbuf_loader_get_format(il->loader);
-	if (!format) return NULL;
-
-	mimev = gdk_pixbuf_format_get_mime_types(format);
-	if (!mimev) return NULL;
-
-	/* return first member of mimev, as GdkPixbufLoader has no way to tell us which exact one ? */
-	mime = g_strdup(mimev[0]);
-	g_strfreev(mimev);
-
-	return mime;
-}
-
 void image_loader_set_requested_size(ImageLoader *il, gint width, gint height)
 {
 	if (!il) return;
@@ -985,9 +1123,9 @@ gdouble image_loader_get_percent(ImageLoader *il)
 {
 	gdouble ret;
 	if (!il) return 0.0;
-	
+
 	g_mutex_lock(il->data_mutex);
-	if (il->bytes_total == 0) 
+	if (il->bytes_total == 0)
 		{
 		ret = 0.0;
 		}
