@@ -1,13 +1,22 @@
 /*
- * Geeqie
- * (C) 2006 John Ellis
- * Copyright (C) 2008 - 2012 The Geeqie Team
+ * Copyright (C) 2006 John Ellis
+ * Copyright (C) 2008 - 2016 The Geeqie Team
  *
  * Author: John Ellis
  *
- * This software is released under the GNU General Public License (GNU GPL).
- * Please read the included file COPYING for more information.
- * This software comes with no warranty of any kind, use at your own risk!
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include "main.h"
@@ -25,6 +34,7 @@
 #include "pixbuf_util.h"
 #include "ui_fileops.h"
 #include "image-load.h"
+#include "glua.h"
 
 /*
  *----------------------------------------------------------------------------
@@ -44,7 +54,7 @@ struct _OverlayStateData {
 	OsdShowFlags show;
 
 	gint ovl_info;
-	
+
 	gint x;
 	gint y;
 
@@ -99,6 +109,14 @@ void set_default_image_overlay_template_string(gchar **template_string)
 	set_image_overlay_template_string(template_string, DEFAULT_OVERLAY_INFO);
 }
 
+void set_image_overlay_font_string(gchar **font_string, const gchar *value)
+{
+	g_assert(font_string);
+
+	g_free(*font_string);
+	*font_string = g_strdup(value);
+}
+
 static OverlayStateData *image_get_osd_data(ImageWindow *imd)
 {
 	OverlayStateData *osd;
@@ -226,13 +244,13 @@ static gchar *keywords_to_string(FileData *fd)
 			{
 			gchar *kw = work->data;
 			work = work->next;
-			
+
 			if (!kw) continue;
 			if (!kwstr)
 				kwstr = g_string_new("");
 			else
 				g_string_append(kwstr, ", ");
-			
+
 			g_string_append(kwstr, kw);
 			}
 		string_list_free(keywords);
@@ -261,7 +279,7 @@ static gchar *image_osd_mkinfo(const gchar *str, ImageWindow *imd, GHashTable *v
 
 	new = g_string_new(str);
 
-	prev = 0;
+	prev = -1;
 
 	while (TRUE)
 		{
@@ -272,7 +290,7 @@ static gchar *image_osd_mkinfo(const gchar *str, ImageWindow *imd, GHashTable *v
 		gchar *extrapos = NULL;
 		gchar *p;
 
-		start = strchr(new->str, delim);
+		start = strchr(new->str + (prev + 1), delim);
 		if (!start)
 			break;
 		end = strchr(start+1, delim);
@@ -305,7 +323,7 @@ static gchar *image_osd_mkinfo(const gchar *str, ImageWindow *imd, GHashTable *v
 
 		if (extrapos)
 			extra = g_strndup(extrapos, end - extrapos);
-					
+
 		name = g_strndup(start+1, (trunc ? trunc : end)-start-1);
 		pos = start - new->str;
 		data = NULL;
@@ -318,20 +336,35 @@ static gchar *image_osd_mkinfo(const gchar *str, ImageWindow *imd, GHashTable *v
 			{
 			data = metadata_read_string(imd->image_fd, COMMENT_KEY, METADATA_PLAIN);
 			}
+		else if (strcmp(name, "imagecomment") == 0)
+			{
+			data = exif_get_image_comment(imd->image_fd);
+			}
+#ifdef HAVE_LUA
+		else if (strncmp(name, "lua/", 4) == 0)
+			{
+			gchar *tmp;
+			tmp = strchr(name+4, '/');
+			if (!tmp)
+				break;
+			*tmp = '\0';
+			data = lua_callvalue(imd->image_fd, name+4, tmp+1);
+			}
+#endif
 		else
 			{
 			data = g_strdup(g_hash_table_lookup(vars, name));
 			if (!data)
 				data = metadata_read_string(imd->image_fd, name, METADATA_FORMATTED);
 			}
-	
+
 		if (data && *data && limit > 0 && strlen(data) > limit + 3)
 			{
 			gchar *new_data = g_strdup_printf("%-*.*s...", limit, limit, data);
 			g_free(data);
 			data = new_data;
 			}
-	
+
 		if (data)
 			{
 			/* Since we use pango markup to display, we need to escape here */
@@ -366,7 +399,7 @@ static gchar *image_osd_mkinfo(const gchar *str, ImageWindow *imd, GHashTable *v
 				gchar *right = extra;
 				gchar *p;
 				guint len = strlen(extra);
-				
+
 				/* Search for left and right parts and unescape characters */
 				for (p = extra; *p; p++, len--)
 					if (p[0] == '\\')
@@ -477,11 +510,11 @@ static GdkPixbuf *image_osd_info_render(OverlayStateData *osd)
 	PangoLayout *layout;
 	const gchar *name;
 	gchar *text;
-	GdkPixbuf *imgpixbuf = NULL;
 	gboolean with_hist;
-	const HistMap *histmap;
+	const HistMap *histmap = NULL;
 	ImageWindow *imd = osd->imd;
 	FileData *fd = image_get_fd(imd);
+	PangoFontDescription *font_desc;
 
 	if (!fd) return NULL;
 
@@ -538,20 +571,20 @@ static GdkPixbuf *image_osd_info_render(OverlayStateData *osd)
 				t = 1;
 				n = 1;
 				}
-	
+
 			if (n < 1) n = 1;
 			if (t < 1) t = 1;
-	
+
 			osd_template_insert(vars, "collection", NULL, OSDT_NONE);
 			}
-		
+
 		osd_template_insert(vars, "number", g_strdup_printf("%d", n), OSDT_NO_DUP);
 		osd_template_insert(vars, "total", g_strdup_printf("%d", t), OSDT_NO_DUP);
 		osd_template_insert(vars, "name", (gchar *) name, OSDT_NONE);
 		osd_template_insert(vars, "date", imd->image_fd ? ((gchar *) text_from_time(imd->image_fd->date)) : "", OSDT_NONE);
 		osd_template_insert(vars, "size", imd->image_fd ? (text_from_size_abrev(imd->image_fd->size)) : g_strdup(""), OSDT_FREE);
 		osd_template_insert(vars, "zoom", image_zoom_get_as_text(imd), OSDT_FREE);
-	
+
 		if (!imd->unknown)
 			{
 			gint w, h;
@@ -563,15 +596,13 @@ static GdkPixbuf *image_osd_info_render(OverlayStateData *osd)
 				{
 				w = gdk_pixbuf_get_width(load_pixbuf);
 				h = gdk_pixbuf_get_height(load_pixbuf);
-				imgpixbuf = load_pixbuf;
 				}
 			else
 				{
 				image_get_image_size(imd, &w, &h);
-				imgpixbuf = (PIXBUF_RENDERER(imd->pr))->pixbuf;
 				}
-		
-			
+
+
 			osd_template_insert(vars, "width", g_strdup_printf("%d", w), OSDT_NO_DUP);
 	 		osd_template_insert(vars, "height", g_strdup_printf("%d", h), OSDT_NO_DUP);
 	 		osd_template_insert(vars, "res", g_strdup_printf("%d × %d", w, h), OSDT_FREE);
@@ -595,14 +626,14 @@ static GdkPixbuf *image_osd_info_render(OverlayStateData *osd)
 	if (with_hist)
 		{
 		histmap = histmap_get(imd->image_fd);
-		if (!histmap) 
+		if (!histmap)
 			{
 			histmap_start_idle(imd->image_fd);
 			with_hist = FALSE;
 			}
 		}
-	
-	
+
+
 	{
 		gint active_marks = 0;
 		gint mark;
@@ -619,7 +650,7 @@ static GdkPixbuf *image_osd_info_render(OverlayStateData *osd)
 
 			for (mark = 0; mark < FILEDATA_MARKS_SIZE; mark++)
 				{
-				g_string_append_printf(buf, file_data_get_mark(fd, mark) ? " <span background='#FF00FF'>%c</span>" : " %c", '1' + mark);
+				g_string_append_printf(buf, file_data_get_mark(fd, mark) ? " <span background='#FF00FF'>%c</span>" : " %c", '1' + (mark < 9 ? mark : -1) );
 				}
 
 			if (*text)
@@ -644,7 +675,10 @@ static GdkPixbuf *image_osd_info_render(OverlayStateData *osd)
 			}
 	}
 
+	font_desc = pango_font_description_from_string(options->image_overlay.font);
 	layout = gtk_widget_create_pango_layout(imd->pr, NULL);
+	pango_layout_set_font_description(layout, font_desc);
+
 	pango_layout_set_markup(layout, text, -1);
 	g_free(text);
 
@@ -665,9 +699,9 @@ static GdkPixbuf *image_osd_info_render(OverlayStateData *osd)
 
 	if (width > 0 && height > 0)
 		{
-		/* TODO: make osd color configurable --Zas */
 		pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, width, height);
-		pixbuf_set_rect_fill(pixbuf, 3, 3, width-6, height-6, 240, 240, 240, 210);
+		pixbuf_set_rect_fill(pixbuf, 3, 3, width-6, height-6, options->image_overlay.background_red, options->image_overlay.background_green,
+															options->image_overlay.background_blue, options->image_overlay.background_alpha);
 		pixbuf_set_rect(pixbuf, 0, 0, width, height, 240, 240, 240, 80, 1, 1, 1, 1);
 		pixbuf_set_rect(pixbuf, 1, 1, width-2, height-2, 240, 240, 240, 130, 1, 1, 1, 1);
 		pixbuf_set_rect(pixbuf, 2, 2, width-4, height-4, 240, 240, 240, 180, 1, 1, 1, 1);
@@ -685,7 +719,8 @@ static GdkPixbuf *image_osd_info_render(OverlayStateData *osd)
 			pixbuf_set_rect_fill(pixbuf, x, y, w, HISTOGRAM_HEIGHT, 220, 220, 220, 210);
 			histogram_draw(osd->histogram, histmap, pixbuf, x, y, w, HISTOGRAM_HEIGHT);
 			}
-		pixbuf_draw_layout(pixbuf, layout, imd->pr, 5, 5, 0, 0, 0, 255);
+		pixbuf_draw_layout(pixbuf, layout, imd->pr, 5, 5, options->image_overlay.text_red, options->image_overlay.text_green,
+															options->image_overlay.text_blue, options->image_overlay.text_alpha);
 	}
 
 	g_object_unref(G_OBJECT(layout));
@@ -765,13 +800,6 @@ static void image_overlay_set(ImageWindow *imd, gint id, GdkPixbuf *pixbuf, gint
 	pixbuf_renderer_overlay_set((PixbufRenderer *)imd->pr, id, pixbuf, x, y);
 }
 
-#if 0 /* unused for now */
-static gint image_overlay_get(ImageWindow *imd, gint id, GdkPixbuf **pixbuf, gint *x, gint *y)
-{
-	return pixbuf_renderer_overlay_get((PixbufRenderer *)imd->pr, id, pixbuf, x, y);
-}
-#endif
-
 static void image_overlay_remove(ImageWindow *imd, gint id)
 {
 	pixbuf_renderer_overlay_remove((PixbufRenderer *)imd->pr, id);
@@ -866,7 +894,7 @@ static gboolean image_osd_update_cb(gpointer data)
 
 	if (osd->show & OSD_SHOW_INFO)
 		{
-		/* redraw when the image was changed, 
+		/* redraw when the image was changed,
 		   with histogram we have to redraw also when loading is finished */
 		if (osd->changed_states & IMAGE_STATE_IMAGE ||
 		    (osd->changed_states & IMAGE_STATE_LOADING && osd->show & OSD_SHOW_HISTOGRAM) ||
@@ -895,7 +923,7 @@ static gboolean image_osd_update_cb(gpointer data)
 		{
 		if (osd->changed_states & IMAGE_STATE_IMAGE)
 			image_osd_icons_reset_time(osd);
-	
+
 		if (osd->changed_states & IMAGE_STATE_COLOR_ADJ)
 			{
 			osd->icon_time[IMAGE_OSD_COLOR] = IMAGE_OSD_DEFAULT_DURATION + 1;
@@ -1037,15 +1065,6 @@ static void image_osd_free(OverlayStateData *osd)
 	g_free(osd);
 }
 
-#if 0
-static void image_osd_remove(ImageWindow *imd)
-{
-	OverlayStateData *osd = image_get_osd_data(imd);
-
-	if (osd) image_osd_free(osd);
-}
-#endif
-
 static void image_osd_destroy_cb(GtkWidget *widget, gpointer data)
 {
 	OverlayStateData *osd = data;
@@ -1065,7 +1084,7 @@ static void image_osd_enable(ImageWindow *imd, OsdShowFlags show)
 		osd->show = OSD_SHOW_NOTHING;
 		osd->x = options->image_overlay.x;
 		osd->y = options->image_overlay.y;
-		
+
 		osd->histogram = histogram_new();
 
 		osd->destroy_id = g_signal_connect(G_OBJECT(imd->pr), "destroy",
@@ -1110,13 +1129,13 @@ void image_osd_copy_status(ImageWindow *src, ImageWindow *dest)
 {
 	Histogram *h_src, *h_dest;
 	image_osd_set(dest, image_osd_get(src));
-	
+
 	h_src = image_osd_get_histogram(src);
 	h_dest = image_osd_get_histogram(dest);
-	
+
 	h_dest->histogram_mode = h_src->histogram_mode;
 	h_dest->histogram_channel = h_src->histogram_channel;
-	
+
 }
 
 /* duration:
