@@ -1,13 +1,22 @@
 /*
- * Geeqie
- * (C) 2006 John Ellis
- * Copyright (C) 2008 - 2012 The Geeqie Team
+ * Copyright (C) 2006 John Ellis
+ * Copyright (C) 2008 - 2016 The Geeqie Team
  *
  * Author: John Ellis
  *
- * This software is released under the GNU General Public License (GNU GPL).
- * Please read the included file COPYING for more information.
- * This software comes with no warranty of any kind, use at your own risk!
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include <gdk/gdkkeysyms.h> /* for keyboard values */
@@ -19,6 +28,7 @@
 #ifdef G_OS_UNIX
 #include <pwd.h>
 #endif
+#include <locale.h>
 
 #include "main.h"
 
@@ -45,12 +55,11 @@
 #include "exif.h"
 #include "histogram.h"
 #include "pixbuf_util.h"
+#include "glua.h"
 
-#ifdef HAVE_LIBCHAMPLAIN
-#ifdef HAVE_LIBCHAMPLAIN_GTK
+#ifdef HAVE_CLUTTER
 #include <clutter-gtk/clutter-gtk.h>
 #endif
-#endif
 
 
 gboolean thumb_format_changed = FALSE;
@@ -97,8 +106,8 @@ void keyboard_scroll_calc(gint *x, gint *y, GdkEventKey *event)
 		delta = 8;
 		}
 
-	*x = *x * delta;
-	*y = *y * delta;
+	*x = *x * delta * options->keyboard_scroll_step;
+	*y = *y * delta * options->keyboard_scroll_step;
 }
 
 
@@ -142,7 +151,7 @@ static void parse_command_line_add_dir(const gchar *dir, gchar **path, gchar **f
 	path_parsed = g_strdup(dir);
 	parse_out_relatives(path_parsed);
 	dir_fd = file_data_new_dir(path_parsed);
-	
+
 
 	if (filelist_read(dir_fd, &files, NULL))
 		{
@@ -212,11 +221,12 @@ static void parse_command_line(gint argc, gchar *argv[])
 	GList *remote_errors = NULL;
 	gboolean remote_do = FALSE;
 	gchar *first_dir = NULL;
-	
+
 	command_line = g_new0(CommandLine, 1);
-	
+
 	command_line->argc = argc;
 	command_line->argv = argv;
+	command_line->regexp = NULL;
 
 	if (argc > 1)
 		{
@@ -246,6 +256,15 @@ static void parse_command_line(gint argc, gchar *argv[])
 				parse_command_line_process_file(cmd_all, &command_line->path, &command_line->file,
 								&list, &command_line->collection_list, &first_dir);
 				}
+			else if (is_collection(cmd_line))
+				{
+				gchar *path = NULL;
+
+				path = collection_path(cmd_line);
+				parse_command_line_process_file(path, &command_line->path, &command_line->file,
+								&list, &command_line->collection_list, &first_dir);
+				g_free(path);
+				}
 			else if (strncmp(cmd_line, "--debug", 7) == 0 && (cmd_line[7] == '\0' || cmd_line[7] == '='))
 				{
 				/* do nothing but do not produce warnings */
@@ -292,6 +311,27 @@ static void parse_command_line(gint argc, gchar *argv[])
 					remote_list = remote_build_list(remote_list, argc - i, &argv[i], &remote_errors);
 					}
 				}
+			else if ((strcmp(cmd_line, "+w") == 0) ||
+						strcmp(cmd_line, "--show-log-window") == 0)
+				{
+				command_line->log_window_show = TRUE;
+				}
+			else if (strncmp(cmd_line, "-o:", 3) == 0)
+				{
+				command_line->log_file = g_strdup(cmd_line + 3);
+				}
+			else if (strncmp(cmd_line, "--log-file:", 11) == 0)
+				{
+				command_line->log_file = g_strdup(cmd_line + 11);
+				}
+			else if (strncmp(cmd_line, "-g:", 3) == 0)
+				{
+				set_regexp(g_strdup(cmd_line+3));
+				}
+			else if (strncmp(cmd_line, "-grep:", 6) == 0)
+				{
+				set_regexp(g_strdup(cmd_line+3));
+				}
 			else if (strcmp(cmd_line, "-rh") == 0 ||
 				 strcmp(cmd_line, "--remote-help") == 0)
 				{
@@ -305,7 +345,7 @@ static void parse_command_line(gint argc, gchar *argv[])
 			else if (strcmp(cmd_line, "-v") == 0 ||
 				 strcmp(cmd_line, "--version") == 0)
 				{
-				printf_term("%s %s\n", GQ_APPNAME, VERSION);
+				printf_term(FALSE, "%s %s\n", GQ_APPNAME, VERSION);
 				exit(0);
 				}
 			else if (strcmp(cmd_line, "--alternate") == 0)
@@ -317,34 +357,39 @@ static void parse_command_line(gint argc, gchar *argv[])
 			else if (strcmp(cmd_line, "-h") == 0 ||
 				 strcmp(cmd_line, "--help") == 0)
 				{
-				printf_term("%s %s\n", GQ_APPNAME, VERSION);
-				printf_term(_("Usage: %s [options] [path]\n\n"), GQ_APPNAME_LC);
-				print_term(_("valid options are:\n"));
-				print_term(_("  +t, --with-tools           force show of tools\n"));
-				print_term(_("  -t, --without-tools        force hide of tools\n"));
-				print_term(_("  -f, --fullscreen           start in full screen mode\n"));
-				print_term(_("  -s, --slideshow            start in slideshow mode\n"));
-				print_term(_("  -l, --list                 open collection window for command line\n"));
-				print_term(_("      --geometry=GEOMETRY    set main window location\n"));
-				print_term(_("  -r, --remote               send following commands to open window\n"));
-				print_term(_("  -rh,--remote-help          print remote command list\n"));
+				printf_term(FALSE, "%s %s\n", GQ_APPNAME, VERSION);
+				printf_term(FALSE, _("Usage: %s [options] [path]\n\n"), GQ_APPNAME_LC);
+				print_term(FALSE, _("valid options are:\n"));
+				print_term(FALSE, _("  +t, --with-tools                 force show of tools\n"));
+				print_term(FALSE, _("  -t, --without-tools              force hide of tools\n"));
+				print_term(FALSE, _("  -f, --fullscreen                 start in full screen mode\n"));
+				print_term(FALSE, _("  -s, --slideshow                  start in slideshow mode\n"));
+				print_term(FALSE, _("  -l, --list [files] [collections] open collection window for command line\n"));
+				print_term(FALSE, _("      --blank                      start with blank file list\n"));
+				print_term(FALSE, _("      --geometry=XxY+XOFF+YOFF     set main window location\n"));
+				print_term(FALSE, _("  -r, --remote                     send following commands to open window\n"));
+				print_term(FALSE, _("  -rh,--remote-help                print remote command list\n"));
 #ifdef DEBUG
-				print_term(_("  --debug[=level]            turn on debug output\n"));
+				print_term(FALSE, _("      --debug[=level]              turn on debug output\n"));
+				print_term(FALSE, _("  -g:<regexp>, --grep:<regexp>     filter debug output\n"));
 #endif
-				print_term(_("  -v, --version              print version info\n"));
-				print_term(_("  -h, --help                 show this message\n\n"));
+				print_term(FALSE, _("  +w, --show-log-window            show log window\n"));
+				print_term(FALSE, _("  -o:<file>, --log-file:<file>     save log data to file\n"));
+				print_term(FALSE, _("  -v, --version                    print version info\n"));
+				print_term(FALSE, _("  -h, --help                       show this message\n\n"));
 
 #if 0
 				/* these options are not officially supported!
 				 * only for testing new features, no need to translate them */
-				print_term(  "  --alternate                use alternate similarity algorithm\n");
+				print_term(FALSE, "  --alternate                use alternate similarity algorithm\n");
 #endif
 
+
 				exit(0);
 				}
 			else if (!remote_do)
 				{
-				printf_term(_("invalid or ignored: %s\nUse --help for options\n"), cmd_line);
+				printf_term(TRUE, _("invalid or ignored: %s\nUse --help for options\n"), cmd_line);
 				}
 
 			g_free(cmd_all);
@@ -372,17 +417,17 @@ static void parse_command_line(gint argc, gchar *argv[])
 		if (remote_errors)
 			{
 			GList *work = remote_errors;
-			
-			printf_term(_("Invalid or ignored remote options: "));
+
+			printf_term(TRUE,_("Invalid or ignored remote options: "));
 			while (work)
 				{
 				gchar *opt = work->data;
-						
-				printf_term("%s%s", (work == remote_errors) ? "" : ", ", opt);
+
+				printf_term(TRUE, "%s%s", (work == remote_errors) ? "" : ", ", opt);
 				work = work->next;
 				}
 
-			printf_term(_("\nUse --remote-help for valid remote options.\n"));
+			printf_term(TRUE, _("\nUse --remote-help for valid remote options.\n"));
 			}
 
 		remote_control(argv[0], remote_list, command_line->path, list, command_line->collection_list);
@@ -543,7 +588,7 @@ static gboolean gq_accel_map_save(const gchar *path)
 		log_printf(_("error saving file: %s\n"), path);
 		return FALSE;
 		}
-	
+
 	gstring = g_string_new("; ");
 	if (g_get_prgname())
 		g_string_append(gstring, g_get_prgname());
@@ -628,6 +673,8 @@ static void exit_program_final(void)
 		layout_free(lw);
 		}
 
+	secure_close(command_line->ssi);
+
 	gtk_main_quit();
 }
 
@@ -674,7 +721,7 @@ static gint exit_confirm_dlg(void)
 	g_free(msg);
 	msg = g_strdup_printf(_("Quit %s"), GQ_APPNAME);
 	generic_dialog_add_message(exit_dialog, GTK_STOCK_DIALOG_QUESTION,
-				   msg, _("Collections have been modified. Quit anyway?"));
+				   msg, _("Collections have been modified. Quit anyway?"), TRUE);
 	g_free(msg);
 	generic_dialog_add_button(exit_dialog, GTK_STOCK_QUIT, NULL, exit_confirm_exit_cb, TRUE);
 
@@ -699,7 +746,7 @@ void exit_program(void)
 	exit_program_final();
 }
 
-/* This code is supposed to handle situation when a file mmaped by image_loader 
+/* This code is supposed to handle situation when a file mmaped by image_loader
  * or by exif loader is truncated by some other process.
  * This is probably not completely correct according to posix, because
  * mmap is not in the list of calls that can be used safely in signal handler,
@@ -737,17 +784,19 @@ gint main(gint argc, gchar *argv[])
 	CollectionData *cd = NULL;
 
 #ifdef HAVE_GTHREAD
+#if !GLIB_CHECK_VERSION(2,32,0)
 	g_thread_init(NULL);
+#endif
 	gdk_threads_init();
 	gdk_threads_enter();
 
 #endif
-	
+
 	/* init execution time counter (debug only) */
 	init_exec_time();
 
 	/* setup locale, i18n */
-	gtk_set_locale();
+	setlocale(LC_ALL, "");
 
 #ifdef ENABLE_NLS
 	bindtextdomain(PACKAGE, GQ_LOCALEDIR);
@@ -756,7 +805,11 @@ gint main(gint argc, gchar *argv[])
 #endif
 
 	exif_init();
-	
+
+#ifdef HAVE_LUA
+	lua_init();
+#endif
+
 	/* setup random seed for random slideshow */
 	srand(time(NULL));
 
@@ -768,22 +821,18 @@ gint main(gint argc, gchar *argv[])
 	file_data_register_notify_func(histogram_notify_cb, NULL, NOTIFY_PRIORITY_HIGH);
 	file_data_register_notify_func(collect_manager_notify_cb, NULL, NOTIFY_PRIORITY_LOW);
 	file_data_register_notify_func(metadata_notify_cb, NULL, NOTIFY_PRIORITY_LOW);
-	
+
 
 	gtkrc_load();
 
 	parse_command_line_for_debug_option(argc, argv);
-	DEBUG_1("%s main: gtk_init", get_exec_time());	 
-#ifdef HAVE_LIBCHAMPLAIN
-#ifdef HAVE_LIBCHAMPLAIN_GTK
+	DEBUG_1("%s main: gtk_init", get_exec_time());
+#ifdef HAVE_CLUTTER
 	if (gtk_clutter_init(&argc, &argv) != CLUTTER_INIT_SUCCESS)
 		{
 		log_printf("Can't initialize clutter-gtk.\n");
 		exit(1);
 		}
-#else
-	gtk_init(&argc, &argv);
-#endif
 #else
 	gtk_init(&argc, &argv);
 #endif
@@ -798,17 +847,17 @@ gint main(gint argc, gchar *argv[])
 		log_printf("!!! %s may quit unexpectedly with a relocation error.\n", GQ_APPNAME);
 		}
 
-	DEBUG_1("%s main: pixbuf_inline_register_stock_icons", get_exec_time());	 
+	DEBUG_1("%s main: pixbuf_inline_register_stock_icons", get_exec_time());
 	pixbuf_inline_register_stock_icons();
 
-	DEBUG_1("%s main: setting default options before commandline handling", get_exec_time());	 
+	DEBUG_1("%s main: setting default options before commandline handling", get_exec_time());
 	options = init_options(NULL);
 	setup_default_options(options);
 
-	DEBUG_1("%s main: parse_command_line", get_exec_time());	 
+	DEBUG_1("%s main: parse_command_line", get_exec_time());
 	parse_command_line(argc, argv);
 
-	DEBUG_1("%s main: mkdir_if_not_exists", get_exec_time());	 
+	DEBUG_1("%s main: mkdir_if_not_exists", get_exec_time());
 	/* these functions don't depend on config file */
 	mkdir_if_not_exists(get_rc_dir());
 	mkdir_if_not_exists(get_collections_dir());
@@ -823,16 +872,16 @@ gint main(gint argc, gchar *argv[])
 	/* restore session from the config file */
 
 
-	DEBUG_1("%s main: load_options", get_exec_time());	 
+	DEBUG_1("%s main: load_options", get_exec_time());
 	if (!load_options(options))
 		{
 		/* load_options calls these functions after it parses global options, we have to call it here if it fails */
 		filter_add_defaults();
-		filter_rebuild(); 
+		filter_rebuild();
 		}
 
 	/* handle missing config file and commandline additions*/
-	if (!layout_window_list) 
+	if (!layout_window_list)
 		{
 		/* broken or no config file */
 		layout_new_from_config(NULL, NULL, TRUE);
@@ -858,6 +907,15 @@ gint main(gint argc, gchar *argv[])
 			}
 		}
 
+	if (command_line->log_file)
+		{
+		gchar *pathl;
+		gchar *path = g_strdup(command_line->log_file);
+
+		pathl = path_from_utf8(path);
+		command_line->ssi = secure_open(pathl);
+		}
+
 	if (command_line->cmd_list ||
 	    (command_line->startup_command_line_collection && command_line->collection_list))
 		{
@@ -916,8 +974,8 @@ gint main(gint argc, gchar *argv[])
 	buf = g_build_filename(get_rc_dir(), ".command", NULL);
 	remote_connection = remote_server_init(buf, cd);
 	g_free(buf);
-	
-	DEBUG_1("%s main: gtk_main", get_exec_time());	 
+
+	DEBUG_1("%s main: gtk_main", get_exec_time());
 	gtk_main();
 #ifdef HAVE_GTHREAD
 	gdk_threads_leave();
