@@ -1,19 +1,29 @@
 /*
- * Geeqie
- * (C) 2004 John Ellis
- * Copyright (C) 2008 - 2012 The Geeqie Team
+ * Copyright (C) 2004 John Ellis
+ * Copyright (C) 2008 - 2016 The Geeqie Team
  *
  * Author: John Ellis
  *
- * This software is released under the GNU General Public License (GNU GPL).
- * Please read the included file COPYING for more information.
- * This software comes with no warranty of any kind, use at your own risk!
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include "main.h"
 #include "print.h"
 
 #include "filedata.h"
+#include "filefilter.h"
 #include "image.h"
 #include "image-load.h"
 #include "pixbuf_util.h"
@@ -1367,10 +1377,10 @@ static gboolean print_job_ps_page_text(PrintWindow *pw, const gchar *text, gdoub
 	pixbuf_draw_layout(pixbuf, layout, pw->dialog->dialog, 0, 0, r, g, b, 255);
 	g_object_unref(G_OBJECT(layout));
 
-	ret = print_job_ps_page_image(pw, pixbuf, x, y, 
+	ret = print_job_ps_page_image(pw, pixbuf, x, y,
 				       /* do not allow rescaling of the pixbuf due to rounding errors */
 	                              ((gdouble)lw + 0.01) / scale_to_max_dpi,
-	                              ((gdouble)lh + 0.01) / scale_to_max_dpi, 
+	                              ((gdouble)lh + 0.01) / scale_to_max_dpi,
 	                              0, 0);
 
 	g_object_unref(G_OBJECT(pixbuf));
@@ -1440,7 +1450,7 @@ static gboolean print_job_rgb_page_new(PrintWindow *pw, gint page)
 		const gchar *ext;
 		gchar *base;
 
-		ext = extension_from_path(pw->output_path);
+		ext = registered_extension_from_path(pw->output_path);
 
 		if (ext)
 			{
@@ -1959,18 +1969,14 @@ static void print_job_throw_error(PrintWindow *pw, const gchar *message)
 	GtkWidget *label;
 	gchar *buf;
 
-#if GTK_CHECK_VERSION(2,20,0)
 	if (gtk_widget_get_visible(pw->dialog->dialog)) parent = pw->dialog->dialog;
-#else
-	if (GTK_WIDGET_VISIBLE(pw->dialog->dialog)) parent = pw->dialog->dialog;
-#endif
 
 	gd = generic_dialog_new(_("Printing error"), "print_warning",
 				parent, TRUE, NULL, NULL);
 	generic_dialog_add_button(gd, GTK_STOCK_OK, NULL, NULL, TRUE);
 
 	buf = g_strdup_printf(_("An error occured printing to %s."), print_output_name(pw->output));
-	generic_dialog_add_message(gd, GTK_STOCK_DIALOG_ERROR, _("Printing error"), buf);
+	generic_dialog_add_message(gd, GTK_STOCK_DIALOG_ERROR, _("Printing error"), buf, TRUE);
 	g_free(buf);
 
 	group = pref_group_new(gd->vbox, FALSE, _("Details"), GTK_ORIENTATION_VERTICAL);
@@ -2061,7 +2067,6 @@ static void print_job_render_image_loader_done(ImageLoader *il, gpointer data)
 		gdouble sw, sh;
 		gdouble dw, dh;
 		gdouble x, y, w, h;
-		gdouble scale;
 		gdouble offx, offy;
 
 		sw = (gdouble)gdk_pixbuf_get_width(pixbuf);
@@ -2074,13 +2079,11 @@ static void print_job_render_image_loader_done(ImageLoader *il, gpointer data)
 			{
 			w = dw;
 			h = dw / sw * sh;
-			scale = w / sw;
 			}
 		else
 			{
 			h = dh;
 			w = dh / sh *sw;
-			scale = h / sh;
 			}
 
 		if (pw->image_scale >= 5.0)
@@ -2191,7 +2194,6 @@ static void print_job_render_proof_loader_done(ImageLoader *il, gpointer data)
 	gdouble w, h;
 	gdouble proof_w, proof_h;
 	gdouble icon_w, icon_h;
-	gdouble scale;
 	gboolean success = TRUE;
 
 	if (pw->proof_columns < 1 || pw->proof_rows < 1)
@@ -2206,6 +2208,10 @@ static void print_job_render_proof_loader_done(ImageLoader *il, gpointer data)
 
 	pixbuf = image_loader_get_pixbuf(il);
 
+	if (options->image.exif_proof_rotate_enable == TRUE) {
+		pixbuf = pixbuf_apply_orientation(pixbuf, il->fd->exif_orientation);
+	}
+
 	w = gdk_pixbuf_get_width(pixbuf);
 	h = gdk_pixbuf_get_height(pixbuf);
 
@@ -2213,13 +2219,11 @@ static void print_job_render_proof_loader_done(ImageLoader *il, gpointer data)
 		{
 		icon_w = pw->proof_width;
 		icon_h = pw->proof_width / w * h;
-		scale = icon_w / w;
 		}
 	else
 		{
 		icon_h = pw->proof_height;
 		icon_w = pw->proof_height / h * w;
-		scale = icon_h / h;
 		}
 
 	y = pw->proof_position / pw->proof_columns;
@@ -2485,11 +2489,7 @@ static void print_job_close(PrintWindow *pw, gint error)
 		pw->job_pixbuf = NULL;
 		}
 
-#if GTK_CHECK_VERSION(2,20,0)
 	if (pw->dialog && !gtk_widget_get_visible(pw->dialog->dialog))
-#else
-	if (pw->dialog && !GTK_WIDGET_VISIBLE(pw->dialog->dialog))
-#endif
 		{
 		g_idle_add_full(G_PRIORITY_HIGH_IDLE, print_job_close_finish_cb, pw, NULL);
 		}
@@ -2599,7 +2599,7 @@ static gboolean print_job_start(PrintWindow *pw, RenderFormat format, PrintOutpu
 					   print_job_cancel_cb, pw);
 
 	msg = g_strdup_printf(_("Printing %d pages to %s."), print_layout_page_count(pw), print_output_name(pw->output));
-	generic_dialog_add_message(pw->job_dialog, NULL, msg, NULL);
+	generic_dialog_add_message(pw->job_dialog, NULL, msg, NULL, TRUE);
 	g_free(msg);
 
 	if (pw->job_output == PRINT_OUTPUT_PS_FILE ||
@@ -2668,11 +2668,11 @@ static GtkWidget *print_combo_menu(const gchar *text[], gint count, gint preferr
 	GtkWidget *combo;
 	gint i;
 
-	combo = gtk_combo_box_new_text();
+	combo = gtk_combo_box_text_new();
 
 	for (i = 0 ; i < count; i++)
 		{
-		gtk_combo_box_append_text(GTK_COMBO_BOX(combo), _(text[i]));
+		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(combo), _(text[i]));
 		}
 
 	if (preferred >= 0 && preferred < count)
@@ -2700,12 +2700,12 @@ static GtkWidget *print_paper_menu(GtkWidget *table, gint column, gint row,
 
 	pref_table_label(table, column, row, (_("Format:")), 1.0);
 
-	combo = gtk_combo_box_new_text();
+	combo = gtk_combo_box_text_new();
 
 	i = 0;
 	while (print_paper_sizes[i].description)
 		{
-		gtk_combo_box_append_text(GTK_COMBO_BOX(combo), _(print_paper_sizes[i].description));
+		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(combo), _(print_paper_sizes[i].description));
 		i++;
 		}
 
@@ -2981,7 +2981,7 @@ static void print_custom_entry_set(PrintWindow *pw, GtkWidget *combo)
 		work = work->next;
 
 		buf = g_strdup_printf(PRINT_LPR_CUSTOM, name);
-		gtk_combo_box_append_text(GTK_COMBO_BOX(combo), buf);
+		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(combo), buf);
 		g_free(buf);
 		}
 	string_list_free(list);
@@ -3222,12 +3222,6 @@ static void print_text_menu(GtkWidget *box, PrintWindow *pw)
 	pref_spin_new(group, _("Size:"), _("points"),
 		      8.0, 100.0, 1.0, 0, pw->text_points,
 		      G_CALLBACK(print_text_cb_points), pw);
-
-#if 0
-	button = color_selection_new();
-	gtk_box_pack_start(GTK_BOX(group), button, FALSE, FALSE, 0);
-	gtk_widget_show(button);
-#endif
 }
 
 /*
@@ -3320,9 +3314,9 @@ void print_window_new(FileData *fd, GList *selection, GList *list, GtkWidget *pa
 
 	pw->source = print_pref_int(PRINT_PREF_SOURCE, PRINT_SOURCE_SELECTION);
 	pw->layout = print_pref_int(PRINT_PREF_LAYOUT, PRINT_LAYOUT_IMAGE);
-	
+
 	pw->image_scale = print_pref_double(PRINT_PREF_IMAGE_SCALE, 100.0);
-	
+
 	pw->output = print_pref_int(PRINT_PREF_OUTPUT, PRINT_OUTPUT_PS_LPR);
 	pw->output_format = print_pref_int(PRINT_PREF_FORMAT, PRINT_FILE_JPG_NORMAL);
 
