@@ -1,13 +1,22 @@
 /*
- * Geeqie
- * (C) 2006 John Ellis
- * Copyright (C) 2008 - 2012 The Geeqie Team
+ * Copyright (C) 2006 John Ellis
+ * Copyright (C) 2008 - 2016 The Geeqie Team
  *
  * Author: John Ellis
  *
- * This software is released under the GNU General Public License (GNU GPL).
- * Please read the included file COPYING for more information.
- * This software comes with no warranty of any kind, use at your own risk!
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include <glib/gstdio.h>
@@ -34,7 +43,8 @@
 #include "bar.h"
 #include "metadata.h"
 #include "bar_gps.h"
-
+#include "dupe.h"
+#include "ui_utildlg.h"
 
 /*
  *-----------------------------------------------------------------------------
@@ -49,7 +59,7 @@ void write_indent(GString *str, gint indent)
 
 void write_char_option(GString *str, gint indent, const gchar *label, const gchar *text)
 {
-	/* this is needed for overlay string, because g_markup_escape_text does not handle \n and such, 
+	/* this is needed for overlay string, because g_markup_escape_text does not handle \n and such,
 	   ideas for improvement are welcome
 	*/
 	static const unsigned char no_quote_utf[] = {
@@ -67,13 +77,22 @@ void write_char_option(GString *str, gint indent, const gchar *label, const gcha
 		'"',  0 /* '"' is handled in g_markup_escape_text */
 	};
 
-	gchar *escval1 = g_strescape(text ? text : "", (gchar *) no_quote_utf); 
+	gchar *escval1 = g_strescape(text ? text : "", (gchar *) no_quote_utf);
 	gchar *escval2 = g_markup_escape_text(escval1, -1);
 	g_string_append_printf(str, "%s = \"%s\" ", label, escval2);
 	g_free(escval2);
 	g_free(escval1);
 }
 
+/* dummy read for old/obsolete/futur/deprecated/unused options */
+gboolean read_dummy_option(const gchar *option, const gchar *label, const gchar *message)
+{
+	if (g_ascii_strcasecmp(option, label) != 0) return FALSE;
+	log_printf(_("Option %s ignored: %s\n"), option, message);
+	return TRUE;
+}
+
+
 gboolean read_char_option(const gchar *option, const gchar *label, const gchar *value, gchar **text)
 {
 	if (g_ascii_strcasecmp(option, label) != 0) return FALSE;
@@ -139,6 +158,31 @@ gboolean read_int_option(const gchar *option, const gchar *label, const gchar *v
 	return TRUE;
 }
 
+void write_ushort_option(GString *str, gint indent, const gchar *label, guint16 n)
+{
+	g_string_append_printf(str, "%s = \"%uh\" ", label, n);
+}
+
+gboolean read_ushort_option(const gchar *option, const gchar *label, const gchar *value, guint16 *n)
+{
+	if (g_ascii_strcasecmp(option, label) != 0) return FALSE;
+	if (!n) return FALSE;
+
+	if (g_ascii_isdigit(value[0]))
+		{
+		*n = strtoul(value, NULL, 10);
+		}
+	else
+		{
+		if (g_ascii_strcasecmp(value, "true") == 0)
+			*n = 1;
+		else
+			*n = 0;
+		}
+
+	return TRUE;
+}
+
 void write_uint_option(GString *str, gint indent, const gchar *label, guint n)
 {
 	g_string_append_printf(str, "%s = \"%u\" ", label, n);
@@ -160,7 +204,7 @@ gboolean read_uint_option(const gchar *option, const gchar *label, const gchar *
 		else
 			*n = 0;
 		}
-	
+
 	return TRUE;
 }
 
@@ -230,7 +274,7 @@ gboolean read_int_unit_option(const gchar *option, const gchar *label, const gch
 
 	*n = l * subunits + r;
 	g_free(buf);
-	
+
 	return TRUE;
 }
 
@@ -260,31 +304,45 @@ gboolean read_bool_option(const gchar *option, const gchar *label, const gchar *
 
 static void write_global_attributes(GString *outstr, gint indent)
 {
-//	WRITE_SUBTITLE("General Options");
-
+	/* General Options */
 	WRITE_NL(); WRITE_BOOL(*options, show_icon_names);
 	WRITE_SEPARATOR();
 
 	WRITE_NL(); WRITE_BOOL(*options, tree_descend_subdirs);
+	WRITE_NL(); WRITE_BOOL(*options, view_dir_list_single_click_enter);
 	WRITE_NL(); WRITE_BOOL(*options, lazy_image_sync);
 	WRITE_NL(); WRITE_BOOL(*options, update_on_time_change);
 	WRITE_SEPARATOR();
 
 	WRITE_NL(); WRITE_BOOL(*options, progressive_key_scrolling);
+	WRITE_NL(); WRITE_UINT(*options, keyboard_scroll_step);
 
 	WRITE_NL(); WRITE_UINT(*options, duplicates_similarity_threshold);
+	WRITE_NL(); WRITE_UINT(*options, duplicates_match);
+	WRITE_NL(); WRITE_UINT(*options, duplicates_select_type);
+	WRITE_NL(); WRITE_BOOL(*options, duplicates_thumbnails);
+	WRITE_NL(); WRITE_BOOL(*options, rot_invariant_sim);
+	WRITE_NL(); WRITE_BOOL(*options, sort_totals);
 	WRITE_SEPARATOR();
 
 	WRITE_NL(); WRITE_BOOL(*options, mousewheel_scrolls);
+	WRITE_NL(); WRITE_BOOL(*options, image_lm_click_nav);
+	WRITE_NL(); WRITE_BOOL(*options, image_l_click_video);
+	WRITE_NL(); WRITE_CHAR(*options, image_l_click_video_editor);
 	WRITE_NL(); WRITE_INT(*options, open_recent_list_maxsize);
 	WRITE_NL(); WRITE_INT(*options, dnd_icon_size);
 	WRITE_NL(); WRITE_BOOL(*options, place_dialogs_under_mouse);
+	WRITE_NL(); WRITE_INT(*options, clipboard_selection);
 
 	WRITE_NL(); WRITE_BOOL(*options, save_window_positions);
+	WRITE_NL(); WRITE_BOOL(*options, use_saved_window_positions_for_new_windows);
 	WRITE_NL(); WRITE_BOOL(*options, tools_restore_state);
+	WRITE_NL(); WRITE_BOOL(*options, save_dialog_window_positions);
 
-//	WRITE_SUBTITLE("File operations Options");
+	WRITE_NL(); WRITE_UINT(*options, log_window_lines);
+	WRITE_NL(); WRITE_BOOL(*options, log_window.timer_data);
 
+	/* File operations Options */
 	WRITE_NL(); WRITE_BOOL(*options, file_ops.enable_in_place_rename);
 	WRITE_NL(); WRITE_BOOL(*options, file_ops.confirm_delete);
 	WRITE_NL(); WRITE_BOOL(*options, file_ops.enable_delete_key);
@@ -292,27 +350,12 @@ static void write_global_attributes(GString *outstr, gint indent)
 	WRITE_NL(); WRITE_CHAR(*options, file_ops.safe_delete_path);
 	WRITE_NL(); WRITE_INT(*options, file_ops.safe_delete_folder_maxsize);
 
-
-
-
-//	WRITE_SUBTITLE("Properties dialog Options");
+	/* Properties dialog Options */
 	WRITE_NL(); WRITE_CHAR(*options, properties.tabs_order);
 
-//	WRITE_SUBTITLE("Image Options");
-
+	/* Image Options */
 	WRITE_NL(); WRITE_UINT(*options, image.zoom_mode);
 
-//	g_string_append_printf(outstr, "# image.zoom_mode possible values are:\n"
-//			    "#   original\n"
-//			    "#   fit\n"
-//			    "#   dont_change\n");
-//	g_string_append_printf(outstr, "image.zoom_mode: ");
-//	switch (options->image.zoom_mode)
-//	{
-//	case ZOOM_RESET_ORIGINAL: g_string_append_printf(outstr, "original\n"); break;
-//	case ZOOM_RESET_FIT_WINDOW: g_string_append_printf(outstr, "fit\n"); break;
-//	case ZOOM_RESET_NONE: g_string_append_printf(outstr, "dont_change\n"); break;
-//	}
 	WRITE_SEPARATOR();
 	WRITE_NL(); WRITE_BOOL(*options, image.zoom_2pass);
 	WRITE_NL(); WRITE_BOOL(*options, image.zoom_to_fit_allow_expand);
@@ -323,18 +366,20 @@ static void write_global_attributes(GString *outstr, gint indent)
 	WRITE_NL(); WRITE_INT(*options, image.max_window_size);
 	WRITE_NL(); WRITE_BOOL(*options, image.limit_autofit_size);
 	WRITE_NL(); WRITE_INT(*options, image.max_autofit_size);
+	WRITE_NL(); WRITE_INT(*options, image.max_enlargement_size);
 	WRITE_NL(); WRITE_UINT(*options, image.scroll_reset_method);
 	WRITE_NL(); WRITE_INT(*options, image.tile_cache_max);
 	WRITE_NL(); WRITE_INT(*options, image.image_cache_max);
-	WRITE_NL(); WRITE_UINT(*options, image.dither_quality);
 	WRITE_NL(); WRITE_BOOL(*options, image.enable_read_ahead);
 	WRITE_NL(); WRITE_BOOL(*options, image.exif_rotate_enable);
 	WRITE_NL(); WRITE_BOOL(*options, image.use_custom_border_color);
 	WRITE_NL(); WRITE_BOOL(*options, image.use_custom_border_color_in_fullscreen);
 	WRITE_NL(); WRITE_COLOR(*options, image.border_color);
+	WRITE_NL(); WRITE_COLOR(*options, image.alpha_color_1);
+	WRITE_NL(); WRITE_COLOR(*options, image.alpha_color_2);
+	WRITE_NL(); WRITE_BOOL(*options, image.use_clutter_renderer);
 
-//	WRITE_SUBTITLE("Thumbnails Options");
-
+	/* Thumbnails Options */
 	WRITE_NL(); WRITE_INT(*options, thumbnails.max_width);
 	WRITE_NL(); WRITE_INT(*options, thumbnails.max_height);
 	WRITE_NL(); WRITE_BOOL(*options, thumbnails.enable_caching);
@@ -343,17 +388,16 @@ static void write_global_attributes(GString *outstr, gint indent)
 	WRITE_NL(); WRITE_BOOL(*options, thumbnails.spec_standard);
 	WRITE_NL(); WRITE_UINT(*options, thumbnails.quality);
 	WRITE_NL(); WRITE_BOOL(*options, thumbnails.use_exif);
+	WRITE_NL(); WRITE_BOOL(*options, thumbnails.use_ft_metadata);
+// 	WRITE_NL(); WRITE_BOOL(*options, thumbnails.use_ft_metadata_small);
 
-
-//	WRITE_SUBTITLE("File sorting Options");
-
+	/* File sorting Options */
 	WRITE_NL(); WRITE_INT(*options, file_sort.method);
 	WRITE_NL(); WRITE_BOOL(*options, file_sort.ascending);
 	WRITE_NL(); WRITE_BOOL(*options, file_sort.case_sensitive);
+	WRITE_NL(); WRITE_BOOL(*options, file_sort.natural);
 
-
-//	WRITE_SUBTITLE("Fullscreen Options");
-
+	/* Fullscreen Options */
 	WRITE_NL(); WRITE_INT(*options, fullscreen.screen);
 	WRITE_NL(); WRITE_BOOL(*options, fullscreen.clean_flip);
 	WRITE_NL(); WRITE_BOOL(*options, fullscreen.disable_saver);
@@ -361,80 +405,57 @@ static void write_global_attributes(GString *outstr, gint indent)
 
 	WRITE_SEPARATOR();
 
-//	WRITE_SUBTITLE("Image Overlay Options");
+	/* Image Overlay Options */
 	WRITE_NL(); WRITE_CHAR(*options, image_overlay.template_string);
 
-//	g_string_append_printf(outstr, "# these are relative positions:\n");
-//	g_string_append_printf(outstr, "# x >= 0: |x| pixels from left border\n");
-//	g_string_append_printf(outstr, "# x < 0 : |x| pixels from right border\n");
-//	g_string_append_printf(outstr, "# y >= 0: |y| pixels from top border\n");
-//	g_string_append_printf(outstr, "# y < 0 : |y| pixels from bottom border\n");
 	WRITE_NL(); WRITE_INT(*options, image_overlay.x);
 	WRITE_NL(); WRITE_INT(*options, image_overlay.y);
+	WRITE_NL(); WRITE_INT(*options, image_overlay.text_red);
+	WRITE_NL(); WRITE_INT(*options, image_overlay.text_green);
+	WRITE_NL(); WRITE_INT(*options, image_overlay.text_blue);
+	WRITE_NL(); WRITE_INT(*options, image_overlay.text_alpha);
+	WRITE_NL(); WRITE_INT(*options, image_overlay.background_red);
+	WRITE_NL(); WRITE_INT(*options, image_overlay.background_green);
+	WRITE_NL(); WRITE_INT(*options, image_overlay.background_blue);
+	WRITE_NL(); WRITE_INT(*options, image_overlay.background_alpha);
+	WRITE_NL(); WRITE_CHAR(*options, image_overlay.font);
 
-
-//	WRITE_SUBTITLE("Slideshow Options");
-
+	/* Slideshow Options */
 	WRITE_NL(); WRITE_INT_UNIT(*options, slideshow.delay, SLIDESHOW_SUBSECOND_PRECISION);
 	WRITE_NL(); WRITE_BOOL(*options, slideshow.random);
 	WRITE_NL(); WRITE_BOOL(*options, slideshow.repeat);
 
-
-//	WRITE_SUBTITLE("Collection Options");
-
+	/* Collection Options */
 	WRITE_NL(); WRITE_BOOL(*options, collections.rectangular_selection);
 
-
-//	WRITE_SUBTITLE("Filtering Options");
-
+	/* Filtering Options */
 	WRITE_NL(); WRITE_BOOL(*options, file_filter.show_hidden_files);
+	WRITE_NL(); WRITE_BOOL(*options, file_filter.show_parent_directory);
 	WRITE_NL(); WRITE_BOOL(*options, file_filter.show_dot_directory);
+	WRITE_NL(); WRITE_BOOL(*options, file_filter.disable_file_extension_checks);
 	WRITE_NL(); WRITE_BOOL(*options, file_filter.disable);
 	WRITE_SEPARATOR();
 
-
-//	WRITE_SUBTITLE("Sidecars Options");
-
+	/* Sidecars Options */
 	WRITE_NL(); WRITE_CHAR(*options, sidecar.ext);
 
-
-
-//	WRITE_SUBTITLE("Shell command");
+	/* Shell command */
 	WRITE_NL(); WRITE_CHAR(*options, shell.path);
 	WRITE_NL(); WRITE_CHAR(*options, shell.options);
 
-
-//	WRITE_SUBTITLE("Helpers");
-//	g_string_append_printf(outstr, "# Html browser\n");
-//	g_string_append_printf(outstr, "# command_name is: the binary's name to look for in the path\n");
-//	g_string_append_printf(outstr, "# If command_name is empty, the program will try various common html browsers\n");
-//	g_string_append_printf(outstr, "# command_line is:\n");
-//	g_string_append_printf(outstr, "# \"\" (empty string)  = execute binary with html file path as command line\n");
-//	g_string_append_printf(outstr, "# \"string\"           = execute string and use results for command line\n");
-//	g_string_append_printf(outstr, "# \"!string\"          = use text following ! as command line, replacing optional %%s with html file path\n");
+	/* Helpers */
 	WRITE_NL(); WRITE_CHAR(*options, helpers.html_browser.command_name);
 	WRITE_NL(); WRITE_CHAR(*options, helpers.html_browser.command_line);
 
-/* FIXME:
-	WRITE_SUBTITLE("Exif Options");
-	g_string_append_printf(outstr, "# Display: 0: never\n"
-			    "#          1: if set\n"
-			    "#          2: always\n\n");
-	for (i = 0; ExifUIList[i].key; i++)
-		{
-		g_string_append_printf(outstr, "exif.display.");
-		write_int_option(outstr, 2, (gchar *)ExifUIList[i].key, ExifUIList[i].current);
-		}
-*/
-
-//	WRITE_SUBTITLE("Metadata Options");
+	/* Metadata Options */
 	WRITE_NL(); WRITE_BOOL(*options, metadata.enable_metadata_dirs);
-	WRITE_NL(); WRITE_BOOL(*options, metadata.save_in_image_file); 
+	WRITE_NL(); WRITE_BOOL(*options, metadata.save_in_image_file);
 	WRITE_NL(); WRITE_BOOL(*options, metadata.save_legacy_IPTC);
 	WRITE_NL(); WRITE_BOOL(*options, metadata.warn_on_write_problems);
 	WRITE_NL(); WRITE_BOOL(*options, metadata.save_legacy_format);
 	WRITE_NL(); WRITE_BOOL(*options, metadata.sync_grouped_files);
 	WRITE_NL(); WRITE_BOOL(*options, metadata.confirm_write);
+	WRITE_NL(); WRITE_BOOL(*options, metadata.sidecar_extended_name);
 	WRITE_NL(); WRITE_INT(*options, metadata.confirm_timeout);
 	WRITE_NL(); WRITE_BOOL(*options, metadata.confirm_after_timeout);
 	WRITE_NL(); WRITE_BOOL(*options, metadata.confirm_on_image_change);
@@ -442,15 +463,21 @@ static void write_global_attributes(GString *outstr, gint indent)
 	WRITE_NL(); WRITE_BOOL(*options, metadata.keywords_case_sensitive);
 	WRITE_NL(); WRITE_BOOL(*options, metadata.write_orientation);
 
-	WRITE_NL(); WRITE_UINT(*options, stereo.mode);
-	WRITE_NL(); WRITE_UINT(*options, stereo.fsmode);
+	WRITE_NL(); WRITE_INT(*options, stereo.mode);
+	WRITE_NL(); WRITE_INT(*options, stereo.fsmode);
 	WRITE_NL(); WRITE_BOOL(*options, stereo.enable_fsmode);
-	WRITE_NL(); WRITE_UINT(*options, stereo.fixed_w);
-	WRITE_NL(); WRITE_UINT(*options, stereo.fixed_h);
-	WRITE_NL(); WRITE_UINT(*options, stereo.fixed_x1);
-	WRITE_NL(); WRITE_UINT(*options, stereo.fixed_y1);
-	WRITE_NL(); WRITE_UINT(*options, stereo.fixed_x2);
-	WRITE_NL(); WRITE_UINT(*options, stereo.fixed_y2);
+	WRITE_NL(); WRITE_INT(*options, stereo.fixed_w);
+	WRITE_NL(); WRITE_INT(*options, stereo.fixed_h);
+	WRITE_NL(); WRITE_INT(*options, stereo.fixed_x1);
+	WRITE_NL(); WRITE_INT(*options, stereo.fixed_y1);
+	WRITE_NL(); WRITE_INT(*options, stereo.fixed_x2);
+	WRITE_NL(); WRITE_INT(*options, stereo.fixed_y2);
+
+	/* copy move rename */
+	WRITE_NL(); WRITE_INT(*options, cp_mv_rn.auto_start);
+	WRITE_NL(); WRITE_INT(*options, cp_mv_rn.auto_padding);
+	WRITE_NL(); WRITE_CHAR(*options, cp_mv_rn.auto_end);
+	WRITE_NL(); WRITE_INT(*options, cp_mv_rn.formatted_start);
 }
 
 static void write_color_profile(GString *outstr, gint indent)
@@ -467,6 +494,7 @@ static void write_color_profile(GString *outstr, gint indent)
 	WRITE_BOOL(options->color_profile, use_image);
 	WRITE_INT(options->color_profile, input_type);
 	WRITE_BOOL(options->color_profile, use_x11_screen_profile);
+	WRITE_INT(options->color_profile, render_intent);
 	WRITE_STRING(">");
 
 	indent++;
@@ -495,7 +523,7 @@ gboolean save_config_to_file(const gchar *utf8_path, ConfOptions *options)
 	GString *outstr;
 	gint indent = 0;
 	GList *work;
-	
+
 	rc_pathl = path_from_utf8(utf8_path);
 	ssi = secure_open(rc_pathl);
 	g_free(rc_pathl);
@@ -520,7 +548,7 @@ gboolean save_config_to_file(const gchar *utf8_path, ConfOptions *options)
 
 	WRITE_STRING("<gq>\n");
 	indent++;
-	
+
 	WRITE_NL(); WRITE_STRING("<global\n");
 	indent++;
 	write_global_attributes(outstr, indent + 1);
@@ -540,8 +568,8 @@ gboolean save_config_to_file(const gchar *utf8_path, ConfOptions *options)
 	WRITE_NL(); WRITE_STRING("</global>\n");
 
 	WRITE_SEPARATOR();
-	WRITE_SUBTITLE("Layout Options");
 
+	/* Layout Options */
 	work = layout_window_list;
 	while (work)
 		{
@@ -581,31 +609,46 @@ static gboolean load_global_params(const gchar **attribute_names, const gchar **
 		const gchar *option = *attribute_names++;
 		const gchar *value = *attribute_values++;
 
-
-		/* general options */
+		/* General options */
 		if (READ_BOOL(*options, show_icon_names)) continue;
 
 		if (READ_BOOL(*options, tree_descend_subdirs)) continue;
+		if (READ_BOOL(*options, view_dir_list_single_click_enter)) continue;
 		if (READ_BOOL(*options, lazy_image_sync)) continue;
 		if (READ_BOOL(*options, update_on_time_change)) continue;
 
 		if (READ_UINT_CLAMP(*options, duplicates_similarity_threshold, 0, 100)) continue;
+		if (READ_UINT_CLAMP(*options, duplicates_match, 0, DUPE_MATCH_NAME_CI)) continue;
+		if (READ_UINT_CLAMP(*options, duplicates_select_type, 0, DUPE_SELECT_GROUP2)) continue;
+		if (READ_BOOL(*options, duplicates_thumbnails)) continue;
+		if (READ_BOOL(*options, rot_invariant_sim)) continue;
+		if (READ_BOOL(*options, sort_totals)) continue;
 
 		if (READ_BOOL(*options, progressive_key_scrolling)) continue;
+		if (READ_UINT_CLAMP(*options, keyboard_scroll_step, 1, 32)) continue;
 
 		if (READ_BOOL(*options, mousewheel_scrolls)) continue;
+		if (READ_BOOL(*options, image_lm_click_nav)) continue;
+		if (READ_BOOL(*options, image_l_click_video)) continue;
+		if (READ_CHAR(*options, image_l_click_video_editor)) continue;
 
 		if (READ_INT(*options, open_recent_list_maxsize)) continue;
 		if (READ_INT(*options, dnd_icon_size)) continue;
 		if (READ_BOOL(*options, place_dialogs_under_mouse)) continue;
+		if (READ_INT(*options, clipboard_selection)) continue;
 
 		if (READ_BOOL(*options, save_window_positions)) continue;
+		if (READ_BOOL(*options, use_saved_window_positions_for_new_windows)) continue;
 		if (READ_BOOL(*options, tools_restore_state)) continue;
+		if (READ_BOOL(*options, save_dialog_window_positions)) continue;
 
-		/* properties dialog options */
+		if (READ_INT(*options, log_window_lines)) continue;
+		if (READ_BOOL(*options, log_window.timer_data)) continue;
+
+		/* Properties dialog options */
 		if (READ_CHAR(*options, properties.tabs_order)) continue;
 
-		/* image options */
+		/* Image options */
 		if (READ_UINT_CLAMP(*options, image.zoom_mode, 0, ZOOM_RESET_NONE)) continue;
 		if (READ_BOOL(*options, image.zoom_2pass)) continue;
 		if (READ_BOOL(*options, image.zoom_to_fit_allow_expand)) continue;
@@ -614,19 +657,22 @@ static gboolean load_global_params(const gchar **attribute_names, const gchar **
 		if (READ_INT(*options, image.max_window_size)) continue;
 		if (READ_BOOL(*options, image.limit_autofit_size)) continue;
 		if (READ_INT(*options, image.max_autofit_size)) continue;
+		if (READ_INT(*options, image.max_enlargement_size)) continue;
 		if (READ_UINT_CLAMP(*options, image.scroll_reset_method, 0, PR_SCROLL_RESET_COUNT - 1)) continue;
 		if (READ_INT(*options, image.tile_cache_max)) continue;
 		if (READ_INT(*options, image.image_cache_max)) continue;
 		if (READ_UINT_CLAMP(*options, image.zoom_quality, GDK_INTERP_NEAREST, GDK_INTERP_HYPER)) continue;
-		if (READ_UINT_CLAMP(*options, image.dither_quality, GDK_RGB_DITHER_NONE, GDK_RGB_DITHER_MAX)) continue;
 		if (READ_INT(*options, image.zoom_increment)) continue;
 		if (READ_BOOL(*options, image.enable_read_ahead)) continue;
 		if (READ_BOOL(*options, image.exif_rotate_enable)) continue;
 		if (READ_BOOL(*options, image.use_custom_border_color)) continue;
 		if (READ_BOOL(*options, image.use_custom_border_color_in_fullscreen)) continue;
 		if (READ_COLOR(*options, image.border_color)) continue;
+		if (READ_COLOR(*options, image.alpha_color_1)) continue;
+		if (READ_COLOR(*options, image.alpha_color_2)) continue;
+		if (READ_BOOL(*options, image.use_clutter_renderer)) continue;
 
-		/* thumbnails options */
+		/* Thumbnails options */
 		if (READ_INT_CLAMP(*options, thumbnails.max_width, 16, 512)) continue;
 		if (READ_INT_CLAMP(*options, thumbnails.max_height, 16, 512)) continue;
 
@@ -636,13 +682,16 @@ static gboolean load_global_params(const gchar **attribute_names, const gchar **
 		if (READ_BOOL(*options, thumbnails.spec_standard)) continue;
 		if (READ_UINT_CLAMP(*options, thumbnails.quality, GDK_INTERP_NEAREST, GDK_INTERP_HYPER)) continue;
 		if (READ_BOOL(*options, thumbnails.use_exif)) continue;
+		if (READ_BOOL(*options, thumbnails.use_ft_metadata)) continue;
+// 		if (READ_BOOL(*options, thumbnails.use_ft_metadata_small)) continue;
 
-		/* file sorting options */
+		/* File sorting options */
 		if (READ_UINT(*options, file_sort.method)) continue;
 		if (READ_BOOL(*options, file_sort.ascending)) continue;
 		if (READ_BOOL(*options, file_sort.case_sensitive)) continue;
+		if (READ_BOOL(*options, file_sort.natural)) continue;
 
-		/* file operations *options */
+		/* File operations *options */
 		if (READ_BOOL(*options, file_ops.enable_in_place_rename)) continue;
 		if (READ_BOOL(*options, file_ops.confirm_delete)) continue;
 		if (READ_BOOL(*options, file_ops.enable_delete_key)) continue;
@@ -650,31 +699,39 @@ static gboolean load_global_params(const gchar **attribute_names, const gchar **
 		if (READ_CHAR(*options, file_ops.safe_delete_path)) continue;
 		if (READ_INT(*options, file_ops.safe_delete_folder_maxsize)) continue;
 
-		/* fullscreen options */
+		/* Fullscreen options */
 		if (READ_INT(*options, fullscreen.screen)) continue;
 		if (READ_BOOL(*options, fullscreen.clean_flip)) continue;
 		if (READ_BOOL(*options, fullscreen.disable_saver)) continue;
 		if (READ_BOOL(*options, fullscreen.above)) continue;
 
-		/* image overlay */
+		/* Image overlay */
 		if (READ_CHAR(*options, image_overlay.template_string)) continue;
 		if (READ_INT(*options, image_overlay.x)) continue;
 		if (READ_INT(*options, image_overlay.y)) continue;
+		if (READ_USHORT(*options, image_overlay.text_red)) continue;
+		if (READ_USHORT(*options, image_overlay.text_green)) continue;
+		if (READ_USHORT(*options, image_overlay.text_blue)) continue;
+		if (READ_USHORT(*options, image_overlay.text_alpha)) continue;
+		if (READ_USHORT(*options, image_overlay.background_red)) continue;
+		if (READ_USHORT(*options, image_overlay.background_green)) continue;
+		if (READ_USHORT(*options, image_overlay.background_blue)) continue;
+		if (READ_USHORT(*options, image_overlay.background_alpha)) continue;
+		if (READ_CHAR(*options, image_overlay.font)) continue;
 
-
-		/* slideshow options */
+		/* Slideshow options */
 		if (READ_INT_UNIT(*options, slideshow.delay, SLIDESHOW_SUBSECOND_PRECISION)) continue;
 		if (READ_BOOL(*options, slideshow.random)) continue;
 		if (READ_BOOL(*options, slideshow.repeat)) continue;
 
-		/* collection options */
-
+		/* Collection options */
 		if (READ_BOOL(*options, collections.rectangular_selection)) continue;
 
-		/* filtering options */
-
+		/* Filtering options */
 		if (READ_BOOL(*options, file_filter.show_hidden_files)) continue;
+		if (READ_BOOL(*options, file_filter.show_parent_directory)) continue;
 		if (READ_BOOL(*options, file_filter.show_dot_directory)) continue;
+		if (READ_BOOL(*options, file_filter.disable_file_extension_checks)) continue;
 		if (READ_BOOL(*options, file_filter.disable)) continue;
 		if (READ_CHAR(*options, sidecar.ext)) continue;
 
@@ -687,17 +744,8 @@ static gboolean load_global_params(const gchar **attribute_names, const gchar **
 		/* Helpers */
 		if (READ_CHAR(*options, helpers.html_browser.command_name)) continue;
 		if (READ_CHAR(*options, helpers.html_browser.command_line)) continue;
-		/* Exif */
-/*
-		if (0 == g_ascii_strncasecmp(option, "exif.display.", 13))
-			{
-			for (i = 0; ExifUIList[i].key; i++)
-				if (0 == g_ascii_strcasecmp(option + 13, ExifUIList[i].key))
-					ExifUIList[i].current = strtol(value, NULL, 10);
-			continue;
-			}
-*/
-		/* metadata */		
+
+		/* Metadata */
 		if (READ_BOOL(*options, metadata.enable_metadata_dirs)) continue;
 		if (READ_BOOL(*options, metadata.save_in_image_file)) continue;
 		if (READ_BOOL(*options, metadata.save_legacy_IPTC)) continue;
@@ -705,6 +753,7 @@ static gboolean load_global_params(const gchar **attribute_names, const gchar **
 		if (READ_BOOL(*options, metadata.save_legacy_format)) continue;
 		if (READ_BOOL(*options, metadata.sync_grouped_files)) continue;
 		if (READ_BOOL(*options, metadata.confirm_write)) continue;
+		if (READ_BOOL(*options, metadata.sidecar_extended_name)) continue;
 		if (READ_BOOL(*options, metadata.confirm_after_timeout)) continue;
 		if (READ_INT(*options, metadata.confirm_timeout)) continue;
 		if (READ_BOOL(*options, metadata.confirm_on_image_change)) continue;
@@ -712,16 +761,26 @@ static gboolean load_global_params(const gchar **attribute_names, const gchar **
 		if (READ_BOOL(*options, metadata.keywords_case_sensitive)) continue;
 		if (READ_BOOL(*options, metadata.write_orientation)) continue;
 
-		if (READ_UINT(*options, stereo.mode)) continue;
-		if (READ_UINT(*options, stereo.fsmode)) continue;
+		if (READ_INT(*options, stereo.mode)) continue;
+		if (READ_INT(*options, stereo.fsmode)) continue;
 		if (READ_BOOL(*options, stereo.enable_fsmode)) continue;
-		if (READ_UINT(*options, stereo.fixed_w)) continue;
-		if (READ_UINT(*options, stereo.fixed_h)) continue;
-		if (READ_UINT(*options, stereo.fixed_x1)) continue;
-		if (READ_UINT(*options, stereo.fixed_y1)) continue;
-		if (READ_UINT(*options, stereo.fixed_x2)) continue;
-		if (READ_UINT(*options, stereo.fixed_y2)) continue;
+		if (READ_INT(*options, stereo.fixed_w)) continue;
+		if (READ_INT(*options, stereo.fixed_h)) continue;
+		if (READ_INT(*options, stereo.fixed_x1)) continue;
+		if (READ_INT(*options, stereo.fixed_y1)) continue;
+		if (READ_INT(*options, stereo.fixed_x2)) continue;
+		if (READ_INT(*options, stereo.fixed_y2)) continue;
 
+		/* copy move rename */
+		if (READ_INT(*options, cp_mv_rn.auto_start))  continue;
+		if (READ_INT(*options, cp_mv_rn.auto_padding)) continue;
+		if (READ_CHAR(*options, cp_mv_rn.auto_end)) continue;
+		if (READ_INT(*options, cp_mv_rn.formatted_start)) continue;
+
+		/* Dummy options */
+		if (READ_DUMMY(*options, image.dither_quality, "deprecated since 2012-08-13")) continue;
+
+		/* Unknown options */
 		log_printf("unknown attribute %s = %s\n", option, value);
 		}
 
@@ -740,6 +799,7 @@ static void options_load_color_profiles(GQParserData *parser_data, GMarkupParseC
 		if (READ_INT(options->color_profile, input_type)) continue;
 		if (READ_CHAR(options->color_profile, screen_file)) continue;
 		if (READ_BOOL(options->color_profile, use_x11_screen_profile)) continue;
+		if (READ_INT(options->color_profile, render_intent)) continue;
 
 		log_printf("unknown attribute %s = %s\n", option, value);
 		}
@@ -784,7 +844,7 @@ static const gchar *options_get_id(const gchar **attribute_names, const gchar **
 		{
 		const gchar *option = *attribute_names++;
 		const gchar *value = *attribute_values++;
-		
+
 		if (strcmp(option, "id") == 0) return value;
 
 		}
@@ -829,7 +889,7 @@ static void options_parse_filter(GQParserData *parser_data, GMarkupParseContext
 static void options_parse_filter_end(GQParserData *parser_data, GMarkupParseContext *context, const gchar *element_name, gpointer data, GError **error)
 {
 	if (parser_data->startup) filter_add_defaults();
-	filter_rebuild(); 
+	filter_rebuild();
 }
 
 static void options_parse_keyword_end(GQParserData *parser_data, GMarkupParseContext *context, const gchar *element_name, gpointer data, GError **error)
@@ -920,6 +980,22 @@ static void options_parse_pane_exif(GQParserData *parser_data, GMarkupParseConte
 		}
 }
 
+static void options_parse_pane_keywords(GQParserData *parser_data, GMarkupParseContext *context, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer data, GError **error)
+{
+	GtkWidget *pane = data;
+
+	if (g_ascii_strcasecmp(element_name, "expanded") == 0)
+		{
+		bar_pane_keywords_entry_add_from_config(pane, attribute_names, attribute_values);
+		options_parse_func_push(parser_data, options_parse_leaf, NULL, NULL);
+		}
+	else
+		{
+		log_printf("unexpected in <pane_keywords>: <%s>\n", element_name);
+		options_parse_func_push(parser_data, options_parse_leaf, NULL, NULL);
+		}
+}
+
 static void options_parse_bar(GQParserData *parser_data, GMarkupParseContext *context, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer data, GError **error)
 {
 	GtkWidget *bar = data;
@@ -995,7 +1071,7 @@ static void options_parse_bar(GQParserData *parser_data, GMarkupParseContext *co
 			pane = bar_pane_keywords_new_from_config(attribute_names, attribute_values);
 			bar_add(bar, pane);
 			}
-		options_parse_func_push(parser_data, options_parse_leaf, NULL, NULL);
+		options_parse_func_push(parser_data, options_parse_pane_keywords, NULL, pane);
 		}
 	else if (g_ascii_strcasecmp(element_name, "clear") == 0)
 		{
@@ -1028,23 +1104,16 @@ static void options_parse_toolbar(GQParserData *parser_data, GMarkupParseContext
 		options_parse_func_push(parser_data, options_parse_leaf, NULL, NULL);
 		}
 }
-
-static void options_parse_statusbar(GQParserData *parser_data, GMarkupParseContext *context, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer data, GError **error)
+static void options_parse_dialogs(GQParserData *parser_data, GMarkupParseContext *context, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer data, GError **error)
 {
-	LayoutWindow *lw = data;
-	if (g_ascii_strcasecmp(element_name, "toolitem") == 0)
+	if (g_ascii_strcasecmp(element_name, "window") == 0)
 		{
-		layout_toolbar_add_from_config(lw, TOOLBAR_STATUS, attribute_names, attribute_values);
-		options_parse_func_push(parser_data, options_parse_leaf, NULL, NULL);
-		}
-	else if (g_ascii_strcasecmp(element_name, "clear") == 0)
-		{
-		layout_toolbar_clear(lw, TOOLBAR_STATUS);
+		generic_dialog_windows_load_config(attribute_names, attribute_values);
 		options_parse_func_push(parser_data, options_parse_leaf, NULL, NULL);
 		}
 	else
 		{
-		log_printf("unexpected in <statusbar>: <%s>\n", element_name);
+		log_printf("unexpected in <dialogs>: <%s>\n", element_name);
 		options_parse_func_push(parser_data, options_parse_leaf, NULL, NULL);
 		}
 }
@@ -1063,13 +1132,12 @@ static void options_parse_layout(GQParserData *parser_data, GMarkupParseContext
 			{
 			bar_update_from_config(lw->bar, attribute_names, attribute_values);
 			}
-			
+
 		options_parse_func_push(parser_data, options_parse_bar, NULL, lw->bar);
 		}
 	else if (g_ascii_strcasecmp(element_name, "bar_sort") == 0)
 		{
-		GtkWidget *bar = bar_sort_new_from_config(lw, attribute_names, attribute_values);
-		layout_bar_sort_set(lw, bar);
+		bar_sort_cold_start(lw, attribute_names, attribute_values);
 		options_parse_func_push(parser_data, options_parse_leaf, NULL, NULL);
 		}
 	else if (g_ascii_strcasecmp(element_name, "toolbar") == 0)
@@ -1078,7 +1146,11 @@ static void options_parse_layout(GQParserData *parser_data, GMarkupParseContext
 		}
 	else if (g_ascii_strcasecmp(element_name, "statusbar") == 0)
 		{
-		options_parse_func_push(parser_data, options_parse_statusbar, NULL, lw);
+		options_parse_func_push(parser_data, options_parse_leaf, NULL, NULL);
+		}
+	else if (g_ascii_strcasecmp(element_name, "dialogs") == 0)
+		{
+		options_parse_func_push(parser_data, options_parse_dialogs, NULL, NULL);
 		}
 	else
 		{
@@ -1107,12 +1179,12 @@ static void options_parse_toplevel(GQParserData *parser_data, GMarkupParseContex
 		options_parse_func_push(parser_data, options_parse_global, options_parse_global_end, NULL);
 		return;
 		}
-	
+
 	if (g_ascii_strcasecmp(element_name, "layout") == 0)
 		{
 		LayoutWindow *lw;
 		lw = layout_find_by_layout_id(options_get_id(attribute_names, attribute_values));
-		if (lw) 
+		if (lw)
 			{
 			layout_update_from_config(lw, attribute_names, attribute_values);
 			}
@@ -1144,7 +1216,6 @@ struct _GQParserFuncData
 {
 	GQParserStartFunc start_func;
 	GQParserEndFunc end_func;
-//	GQParserTextFunc text_func;
 	gpointer data;
 };
 
@@ -1154,7 +1225,7 @@ void options_parse_func_push(GQParserData *parser_data, GQParserStartFunc start_
 	func_data->start_func = start_func;
 	func_data->end_func = end_func;
 	func_data->data = data;
-	
+
 	parser_data->parse_func_stack = g_list_prepend(parser_data->parse_func_stack, func_data);
 }
 
@@ -1176,12 +1247,12 @@ static void start_element(GMarkupParseContext *context,
 			  const gchar **attribute_names,
 			  const gchar **attribute_values,
 			  gpointer user_data,
-			  GError **error) 
+			  GError **error)
 {
 	GQParserData *parser_data = user_data;
-	GQParserFuncData *func = parser_data->parse_func_stack->data; 
+	GQParserFuncData *func = parser_data->parse_func_stack->data;
 	DEBUG_2("start %s", element_name);
-	
+
 	if (func->start_func)
 		func->start_func(parser_data, context, element_name, attribute_names, attribute_values, func->data, error);
 }
@@ -1189,10 +1260,10 @@ static void start_element(GMarkupParseContext *context,
 static void end_element(GMarkupParseContext *context,
 			  const gchar *element_name,
 			  gpointer user_data,
-			  GError **error) 
+			  GError **error)
 {
 	GQParserData *parser_data = user_data;
-	GQParserFuncData *func = parser_data->parse_func_stack->data; 
+	GQParserFuncData *func = parser_data->parse_func_stack->data;
 	DEBUG_2("end %s", element_name);
 
 	if (func->end_func)
@@ -1222,10 +1293,10 @@ gboolean load_config_from_buf(const gchar *buf, gsize size, gboolean startup)
 	GQParserData *parser_data;
 
 	parser_data = g_new0(GQParserData, 1);
-	
+
 	parser_data->startup = startup;
 	options_parse_func_push(parser_data, options_parse_toplevel, NULL, NULL);
-	
+
 	context = g_markup_parse_context_new(&parser, 0, parser_data, NULL);
 
 	if (g_markup_parse_context_parse(context, buf, size, NULL) == FALSE)
@@ -1233,7 +1304,7 @@ gboolean load_config_from_buf(const gchar *buf, gsize size, gboolean startup)
 		ret = FALSE;
 		DEBUG_1("Parse failed");
 		}
-		
+
 	g_free(parser_data);
 
 	g_markup_parse_context_free(context);
@@ -1246,7 +1317,7 @@ gboolean load_config_from_file(const gchar *utf8_path, gboolean startup)
 	gchar *buf;
 	gboolean ret = TRUE;
 
-	if (g_file_get_contents(utf8_path, &buf, &size, NULL) == FALSE) 
+	if (g_file_get_contents(utf8_path, &buf, &size, NULL) == FALSE)
 		{
 		return FALSE;
 		}
@@ -1254,7 +1325,7 @@ gboolean load_config_from_file(const gchar *utf8_path, gboolean startup)
 	g_free(buf);
 	return ret;
 }
-	
+
 
 
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
