@@ -1,21 +1,31 @@
 /*
- * Geeqie
- * (C) 2004 John Ellis
- * Copyright (C) 2008 - 2012 The Geeqie Team
+ * Copyright (C) 2004 John Ellis
+ * Copyright (C) 2008 - 2016 The Geeqie Team
  *
  * Author: John Ellis
  *
- * This software is released under the GNU General Public License (GNU GPL).
- * Please read the included file COPYING for more information.
- * This software comes with no warranty of any kind, use at your own risk!
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-
 #include "main.h"
 #include "remote.h"
 
+#include "cache_maint.h"
 #include "collect.h"
 #include "filedata.h"
+#include "image.h"
 #include "img-view.h"
 #include "layout.h"
 #include "layout_image.h"
@@ -29,6 +39,7 @@
 #include <signal.h>
 #include <errno.h>
 
+#include "glua.h"
 
 #define SERVER_MAX_CLIENTS 8
 
@@ -42,6 +53,7 @@
 
 static RemoteConnection *remote_client_open(const gchar *path);
 static gint remote_client_send(RemoteConnection *rc, const gchar *text);
+static void gr_raise(const gchar *text, GIOChannel *channel, gpointer data);
 
 
 typedef struct _RemoteClient RemoteClient;
@@ -102,7 +114,7 @@ static gboolean remote_server_client_cb(GIOChannel *source, GIOCondition conditi
 
 		DEBUG_1("HUP detected, closing client.");
 		DEBUG_1("client count %d", g_list_length(rc->clients));
-		
+
 		g_source_remove(client->channel_id);
 		close(client->fd);
 		g_free(client);
@@ -212,14 +224,14 @@ static RemoteConnection *remote_server_open(const gchar *path)
 		}
 
 	rc = g_new0(RemoteConnection, 1);
-	
+
 	rc->server = TRUE;
 	rc->fd = fd;
 	rc->path = g_strdup(path);
 
 	channel = g_io_channel_unix_new(rc->fd);
-	g_io_channel_set_flags(channel, G_IO_FLAG_NONBLOCK, NULL); 
-	
+	g_io_channel_set_flags(channel, G_IO_FLAG_NONBLOCK, NULL);
+
 	rc->channel_id = g_io_add_watch_full(channel, G_PRIORITY_DEFAULT, G_IO_IN,
 					     remote_server_read_cb, rc, NULL);
 	g_io_channel_unref(channel);
@@ -338,7 +350,7 @@ static gboolean remote_client_send(RemoteConnection *rc, const gchar *text)
 			ret = FALSE;
 			}
 		}
-		
+
 
 	/* restore the original signal handler */
 	sigaction(SIGPIPE, &old_action, NULL);
@@ -420,6 +432,49 @@ static void gr_slideshow_start_rec(const gchar *text, GIOChannel *channel, gpoin
 	layout_image_slideshow_start_from_list(NULL, list);
 }
 
+static void gr_cache_thumb(const gchar *text, GIOChannel *channel, gpointer data)
+{
+	if (!g_strcmp0(text, "clear"))
+		cache_maintain_home_remote(FALSE, TRUE);
+	else if (!g_strcmp0(text, "clean"))
+		cache_maintain_home_remote(FALSE, FALSE);
+}
+
+static void gr_cache_shared(const gchar *text, GIOChannel *channel, gpointer data)
+{
+	if (!g_strcmp0(text, "clear"))
+		cache_manager_standard_process_remote(TRUE);
+	else if (!g_strcmp0(text, "clean"))
+		cache_manager_standard_process_remote(FALSE);
+}
+
+static void gr_cache_metadata(const gchar *text, GIOChannel *channel, gpointer data)
+{
+	cache_maintain_home_remote(TRUE, FALSE);
+}
+
+static void gr_cache_render(const gchar *text, GIOChannel *channel, gpointer data)
+{
+	cache_manager_render_remote(text, FALSE, FALSE);
+}
+
+static void gr_cache_render_recurse(const gchar *text, GIOChannel *channel, gpointer data)
+{
+	cache_manager_render_remote(text, TRUE, FALSE);
+}
+
+static void gr_cache_render_standard(const gchar *text, GIOChannel *channel, gpointer data)
+{
+	if(options->thumbnails.spec_standard)
+		cache_manager_render_remote(text, FALSE, TRUE);
+}
+
+static void gr_cache_render_standard_recurse(const gchar *text, GIOChannel *channel, gpointer data)
+{
+	if(options->thumbnails.spec_standard)
+		cache_manager_render_remote(text, TRUE, TRUE);
+}
+
 static void gr_slideshow_toggle(const gchar *text, GIOChannel *channel, gpointer data)
 {
 	layout_image_slideshow_toggle(NULL);
@@ -437,15 +492,47 @@ static void gr_slideshow_stop(const gchar *text, GIOChannel *channel, gpointer d
 
 static void gr_slideshow_delay(const gchar *text, GIOChannel *channel, gpointer data)
 {
-	gdouble n;
+	gdouble t1, t2, t3, n;
+	gint res;
 
-	n = g_ascii_strtod(text, NULL);
-	if (n < SLIDESHOW_MIN_SECONDS || n > SLIDESHOW_MAX_SECONDS)
+	res = sscanf(text, "%lf:%lf:%lf", &t1, &t2, &t3);
+	if (res == 3)
 		{
-		printf_term("Remote slideshow delay out of range (%.1f to %.1f)\n",
-			    SLIDESHOW_MIN_SECONDS, SLIDESHOW_MAX_SECONDS);
-		return;
+		n = (t1 * 3600) + (t2 * 60) + t3;
+		if (n < SLIDESHOW_MIN_SECONDS || n > SLIDESHOW_MAX_SECONDS ||
+				t1 >= 24 || t2 >= 60 || t3 >= 60)
+			{
+			printf_term(TRUE, "Remote slideshow delay out of range (%.1f to %.1f)\n",
+								SLIDESHOW_MIN_SECONDS, SLIDESHOW_MAX_SECONDS);
+			return;
+			}
 		}
+	else if (res == 2)
+		{
+		n = t1 * 60 + t2;
+		if (n < SLIDESHOW_MIN_SECONDS || n > SLIDESHOW_MAX_SECONDS ||
+				t1 >= 60 || t2 >= 60)
+			{
+			printf_term(TRUE, "Remote slideshow delay out of range (%.1f to %.1f)\n",
+								SLIDESHOW_MIN_SECONDS, SLIDESHOW_MAX_SECONDS);
+			return;
+			}
+		}
+	else if (res == 1)
+		{
+		n = t1;
+		if (n < SLIDESHOW_MIN_SECONDS || n > SLIDESHOW_MAX_SECONDS)
+			{
+			printf_term(TRUE, "Remote slideshow delay out of range (%.1f to %.1f)\n",
+								SLIDESHOW_MIN_SECONDS, SLIDESHOW_MAX_SECONDS);
+			return;
+			}
+		}
+	else
+		{
+		n = 0;
+		}
+
 	options->slideshow.delay = (gint)(n * 10.0 + 0.01);
 }
 
@@ -486,7 +573,7 @@ static void gr_quit(const gchar *text, GIOChannel *channel, gpointer data)
 	g_idle_add(gr_quit_idle_cb, NULL);
 }
 
-static void gr_file_load(const gchar *text, GIOChannel *channel, gpointer data)
+static void gr_file_load_no_raise(const gchar *text, GIOChannel *channel, gpointer data)
 {
 	gchar *filename = expand_tilde(text);
 
@@ -508,11 +595,30 @@ static void gr_file_load(const gchar *text, GIOChannel *channel, gpointer data)
 	else
 		{
 		log_printf("remote sent filename that does not exist:\"%s\"\n", filename);
+		layout_set_path(NULL, homedir());
 		}
 
 	g_free(filename);
 }
 
+static void gr_file_load(const gchar *text, GIOChannel *channel, gpointer data)
+{
+	gr_file_load_no_raise(text, channel, data);
+
+	gr_raise(text, channel, data);
+}
+
+static void gr_file_tell(const gchar *text, GIOChannel *channel, gpointer data)
+{
+	LayoutWindow *lw = NULL; /* NULL to force layout_valid() to do some magic */
+	if (!layout_valid(&lw)) return;
+	if (image_get_path(lw->image))
+		{
+		g_io_channel_write_chars(channel, image_get_path(lw->image), -1, NULL, NULL);
+		g_io_channel_write_chars(channel, "\n", -1, NULL, NULL);
+		}
+}
+
 static void gr_config_load(const gchar *text, GIOChannel *channel, gpointer data)
 {
 	gchar *filename = expand_tilde(text);
@@ -524,6 +630,7 @@ static void gr_config_load(const gchar *text, GIOChannel *channel, gpointer data
 	else
 		{
 		log_printf("remote sent filename that does not exist:\"%s\"\n", filename);
+		layout_set_path(NULL, homedir());
 		}
 
 	g_free(filename);
@@ -533,7 +640,7 @@ static void gr_get_sidecars(const gchar *text, GIOChannel *channel, gpointer dat
 {
 	gchar *filename = expand_tilde(text);
 	FileData *fd = file_data_new_group(filename);
-	
+
 	GList *work;
 	if (fd->parent) fd = fd->parent;
 
@@ -556,7 +663,7 @@ static void gr_get_destination(const gchar *text, GIOChannel *channel, gpointer
 {
 	gchar *filename = expand_tilde(text);
 	FileData *fd = file_data_new_group(filename);
-	
+
 	if (fd->change && fd->change->dest)
 		{
 		g_io_channel_write_chars(channel, fd->change->dest, -1, NULL, NULL);
@@ -624,6 +731,39 @@ static void gr_raise(const gchar *text, GIOChannel *channel, gpointer data)
 		}
 }
 
+#ifdef HAVE_LUA
+static void gr_lua(const gchar *text, GIOChannel *channel, gpointer data)
+{
+	gchar *result = NULL;
+	gchar **lua_command;
+
+	lua_command = g_strsplit(text, ",", 2);
+
+	if (lua_command[0] && lua_command[1])
+		{
+		FileData *fd = file_data_new_group(lua_command[0]);
+		result = g_strdup(lua_callvalue(fd, lua_command[1], NULL));
+		if (result)
+			{
+			g_io_channel_write_chars(channel, result, -1, NULL, NULL);
+			}
+		else
+			{
+			g_io_channel_write_chars(channel, N_("lua error: no data"), -1, NULL, NULL);
+			}
+		}
+	else
+		{
+		g_io_channel_write_chars(channel, N_("lua error: no data"), -1, NULL, NULL);
+		}
+
+	g_io_channel_write_chars(channel, "\n", -1, NULL, NULL);
+
+	g_strfreev(lua_command);
+	g_free(result);
+}
+#endif
+
 typedef struct _RemoteCommandEntry RemoteCommandEntry;
 struct _RemoteCommandEntry {
 	gchar *opt_s;
@@ -631,35 +771,48 @@ struct _RemoteCommandEntry {
 	void (*func)(const gchar *text, GIOChannel *channel, gpointer data);
 	gboolean needs_extra;
 	gboolean prefer_command_line;
+	gchar *parameter;
 	gchar *description;
 };
 
 static RemoteCommandEntry remote_commands[] = {
-	/* short, long                  callback,               extra, prefer,description */
-	{ "-n", "--next",               gr_image_next,          FALSE, FALSE, N_("next image") },
-	{ "-b", "--back",               gr_image_prev,          FALSE, FALSE, N_("previous image") },
-	{ NULL, "--first",              gr_image_first,         FALSE, FALSE, N_("first image") },
-	{ NULL, "--last",               gr_image_last,          FALSE, FALSE, N_("last image") },
-	{ "-f", "--fullscreen",         gr_fullscreen_toggle,   FALSE, TRUE,  N_("toggle full screen") },
-	{ "-fs","--fullscreen-start",   gr_fullscreen_start,    FALSE, FALSE, N_("start full screen") },
-	{ "-fS","--fullscreen-stop",    gr_fullscreen_stop,     FALSE, FALSE, N_("stop full screen") },
-	{ "-s", "--slideshow",          gr_slideshow_toggle,	FALSE, TRUE,  N_("toggle slide show") },
-	{ "-ss","--slideshow-start",    gr_slideshow_start,     FALSE, FALSE, N_("start slide show") },
-	{ "-sS","--slideshow-stop",     gr_slideshow_stop,      FALSE, FALSE, N_("stop slide show") },
-	{ NULL, "--slideshow-recurse:", gr_slideshow_start_rec, TRUE,  FALSE, N_("start recursive slide show") },
-	{ "-d", "--delay=",             gr_slideshow_delay,     TRUE,  FALSE, N_("set slide show delay in seconds") },
-	{ "+t", "--tools-show",         gr_tools_show,          FALSE, TRUE,  N_("show tools") },
-	{ "-t", "--tools-hide",	        gr_tools_hide,          FALSE, TRUE,  N_("hide tools") },
-	{ "-q", "--quit",               gr_quit,                FALSE, FALSE, N_("quit") },
-	{ NULL, "--config-load:",       gr_config_load,         TRUE,  FALSE, N_("load config file") },
-	{ NULL, "--get-sidecars:",      gr_get_sidecars,        TRUE,  FALSE, N_("get list of sidecars of the given file") },
-	{ NULL, "--get-destination:",  	gr_get_destination,     TRUE,  FALSE, N_("get destination path for the given file") },
-	{ NULL, "file:",                gr_file_load,           TRUE,  FALSE, N_("open file") },
-	{ NULL, "view:",		gr_file_view,		TRUE,  FALSE, N_("open file in new window") },
-	{ NULL, "--list-clear",         gr_list_clear,          FALSE, FALSE, NULL },
-	{ NULL, "--list-add:",          gr_list_add,            TRUE,  FALSE, NULL },
-	{ NULL, "raise",		gr_raise,		FALSE, FALSE, NULL },
-	{ NULL, NULL, NULL, FALSE, FALSE, NULL }
+	/* short, long                  callback,               extra, prefer, parameter, description */
+	{ "-n", "--next",               gr_image_next,          FALSE, FALSE, NULL, N_("next image") },
+	{ "-b", "--back",               gr_image_prev,          FALSE, FALSE, NULL, N_("previous image") },
+	{ NULL, "--first",              gr_image_first,         FALSE, FALSE, NULL, N_("first image") },
+	{ NULL, "--last",               gr_image_last,          FALSE, FALSE, NULL, N_("last image") },
+	{ "-f", "--fullscreen",         gr_fullscreen_toggle,   FALSE, TRUE,  NULL, N_("toggle full screen") },
+	{ "-fs","--fullscreen-start",   gr_fullscreen_start,    FALSE, FALSE, NULL, N_("start full screen") },
+	{ "-fS","--fullscreen-stop",    gr_fullscreen_stop,     FALSE, FALSE, NULL, N_("stop full screen") },
+	{ "-s", "--slideshow",          gr_slideshow_toggle,    FALSE, TRUE,  NULL, N_("toggle slide show") },
+	{ "-ss","--slideshow-start",    gr_slideshow_start,     FALSE, FALSE, NULL, N_("start slide show") },
+	{ "-sS","--slideshow-stop",     gr_slideshow_stop,      FALSE, FALSE, NULL, N_("stop slide show") },
+	{ NULL, "--slideshow-recurse:", gr_slideshow_start_rec, TRUE,  FALSE, N_("<FOLDER>"), N_("start recursive slide show in FOLDER") },
+	{ "-d", "--delay=",             gr_slideshow_delay,     TRUE,  FALSE, N_("<[H:][M:][N][.M]>"), N_("set slide show delay to Hrs Mins N.M seconds") },
+	{ "+t", "--tools-show",         gr_tools_show,          FALSE, TRUE,  NULL, N_("show tools") },
+	{ "-t", "--tools-hide",	        gr_tools_hide,          FALSE, TRUE,  NULL, N_("hide tools") },
+	{ "-q", "--quit",               gr_quit,                FALSE, FALSE, NULL, N_("quit") },
+	{ NULL, "--config-load:",       gr_config_load,         TRUE,  FALSE, N_("<FILE>"), N_("load configuration from FILE") },
+	{ NULL, "--get-sidecars:",      gr_get_sidecars,        TRUE,  FALSE, N_("<FILE>"), N_("get list of sidecars of FILE") },
+	{ NULL, "--get-destination:",  	gr_get_destination,     TRUE,  FALSE, N_("<FILE>"), N_("get destination path of FILE") },
+	{ NULL, "file:",                gr_file_load,           TRUE,  FALSE, N_("<FILE>"), N_("open FILE, bring Geeqie window to the top") },
+	{ NULL, "File:",                gr_file_load_no_raise,  TRUE,  FALSE, N_("<FILE>"), N_("open FILE, do not bring Geeqie window to the top") },
+	{ NULL, "--tell",               gr_file_tell,           FALSE, FALSE, NULL, N_("print filename of current image") },
+	{ NULL, "view:",                gr_file_view,           TRUE,  FALSE, N_("<FILE>"), N_("open FILE in new window") },
+	{ NULL, "--list-clear",         gr_list_clear,          FALSE, FALSE, NULL, N_("clear command line collection list") },
+	{ NULL, "--list-add:",          gr_list_add,            TRUE,  FALSE, N_("<FILE>"), N_("add FILE to command line collection list") },
+	{ NULL, "raise",                gr_raise,               FALSE, FALSE, NULL, N_("bring the Geeqie window to the top") },
+	{ "-ct:", "--cache-thumbs:",    gr_cache_thumb,         TRUE, FALSE, N_("clear|clean"), N_("clear or clean thumbnail cache") },
+	{ "-cs:", "--cache-shared:",    gr_cache_shared,        TRUE, FALSE, N_("clear|clean"), N_("clear or clean shared thumbnail cache") },
+	{ "-cm","--cache-metadata",      gr_cache_metadata,               FALSE, FALSE, NULL, N_("    clean the metadata cache") },
+	{ "-cr:", "--cache-render:",    gr_cache_render,        TRUE, FALSE, N_("<folder>  "), N_(" render thumbnails") },
+	{ "-crr:", "--cache-render-recurse:", gr_cache_render_recurse, TRUE, FALSE, N_("<folder> "), N_("render thumbnails recursively") },
+	{ "-crs:", "--cache-render-shared:", gr_cache_render_standard, TRUE, FALSE, N_("<folder> "), N_(" render thumbnails (see Help)") },
+	{ "-crsr:", "--cache-render-shared-recurse:", gr_cache_render_standard_recurse, TRUE, FALSE, N_("<folder>"), N_(" render thumbnails recursively (see Help)") },
+#ifdef HAVE_LUA
+	{ NULL, "--lua:",               gr_lua,                 TRUE, FALSE, N_("<FILE>,<lua script>"), N_("run lua script on FILE") },
+#endif
+	{ NULL, NULL, NULL, FALSE, FALSE, NULL, NULL }
 };
 
 static RemoteCommandEntry *remote_command_find(const gchar *text, const gchar **offset)
@@ -720,23 +873,29 @@ static void remote_cb(RemoteConnection *rc, const gchar *text, GIOChannel *chann
 void remote_help(void)
 {
 	gint i;
+	gchar *s_opt_param;
+	gchar *l_opt_param;
 
-	print_term(_("Remote command list:\n"));
+	print_term(FALSE, _("Remote command list:\n"));
 
 	i = 0;
 	while (remote_commands[i].func != NULL)
 		{
 		if (remote_commands[i].description)
 			{
-			printf_term("  %-3s%s %-20s %s\n",
-				    (remote_commands[i].opt_s) ? remote_commands[i].opt_s : "",
+			s_opt_param = g_strconcat(remote_commands[i].opt_s, remote_commands[i].parameter, NULL);
+			l_opt_param = g_strconcat(remote_commands[i].opt_l, remote_commands[i].parameter, NULL);
+			printf_term(FALSE, "  %-11s%-1s %-30s%-s\n",
+				    (remote_commands[i].opt_s) ? s_opt_param : "",
 				    (remote_commands[i].opt_s && remote_commands[i].opt_l) ? "," : " ",
-				    (remote_commands[i].opt_l) ? remote_commands[i].opt_l : "",
+				    (remote_commands[i].opt_l) ? l_opt_param : "",
 				    _(remote_commands[i].description));
+			g_free(s_opt_param);
+			g_free(l_opt_param);
 			}
 		i++;
 		}
-	printf_term(N_("\n  All other command line parameters are used as plain files if they exists.\n"));
+	printf_term(FALSE, N_("\n  All other command line parameters are used as plain files if they exists.\n"));
 }
 
 GList *remote_build_list(GList *list, gint argc, gchar *argv[], GList **errors)
@@ -786,7 +945,7 @@ void remote_control(const gchar *arg_exec, GList *remote_list, const gchar *path
 		gint retry_count = 12;
 		gboolean blank = FALSE;
 
-		printf_term(_("Remote %s not running, starting..."), GQ_APPNAME);
+		printf_term(FALSE, _("Remote %s not running, starting..."), GQ_APPNAME);
 
 		command = g_string_new(arg_exec);
 
@@ -826,11 +985,11 @@ void remote_control(const gchar *arg_exec, GList *remote_list, const gchar *path
 			{
 			usleep((retry_count > 10) ? 500000 : 1000000);
 			rc = remote_client_open(buf);
-			if (!rc) print_term(".");
+			if (!rc) print_term(FALSE, ".");
 			retry_count--;
 			}
 
-		print_term("\n");
+		print_term(FALSE, "\n");
 
 		started = TRUE;
 		}
@@ -922,7 +1081,7 @@ void remote_control(const gchar *arg_exec, GList *remote_list, const gchar *path
 		}
 	else
 		{
-		print_term(_("Remote not available\n"));
+		print_term(TRUE, _("Remote not available\n"));
 		}
 
 	_exit(0);
@@ -932,7 +1091,7 @@ RemoteConnection *remote_server_init(gchar *path, CollectionData *command_collec
 {
 	RemoteConnection *remote_connection = remote_server_open(path);
 	RemoteData *remote_data = g_new(RemoteData, 1);
-	
+
 	remote_data->command_collection = command_collection;
 
 	remote_server_subscribe(remote_connection, remote_cb, remote_data);
