@@ -1,19 +1,28 @@
 /*
- * Geeqie
- * (C) 2006 John Ellis
- * Copyright (C) 2008 - 2010 The Geeqie Team
+ * Copyright (C) 2006 John Ellis
+ * Copyright (C) 2008 - 2016 The Geeqie Team
  *
  * Author: John Ellis
  *
- * This software is released under the GNU General Public License (GNU GPL).
- * Please read the included file COPYING for more information.
- * This software comes with no warranty of any kind, use at your own risk!
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-
 #include "main.h"
 #include "thumb_standard.h"
 
+#include "cache.h"
 #include "image-load.h"
 #include "md5-util.h"
 #include "pixbuf_util.h"
@@ -66,7 +75,6 @@
 
 static void thumb_loader_std_error_cb(ImageLoader *il, gpointer data);
 static gint thumb_loader_std_setup(ThumbLoaderStd *tl, FileData *fd);
-static gint thumb_loader_std_setup_path(ThumbLoaderStd *tl, const gchar *path);
 
 
 ThumbLoaderStd *thumb_loader_std_new(gint width, gint height)
@@ -149,7 +157,8 @@ static gchar *thumb_std_cache_path(const gchar *path, const gchar *uri, gboolean
 		}
 	else
 		{
-		result = g_build_filename(homedir(), THUMB_FOLDER_GLOBAL, cache_subfolder, name, NULL);
+		result = g_build_filename(get_thumbnails_standard_cache_dir(),
+													cache_subfolder, name, NULL);
 		}
 
 	g_free(name);
@@ -390,7 +399,7 @@ static GdkPixbuf *thumb_loader_std_finish(ThumbLoaderStd *tl, GdkPixbuf *pixbuf,
 			{
 			tl->fd->exif_orientation = metadata_read_int(tl->fd, ORIENTATION_KEY, EXIF_ORIENTATION_TOP_LEFT);
 			}
-		
+
 		if (tl->fd->exif_orientation != EXIF_ORIENTATION_TOP_LEFT)
 			{
 			rotated = pixbuf_apply_orientation(pixbuf, tl->fd->exif_orientation);
@@ -435,7 +444,7 @@ static GdkPixbuf *thumb_loader_std_finish(ThumbLoaderStd *tl, GdkPixbuf *pixbuf,
 
 				/* do not save the thumbnail if the source file has changed meanwhile -
 				   the thumbnail is most probably broken */
-				if (stat_utf8(tl->fd->path, &st) && 
+				if (stat_utf8(tl->fd->path, &st) &&
 				    tl->source_mtime == st.st_mtime &&
 				    tl->source_size == st.st_size)
 					{
@@ -514,10 +523,16 @@ static gboolean thumb_loader_std_next_source(ThumbLoaderStd *tl, gboolean remove
 		if (!tl->thumb_path_local)
 			{
 			tl->thumb_path = thumb_loader_std_cache_path(tl, TRUE, NULL, FALSE);
-			if (isfile(tl->thumb_path) && thumb_loader_std_setup_path(tl, tl->thumb_path))
+			if (isfile(tl->thumb_path))
 				{
-				tl->thumb_path_local = TRUE;
-				return TRUE;
+				FileData *fd = file_data_new_no_grouping(tl->thumb_path);
+				if (thumb_loader_std_setup(tl, fd))
+					{
+					file_data_unref(fd);
+					tl->thumb_path_local = TRUE;
+					return TRUE;
+					}
+				file_data_unref(fd);
 				}
 
 			g_free(tl->thumb_path);
@@ -576,14 +591,14 @@ static void thumb_loader_std_error_cb(ImageLoader *il, gpointer data)
 		thumb_loader_std_done_cb(il, data);
 		return;
 		}
-	
+
 	DEBUG_1("thumb image error: %s", tl->fd->path);
 	DEBUG_1("             from: %s", image_loader_get_fd(tl->il)->path);
 
 	if (thumb_loader_std_next_source(tl, TRUE)) return;
 
 	thumb_loader_std_set_fallback(tl);
-	
+
 	if (tl->func_error) tl->func_error(tl, tl->data);
 }
 
@@ -629,14 +644,6 @@ static gboolean thumb_loader_std_setup(ThumbLoaderStd *tl, FileData *fd)
 	return FALSE;
 }
 
-static gboolean thumb_loader_std_setup_path(ThumbLoaderStd *tl, const gchar *path)
-{
-	FileData *fd = file_data_new_simple(path);
-	gboolean ret = thumb_loader_std_setup(tl, fd);
-	file_data_unref(fd);
-	return ret;
-}
-
 /*
  * Note: Currently local_cache only specifies where to save a _new_ thumb, if
  *       a valid existing thumb is found anywhere the local thumb will not be created.
@@ -661,7 +668,7 @@ gboolean thumb_loader_std_start(ThumbLoaderStd *tl, FileData *fd)
 
 
 	tl->fd = file_data_ref(fd);
-	if (!stat_utf8(fd->path, &st))
+	if (!stat_utf8(fd->path, &st) || (tl->fd->format_class != FORMAT_CLASS_IMAGE && tl->fd->format_class != FORMAT_CLASS_RAWIMAGE && tl->fd->format_class != FORMAT_CLASS_VIDEO))
 		{
 		thumb_loader_std_set_fallback(tl);
 		return FALSE;
@@ -670,7 +677,11 @@ gboolean thumb_loader_std_start(ThumbLoaderStd *tl, FileData *fd)
 	tl->source_size = st.st_size;
 	tl->source_mode = st.st_mode;
 
-	if (!thumb_cache) thumb_cache = g_build_filename(homedir(), THUMB_FOLDER_GLOBAL, NULL);
+	if (!thumb_cache)
+		{
+		thumb_cache = g_strdup(get_thumbnails_standard_cache_dir());
+		}
+
 	if (strncmp(tl->fd->path, thumb_cache, strlen(thumb_cache)) != 0)
 		{
 		gchar *pathl;
@@ -689,7 +700,16 @@ gboolean thumb_loader_std_start(ThumbLoaderStd *tl, FileData *fd)
 		tl->thumb_path_local = FALSE;
 
 		found = isfile(tl->thumb_path);
-		if (found && thumb_loader_std_setup_path(tl, tl->thumb_path)) return TRUE;
+		if (found)
+			{
+			FileData *fd = file_data_new_no_grouping(tl->thumb_path);
+			if (thumb_loader_std_setup(tl, fd))
+				{
+				file_data_unref(fd);
+				return TRUE;
+				}
+			file_data_unref(fd);
+			}
 
 		if (thumb_loader_std_fail_check(tl) ||
 		    !thumb_loader_std_next_source(tl, found))
@@ -786,9 +806,9 @@ static void thumb_loader_std_thumb_file_validate_done_cb(ThumbLoaderStd *tl, gpo
 	GdkPixbuf *pixbuf;
 	gboolean valid = FALSE;
 
-	/* get the original thumbnail pixbuf (unrotated, with original options) 
+	/* get the original thumbnail pixbuf (unrotated, with original options)
 	   this is called from image_loader done callback, so tv->tl->il must exist*/
-	pixbuf = image_loader_get_pixbuf(tv->tl->il); 
+	pixbuf = image_loader_get_pixbuf(tv->tl->il);
 	if (pixbuf)
 		{
 		const gchar *uri;
@@ -876,7 +896,8 @@ ThumbLoaderStd *thumb_loader_std_thumb_file_validate(const gchar *thumb_path, gi
 	tv->func_valid = func_valid;
 	tv->data = data;
 
-	if (!thumb_loader_std_setup_path(tv->tl, thumb_path))
+	FileData *fd = file_data_new_no_grouping(thumb_path);
+	if (!thumb_loader_std_setup(tv->tl, fd))
 		{
 		tv->idle_id = g_idle_add(thumb_loader_std_thumb_file_validate_idle_cb, tv);
 		}
@@ -885,6 +906,7 @@ ThumbLoaderStd *thumb_loader_std_thumb_file_validate(const gchar *thumb_path, gi
 		tv->idle_id = 0;
 		}
 
+	file_data_unref(fd);
 	return tv->tl;
 }
 
@@ -951,9 +973,9 @@ static void thumb_std_maint_move_validate_cb(const gchar *path, gboolean valid,
 	TMaintMove *tm = data;
 	GdkPixbuf *pixbuf;
 
-	/* get the original thumbnail pixbuf (unrotated, with original options) 
+	/* get the original thumbnail pixbuf (unrotated, with original options)
 	   this is called from image_loader done callback, so tm->tl->il must exist*/
-	pixbuf = image_loader_get_pixbuf(tm->tl->il); 
+	pixbuf = image_loader_get_pixbuf(tm->tl->il);
 	if (pixbuf)
 		{
 		const gchar *uri;
@@ -974,9 +996,8 @@ static void thumb_std_maint_move_validate_cb(const gchar *path, gboolean valid,
 			tm->tl->cache_enable = TRUE;
 			tm->tl->cache_hit = FALSE;
 			tm->tl->cache_local = FALSE;
-
 			file_data_unref(tm->tl->fd);
-			tm->tl->fd = file_data_new_simple(tm->dest);
+			tm->tl->fd = file_data_new_group(tm->dest);
 			tm->tl->source_mtime = strtol(mtime_str, NULL, 10);
 
 			pathl = path_from_utf8(tm->tl->fd->path);
