@@ -1,13 +1,22 @@
 /*
- * (SLIK) SimpLIstic sKin functions
- * (C) 2006 John Ellis
- * Copyright (C) 2008 - 2012 The Geeqie Team
+ * Copyright (C) 2006 John Ellis
+ * Copyright (C) 2008 - 2016 The Geeqie Team
  *
  * Author: John Ellis
  *
- * This software is released under the GNU General Public License (GNU GPL).
- * Please read the included file COPYING for more information.
- * This software comes with no warranty of any kind, use at your own risk!
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -32,6 +41,9 @@
 #include "ui_utildlg.h"	/* for locale warning dialog */
 #include "md5-util.h"
 
+#include "filefilter.h"
+#include "secure_save.h"
+
 /*
  *-----------------------------------------------------------------------------
  * generic file information and manipulation routines (public)
@@ -40,34 +52,24 @@
 
 
 
-void print_term(const gchar *text_utf8)
+void print_term(gboolean err, const gchar *text_utf8)
 {
 	gchar *text_l;
 
 	text_l = g_locale_from_utf8(text_utf8, -1, NULL, NULL, NULL);
-	fputs((text_l) ? text_l : text_utf8, stderr);
+	if (err)
+		{
+		fputs((text_l) ? text_l : text_utf8, stderr);
+		}
+	else
+		{
+		fputs((text_l) ? text_l : text_utf8, stdout);
+		}
+	if(command_line && command_line->ssi)
+		secure_fputs(command_line->ssi, (text_l) ? text_l : text_utf8);
 	g_free(text_l);
 }
 
-static void encoding_dialog(const gchar *path);
-
-static gboolean encoding_dialog_idle(gpointer data)
-{
-	gchar *path = data;
-
-	encoding_dialog(path);
-	g_free(path);
-
-	return FALSE;
-}
-
-static gint encoding_dialog_delay(gpointer data)
-{
-	g_idle_add(encoding_dialog_idle, data);
-
-	return 0;
-}
-
 static void encoding_dialog(const gchar *path)
 {
 	static gboolean warned_user = FALSE;
@@ -76,14 +78,6 @@ static void encoding_dialog(const gchar *path)
 	const gchar *lc;
 	const gchar *bf;
 
-	/* check that gtk is initialized (loop is level > 0) */
-	if (gtk_main_level() == 0)
-		{
-		/* gtk not initialized */
-		gtk_init_add(encoding_dialog_delay, g_strdup(path));
-		return;
-		}
-
 	if (warned_user) return;
 	warned_user = TRUE;
 
@@ -107,7 +101,7 @@ static void encoding_dialog(const gchar *path)
 		name = g_convert(path, -1, "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
 		string = g_string_append(string, _("\nPreferred encoding appears to be UTF-8, however the file:\n"));
 		g_string_append_printf(string, "\"%s\"\n", (name) ? name : _("[name not displayable]"));
-		
+
 		if (g_utf8_validate(path, -1, NULL))
 			g_string_append_printf(string, _("\"%s\" is encoded in valid UTF-8."), (name) ? name : _("[name not displayable]"));
 		else
@@ -121,7 +115,7 @@ static void encoding_dialog(const gchar *path)
 	generic_dialog_add_button(gd, GTK_STOCK_CLOSE, NULL, NULL, TRUE);
 
 	generic_dialog_add_message(gd, GTK_STOCK_DIALOG_WARNING,
-				   _("Filename encoding locale mismatch"), string->str);
+				   _("Filename encoding locale mismatch"), string->str, TRUE);
 
 	gtk_widget_show(gd->dialog);
 
@@ -196,10 +190,12 @@ const gchar *homedir(void)
 
 	if (!home)
 		home = path_to_utf8(getenv("HOME"));
-	
+
 	if (!home)
 		home = path_to_utf8(g_get_home_dir());
 
+	DEBUG_1("Home directory: %s", home);
+
 	return home;
 }
 
@@ -211,7 +207,9 @@ static gchar *xdg_dir_get(const gchar *key, const gchar *fallback)
 		{
     		return g_build_filename(homedir(), fallback, NULL);
     		}
-	
+
+	DEBUG_1("Got xdg %s: %s", key, dir);
+
 	return path_to_utf8(dir);
 }
 
@@ -220,7 +218,7 @@ const gchar *xdg_data_home_get(void)
 	static const gchar *xdg_data_home = NULL;
 
 	if (xdg_data_home) return xdg_data_home;
-    	
+
 	xdg_data_home = xdg_dir_get("XDG_DATA_HOME", ".local/share");
 
 	return xdg_data_home;
@@ -231,7 +229,7 @@ const gchar *xdg_config_home_get(void)
 	static const gchar *xdg_config_home = NULL;
 
 	if (xdg_config_home) return xdg_config_home;
-    	
+
 	xdg_config_home = xdg_dir_get("XDG_CONFIG_HOME", ".config");
 
 	return xdg_config_home;
@@ -242,7 +240,7 @@ const gchar *xdg_cache_home_get(void)
 	static const gchar *xdg_cache_home = NULL;
 
 	if (xdg_cache_home) return xdg_cache_home;
-    	
+
 	xdg_cache_home = xdg_dir_get("XDG_CACHE_HOME", ".cache");
 
 	return xdg_cache_home;
@@ -251,7 +249,7 @@ const gchar *xdg_cache_home_get(void)
 const gchar *get_rc_dir(void)
 {
 	static gchar *rc_dir = NULL;
-	
+
 	if (rc_dir) return rc_dir;
 
 	if (USE_XDG)
@@ -289,7 +287,7 @@ const gchar *get_trash_dir(void)
 	static gchar *trash_dir = NULL;
 
 	if (trash_dir) return trash_dir;
-	
+
 	if (USE_XDG)
 		{
 		trash_dir = g_build_filename(xdg_data_home_get(), GQ_APPNAME_LC, GQ_TRASH_DIR, NULL);
@@ -496,7 +494,12 @@ gboolean copy_file_attributes(const gchar *s, const gchar *t, gint perms, gint m
 			/* Ignores chown errors, while still doing chown
 			   (so root still can copy files preserving ownership) */
 			ret = TRUE;
-			if (chmod(tl, st.st_mode) < 0) ret = FALSE;
+			if (chmod(tl, st.st_mode) < 0) {
+                            struct stat st2;
+                            if (stat(tl, &st2) != 0 || st2.st_mode != st.st_mode) {
+                                ret = FALSE;
+                            }
+                        }
 			}
 
 		tb.actime = st.st_atime;
@@ -543,17 +546,78 @@ gboolean copy_file(const gchar *s, const gchar *t)
 		goto end;
 		}
 
+	/* Do not dereference absolute symlinks, but copy them "as is".
+	* For a relative symlink, we don't know how to properly change it when
+	* copied/moved to another dir to keep pointing it to same target as
+	* a relative symlink, so we turn it into absolute symlink using
+	* realpath() instead. */
+	struct stat st;
+	if (lstat_utf8(sl, &st) && S_ISLNK(st.st_mode))
+		{
+		gchar *link_target;
+		ssize_t i;
+
+		link_target = g_malloc(st.st_size + 1);
+		i = readlink(sl, link_target, st.st_size);
+		if (i<0)
+			{
+			g_free(link_target);
+			goto orig_copy;  // try a "normal" copy
+			}
+		link_target[st.st_size] = '\0';
+
+		if (link_target[0] != G_DIR_SEPARATOR) // if it is a relative symlink
+			{
+			gchar *absolute;
+
+			absolute = g_malloc(PATH_MAX + 1);
+			char *lastslash = strrchr(sl, G_DIR_SEPARATOR);
+			int len = lastslash - sl + 1;
+
+			strncpy(absolute, sl, len);
+			strcpy(absolute + len, link_target);
+			strcpy(link_target, absolute);
+
+			char *realPath;
+			realPath = realpath(link_target, absolute);
+
+			if (realPath != NULL) // successfully resolved into an absolute path
+				{
+				g_free(link_target);
+				link_target = absolute;
+				}
+			else                 // could not get absolute path, got some error instead
+				{
+				g_free(link_target);
+				g_free(absolute);
+				goto orig_copy;  // so try a "normal" copy
+				}
+			}
+
+		if (stat_utf8(tl, &st)) unlink(tl); // first try to remove directory entry in destination directory if such entry exists
+
+		gint success = (symlink(link_target, tl) == 0);
+		g_free(link_target);
+
+		if (success)
+			{
+			ret = TRUE;
+			goto end;
+			}
+		} // if symlink did not succeed, continue on to try a copy procedure
+	orig_copy:
+
 	fi = fopen(sl, "rb");
 	if (!fi) goto end;
-	
+
 	/* First we write to a temporary file, then we rename it on success,
 	   and attributes from original file are copied */
 	randname = g_strconcat(tl, ".tmp_XXXXXX", NULL);
 	if (!randname) goto end;
-	
+
 	fd = g_mkstemp(randname);
 	if (fd == -1) goto end;
-	
+
 	fo = fdopen(fd, "wb");
 	if (!fo) {
 		close(fd);
@@ -574,7 +638,7 @@ gboolean copy_file(const gchar *s, const gchar *t)
 
 	if (rename(randname, tl) < 0) {
 		unlink(randname);
-		goto end; 	
+		goto end;
 	}
 
 	ret = copy_file_attributes(s, t, TRUE, TRUE);
@@ -715,7 +779,7 @@ gchar *unique_filename_simple(const gchar *path)
 	name = filename_from_path(path);
 	if (!name) return NULL;
 
-	ext = extension_from_path(name);
+	ext = registered_extension_from_path(name);
 
 	if (!ext)
 		{
@@ -747,24 +811,16 @@ const gchar *filename_from_path(const gchar *path)
 
 gchar *remove_level_from_path(const gchar *path)
 {
-	gint p = 0, n = -1;
+	const gchar *base;
 
 	if (!path) return NULL;
 
-	while (path[p])
-		{
-		if (path[p] == G_DIR_SEPARATOR) n = p;
-		p++;
-		}
-	if (n <= 0) n++;
+	base = strrchr(path, G_DIR_SEPARATOR);
+	/* Take account of a file being in the root ( / ) folder - ensure the returned value
+	 * is at least one character long */
+	if (base) return g_strndup(path, (strlen(path)-strlen(base)) == 0 ? 1 : (strlen(path)-strlen(base)));
 
-	return g_strndup(path, (gsize) n);
-}
-
-const gchar *extension_from_path(const gchar *path)
-{
-	if (!path) return NULL;
-	return strrchr(path, '.');
+	return NULL;
 }
 
 gboolean file_extension_match(const gchar *path, const gchar *ext)
@@ -784,18 +840,13 @@ gboolean file_extension_match(const gchar *path, const gchar *ext)
 
 gchar *remove_extension_from_path(const gchar *path)
 {
-	gint p = 0, n = -1;
+	const gchar *reg_ext;
 
 	if (!path) return NULL;
 
-	while (path[p])
-		{
-		if (path[p] == '.') n = p;
-		p++;
-		}
-	if (n < 0) n = p;
+	reg_ext = registered_extension_from_path(path);
 
-	return g_strndup(path, (gsize) n);
+	return g_strndup(path, strlen(path) - (reg_ext == NULL ? 0 : strlen(reg_ext)));
 }
 
 void parse_out_relatives(gchar *path)
@@ -828,7 +879,7 @@ void parse_out_relatives(gchar *path)
 				continue;
 				}
 			}
-	
+
 		if (s != t) path[t] = path[s];
 		t++;
 		s++;
@@ -892,7 +943,7 @@ gboolean recursive_mkdir_if_not_exists(const gchar *path, mode_t mode)
 					p[0] = '\0';
 					end = FALSE;
 					}
-				
+
 				if (!isdir(npath))
 					{
 					DEBUG_1("creating sub dir:%s", npath);
@@ -903,7 +954,7 @@ gboolean recursive_mkdir_if_not_exists(const gchar *path, mode_t mode)
 						return FALSE;
 						}
 					}
-				
+
 				if (!end) p[0] = G_DIR_SEPARATOR;
 				}
 			}
